@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,7 +22,9 @@
 #include <boost/signals2/signal.hpp>
 #include <pcrecpp.h>
 
+#include "wb_sql_editor_form.h"
 #include "query_side_palette.h"
+
 #include "wb_sql_editor_snippets.h"
 #include "wb_sql_editor_help.h"
 #include "grt/tree_model.h"
@@ -174,13 +176,9 @@ public:
   {
     base::Rect bounds = snippet_bounds(snippet);
 
-    int left = (int) bounds.left();
-    int dummy = left;
-    int top = (int) bounds.top();
-    int bottom = (int) bounds.bottom();
-    client_to_screen(left, top);
-    client_to_screen(dummy, bottom);
-    top = (top + bottom) / 2; // Don't change this to top + height / 2, on OS X coordinates are flipped.
+    std::pair<int, int> left_top = client_to_screen((int) bounds.left(), (int) bounds.top());
+    std::pair<int, int> bottom = client_to_screen(0, (int) bounds.bottom());
+    left_top.second = (left_top.second + bottom.second) / 2;
 
     std::string title, description;
     get_snippet_info(snippet, title, description);
@@ -189,27 +187,28 @@ public:
     _snippet_popover->set_read_only(false);
     _snippet_popover->set_text(description);
     _snippet_popover->set_read_only(true);
-    _snippet_popover->show(left, top, mforms::Left);
+    _snippet_popover->show(left_top.first, left_top.second, mforms::Left);
   }
 
   //------------------------------------------------------------------------------------------------
 
-  virtual void mouse_double_click(int button, int x, int y)
+  virtual bool mouse_double_click(mforms::MouseButton button, int x, int y)
   {
-    BaseSnippetList::mouse_double_click(button, x, y);
+    bool result = BaseSnippetList::mouse_double_click(button, x, y);
 
-    switch (button)
+    if (!result)
     {
-    case MouseButtonLeft:
+      if (button == MouseButtonLeft)
       {
         Snippet* snippet = snippet_from_point(x, y);
         if (snippet != NULL && snippet == _selected_snippet)
         {
           edit_snippet(snippet);
+          result = true;
         }
-        break;
       }
     }
+    return result;
   }
 
   //------------------------------------------------------------------------------------------------
@@ -238,10 +237,9 @@ QuerySidePalette::QuerySidePalette(const SqlEditorForm::Ref &owner)
   _help_timer = NULL;
   _grtm = owner->grt_manager();
   _no_help = true;
-  _enable_help = true;
+  _automatic_help = _grtm->get_app_option_int("DbSqlEditor:DisableAutomaticContextHelp", 1) == 0;
   _switching_help = false;
-
-  scoped_connect(signal_tab_changed(), boost::bind(&QuerySidePalette::tab_changed, this));
+  _help_task = GrtThreadedTask::create(_grtm);
 
   mforms::Box *help_page = manage(new Box(false));
   _help_toolbar = prepare_help_toolbar();
@@ -250,7 +248,7 @@ QuerySidePalette::QuerySidePalette(const SqlEditorForm::Ref &owner)
   // Separate box since we need a border around the content, but not the toolbar.
   Box* content_border = manage(new Box(false));
 
-  // No scoped_connect needed since _help_text is member variable.
+  // No scoped_connect needed since _help_text is a member variable.
   scoped_connect(_help_text->signal_link_click(), boost::bind(&QuerySidePalette::click_link, this, _1));
 
 #if _WIN32
@@ -262,7 +260,7 @@ QuerySidePalette::QuerySidePalette(const SqlEditorForm::Ref &owner)
 #endif
 
   _help_text->set_markup_text("");
-  _current_topic = -1;
+  _current_topic_index = -1;
 
   help_page->add(_help_toolbar, false, true);
   content_border->add(_help_text, true, true);
@@ -317,13 +315,13 @@ QuerySidePalette::~QuerySidePalette()
 {
   base::NotificationCenter::get()->remove_observer(this);
 
-  if (_help_timer != NULL)
-    _grtm->cancel_timer(_help_timer);
-  _back_item->release();
-  _forward_item->release();
-  _quick_jump_item->release();
+  cancel_timer();
+  if (_help_task->is_busy() && _help_task->task() != NULL)
+    _help_task->task()->cancel();
 }
 
+//--------------------------------------------------------------------------------------------------
+
 void QuerySidePalette::cancel_timer()
 {
   if (_help_timer != NULL)
@@ -334,128 +332,209 @@ void QuerySidePalette::cancel_timer()
 
 void QuerySidePalette::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info)
 {
-  if (_enable_help)
+    // Selection and caret changes.
+  if ((name == "GNTextSelectionChanged") && _automatic_help && (get_active_tab() == 0))
   {
-    if (name == "GNTextSelectionChanged") // Selection and caret changes.
-    {
-      mforms::Object *object = (mforms::Object*)sender;
-      mforms::CodeEditor *code_editor = dynamic_cast<mforms::CodeEditor*>(object);
-      if (code_editor == NULL)
-        return;
+    mforms::Object *object = (mforms::Object*)sender;
+    mforms::CodeEditor *code_editor = dynamic_cast<mforms::CodeEditor*>(object);
+    if (code_editor == NULL)
+      return;
 
-      Sql_editor *editor = static_cast<Sql_editor*>(code_editor->get_host());
-      // XXX This event is sent for any code editor in the app, not only the ones in this editor.. so we need a better check here
-      if (editor && editor->grtobj().is_valid())
+    Sql_editor *editor = static_cast<Sql_editor*>(code_editor->get_host());
+    if (editor != NULL && editor->grtobj().is_valid())
+    {
+      // See if this editor instance is actually from the IDE this palette sits in.
+      SqlEditorForm::Ref form = _owner.lock();
+      if (form && form->contains_editor(editor))
       {
         check_format_structures(editor);
-        if (_help_timer != NULL)
-          _grtm->cancel_timer(_help_timer);
-        _help_timer = _grtm->run_every(boost::bind(&QuerySidePalette::update_context_help, this, editor), 0.7);
+        cancel_timer();
+        _help_timer = _grtm->run_every(boost::bind(&QuerySidePalette::find_context_help, this, editor), 0.7);
       }
     }
+
+    return;
   }
 } 
 
 //--------------------------------------------------------------------------------------------------
 
-void QuerySidePalette::tab_changed()
+void QuerySidePalette::show_help_text_for_topic(const std::string &topic)
 {
-  _enable_help = get_active_tab() == 0; // Only show help if the help page is actually active.
+  std::string title, text, html_text;
+
+  if (topic.empty())
+  {
+    update_help_ui();
+    return;
+  }
+
+  std::string topic_upper = base::toupper(topic);
+  if (_topic_cache.find(topic_upper) != _topic_cache.end())
+  {
+    std::pair<std::string, std::string> entry = _topic_cache[topic_upper];
+    html_text = entry.second;
+    _help_text->set_markup_text(html_text);
+    _no_help = false;
+
+    return;
+  }
+
+  // Only run one query at a time. There's no sense in issuing new queries if one is still pending
+  // which might indicate a slow server or broken connection.
+  if (_help_task->is_busy() || (_help_task->task() != NULL && _help_task->task()->is_cancelled()))
+    return;
+
+  _last_topic = topic_upper;
+  _help_task->exec(false, boost::bind(&QuerySidePalette::get_help_text_threaded, this, _1));
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void QuerySidePalette::show_help_topic(std::string topic)
+/**
+ * Runs in the background to find the text for the last stored topic. Never called if there's no topic.
+ */
+grt::StringRef QuerySidePalette::get_help_text_threaded(grt::GRT *)
 {
+  SqlEditorForm::Ref form = _owner.lock();
+  if (!form)
+    return "";
+
   std::string title, text, html_text;
 
-  topic = base::toupper(topic);
-  if (!topic.empty())
+  if (DbSqlEditorContextHelp::get_help_text(form, _last_topic, title, text))
   {
-    if (_topic_cache.find(topic) != _topic_cache.end())
-    {
-      std::pair<std::string, std::string> entry = _topic_cache[topic];
-      html_text = entry.second;
-      _help_text->set_markup_text(html_text);
-      _no_help = false;
-    }
-  }
-
-  if (html_text.empty() && !topic.empty())
-  {
-    SqlEditorForm::Ref form = _owner.lock();
-    if (DbSqlEditorContextHelp::get_help_text(form, topic, title, text))
-    {
 #ifdef _WIN32
-      std::string additional_space = "<div style=\"font-size:4pt\"> </div>";
+    std::string additional_space = "<div style=\"font-size:4pt\"> </div>";
 #else
-      std::string additional_space;
+    std::string additional_space;
 #endif
-      html_text = std::string("<html><body style=\"font-family:") + DEFAULT_FONT_FAMILY + "; font-size: 8pt\">"
-        "<b style=\"font-size: 11pt\">Topic: <span style=\"color:#688b5e\">" + title +
-        "</span></b><br>" + additional_space + format_help_as_html(text) + "</body></html>";
-      _help_text->set_markup_text(html_text);
-      _no_help = false;
-      _topic_cache[topic] = std::make_pair(text, html_text);
-    }
+    html_text = std::string("<html><body style=\"font-family:") + DEFAULT_FONT_FAMILY + "; font-size: 8pt\">"
+      "<b style=\"font-size: 11pt\">Topic: <span style=\"color:#688b5e\">" + title +
+      "</span></b><br>" + additional_space + format_help_as_html(text) + "</body></html>";
+    _no_help = false;
+    _topic_cache[_last_topic] = std::make_pair(text, html_text);
   }
-
-  if (html_text.empty())
+  else
   {
     _no_help = true;
+    _last_topic = "";
+  }
+
+  if (!_help_task->task()->is_cancelled())
+    _help_task->execute_in_main_thread(boost::bind(&QuerySidePalette::update_help_ui, this), false, false);
+
+  return "";
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Runs on the main thread for updating UI elements. Called by the help background task.
+ */
+void QuerySidePalette::update_help_ui()
+{
+  if (_last_topic.empty())
+  {
     _help_text->set_markup_text(
       std::string("<hmtl><body style=\"font-family:") + DEFAULT_FONT_FAMILY + "; font-size: 8pt\">"
       "<div style=\"width: 100%\"><b style=\"font-size: 10pt; color:#B0B0B0\">"
       "No Context Help<b><br><br><hr></div></body></html>");
   }
+  else
+  {
+    _help_text->set_markup_text(_topic_cache[_last_topic].second);
 
-  _switching_help = true;
-  _quick_jump_item->set_text(topic);
-  _switching_help = false;
+    _switching_help = true;
+    _quick_jump_item->set_text(_last_topic);
+    _switching_help = false;
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
 
-bool QuerySidePalette::update_context_help(Sql_editor *editor)
+/**
+ * Triggered by timer or manually to find a help topic from the given editor's text + position.
+ */
+bool QuerySidePalette::find_context_help(Sql_editor *editor)
 {
   _help_timer = NULL;
 
-  std::string text, title;
+  if (_help_task->is_busy())
+    return false;
 
-   // Caret position as <column, row>.
+  // If no editor was given use the currently active one in the SQL editor form.
+  if (editor == NULL)
+  {
+    SqlEditorForm::Ref form = _owner.lock();
+    Sql_editor::Ref editor_ref = form->active_sql_editor();
+    if (editor_ref)
+      editor = &(*editor_ref);
+    else
+      return false;
+  }
+
+  // Caret position as <column, row>.
   std::pair<int, int> caret = editor->cursor_pos_row_column(true);
 
-  SqlEditorForm::Ref form = _owner.lock();
-  std::string topic = DbSqlEditorContextHelp::find_help_topic_from_position(form,
-    editor->current_statement(), caret);
-  if (update_help_history(topic))
-    show_help_topic(topic);
-  
+  _help_task->exec(false, boost::bind(&QuerySidePalette::get_help_topic_threaded, this, _1, 
+    editor->current_statement(), caret));
+
   return false; // Don't re-run this task, it's a single-shot.
 }
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Get a help topic from the current editor position. This might involve server access already
+ * to resolve ambiguities, hence it is a threaded task. Once we got the topic we run another task
+ * to get the actual help text from it.
+ */
+grt::StringRef QuerySidePalette::get_help_topic_threaded(grt::GRT *, const std::string &query,
+  std::pair<int, int> caret)
+{
+  SqlEditorForm::Ref form = _owner.lock();
+  if (!form)
+    return "";
+
+  std::string topic = DbSqlEditorContextHelp::find_help_topic_from_position(form, query, caret);
+
+  if (!_help_task->task()->is_cancelled())
+    _help_task->execute_in_main_thread(boost::bind(&QuerySidePalette::process_help_topic, this, topic), false, false);
+
+  return "";
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void QuerySidePalette::process_help_topic(const std::string &topic)
+{
+  if (update_help_history(topic))
+    show_help_text_for_topic(topic);
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Adds the given topic to the topic history if the current topic is not the same and updates
  * the forward/backward buttons.
  *
  * @returns True if the topic was added to the history, otherwise false.
  */
-bool QuerySidePalette::update_help_history(std::string topic)
+bool QuerySidePalette::update_help_history(const std::string &topic)
 {
-  topic = base::toupper(topic);
-  if (_current_topic > 0 && _topic_history[_current_topic] == topic)
+  std::string topic_upper = base::toupper(topic);
+  if (_current_topic_index > 0 && _topic_history[_current_topic_index] == topic_upper)
     return _no_help;
 
   if (!topic.empty())
   {
-    _current_topic++;
+    _current_topic_index++;
 
     // Remove everything after the new topic position.
-    _topic_history.erase(_topic_history.begin() + _current_topic, _topic_history.end());
-    _topic_history.push_back(topic);
-    _back_item->set_enabled(_current_topic > 0);
+    _topic_history.erase(_topic_history.begin() + _current_topic_index, _topic_history.end());
+    _topic_history.push_back(topic_upper);
+    _back_item->set_enabled(_current_topic_index > 0);
     _forward_item->set_enabled(false);
   }
   return true;
@@ -473,7 +552,7 @@ void QuerySidePalette::click_link(const std::string &link)
     while (topic.find("  ") != std::string::npos)
       base::replace(topic, "  ", " ");
     if (update_help_history(topic))
-      show_help_topic(topic);
+      show_help_text_for_topic(topic);
   }
   else
     mforms::Utilities::open_url(link);
@@ -483,7 +562,7 @@ void QuerySidePalette::click_link(const std::string &link)
 
 ToolBar* QuerySidePalette::prepare_snippet_toolbar()
 {
-  ToolBar* toolbar = manage(new ToolBar(mforms::PaletteToolBar));
+  ToolBar* toolbar = manage(new ToolBar(mforms::SecondaryToolBar));
   toolbar->set_name("snippet_toolbar");
 #ifndef __APPLE__
   toolbar->set_padding(0, 0, 0, 0);
@@ -495,7 +574,6 @@ ToolBar* QuerySidePalette::prepare_snippet_toolbar()
   item->set_tooltip(_("Add the current editor content as new snippet"));
   scoped_connect(item->signal_activated(), boost::bind(&QuerySidePalette::snippet_toolbar_item_activated, this, _1));
   toolbar->add_item(item);
-  item->release();
 
   /* not needed in toolbar
   item = manage(new ToolBarItem(mforms::ActionItem));
@@ -504,14 +582,12 @@ ToolBar* QuerySidePalette::prepare_snippet_toolbar()
   item->set_tooltip(_("Delete the currently selected snippet"));
   scoped_connect(item->signal_activated(), boost::bind(&QuerySidePalette::snippet_toolbar_item_activated, this, _1));
   toolbar->add_item(item);
-  item->release();
    */
 
-  item = manage(new ToolBarItem(mforms::SeparatorItem));
+  item = mforms::manage(new ToolBarItem(mforms::SeparatorItem));
   toolbar->add_item(item);
-  item->release();
 
-  item = manage(new ToolBarItem(mforms::SelectorItem));
+  item = mforms::manage(new ToolBarItem(mforms::SelectorItem));
   item->set_name("select_category");
 
   DbSqlEditorSnippets* snippets_model = DbSqlEditorSnippets::get_instance();
@@ -520,28 +596,24 @@ ToolBar* QuerySidePalette::prepare_snippet_toolbar()
   item->set_text(USER_SNIPPETS);
   item->set_tooltip(_("Select a snippet category for display"));
   toolbar->add_item(item);
-  item->release();
 
-  item = manage(new ToolBarItem(mforms::SeparatorItem));
+  item = mforms::manage(new ToolBarItem(mforms::SeparatorItem));
   toolbar->add_item(item);
-  item->release();
 
 
-  item = manage(new ToolBarItem(mforms::ActionItem));
+  item = mforms::manage(new ToolBarItem(mforms::ActionItem));
   item->set_name("replace_text");
   item->set_icon(App::get()->get_resource_path("snippet_use.png"));
   item->set_tooltip(_("Replace the current text by this snippet"));
   scoped_connect(item->signal_activated(), boost::bind(&QuerySidePalette::snippet_toolbar_item_activated, this, _1));
   toolbar->add_item(item);
-  item->release();
 
-  item = manage(new ToolBarItem(mforms::ActionItem));
+  item = mforms::manage(new ToolBarItem(mforms::ActionItem));
   item->set_name("insert_text");
   item->set_icon(App::get()->get_resource_path("snippet_insert.png"));
   item->set_tooltip(_("Insert the snippet text at the current caret position replacing selected text if there is any"));
   scoped_connect(item->signal_activated(), boost::bind(&QuerySidePalette::snippet_toolbar_item_activated, this, _1));
   toolbar->add_item(item);
-  item->release();
 
   item = manage(new ToolBarItem(mforms::ActionItem));
   item->set_name("copy_to_clipboard");
@@ -549,7 +621,6 @@ ToolBar* QuerySidePalette::prepare_snippet_toolbar()
   item->set_tooltip(_("Copy the snippet text to the clipboard"));
   scoped_connect(item->signal_activated(), boost::bind(&QuerySidePalette::snippet_toolbar_item_activated, this, _1));
   toolbar->add_item(item);
-  item->release();
 
   return toolbar;
 }
@@ -603,7 +674,7 @@ void QuerySidePalette::close_popover()
 
 ToolBar* QuerySidePalette::prepare_help_toolbar()
 {
-  ToolBar* toolbar = manage(new ToolBar(mforms::PaletteToolBar));
+  ToolBar* toolbar = manage(new ToolBar(mforms::SecondaryToolBar));
   toolbar->set_name("help_toolbar");
 #ifndef __APPLE__
   toolbar->set_padding(0, 0, 0, 0);
@@ -626,9 +697,26 @@ ToolBar* QuerySidePalette::prepare_help_toolbar()
   scoped_connect(_forward_item->signal_activated(), boost::bind(&QuerySidePalette::help_toolbar_item_activated, this, _1));
   toolbar->add_item(_forward_item);
 
-  ToolBarItem* item = manage(new ToolBarItem(mforms::SeparatorItem));
+  toolbar->add_item(manage(new ToolBarItem(mforms::SeparatorItem)));
+
+  ToolBarItem *item = manage(new ToolBarItem(mforms::ToggleItem));
+  item->set_name("toggle-auto-context-help");
+  item->set_icon(App::get()->get_resource_path("wb-toolbar_automatic-help-off.png"));
+  item->set_alt_icon(App::get()->get_resource_path("wb-toolbar_automatic-help-on.png"));
+  item->set_tooltip(_("Toggle automatic context help"));
+  item->set_checked(_automatic_help);
+  scoped_connect(item->signal_activated(), boost::bind(&QuerySidePalette::help_toolbar_item_activated, this, _1));
   toolbar->add_item(item);
-  item->release();
+
+  _manual_help_item = manage(new ToolBarItem(mforms::ActionItem));
+  _manual_help_item->set_name("manual-context-help");
+  _manual_help_item->set_icon(App::get()->get_resource_path("wb-toolbar_manual-help.png"));
+  _manual_help_item->set_tooltip(_("Get context help for the item at the current caret position"));
+  _manual_help_item->set_enabled(!_automatic_help);
+  scoped_connect(_manual_help_item->signal_activated(), boost::bind(&QuerySidePalette::help_toolbar_item_activated, this, _1));
+  toolbar->add_item(_manual_help_item);
+
+  toolbar->add_item(manage(new ToolBarItem(mforms::SeparatorItem)));
 
   _quick_jump_item = manage(new ToolBarItem(mforms::SelectorItem));
   _quick_jump_item->set_name("quick_jump");
@@ -660,21 +748,21 @@ void QuerySidePalette::help_toolbar_item_activated(ToolBarItem* item)
     return;
 
   std::string action = item->get_name();
-  if (action == "back" && _current_topic > 0)
+  if (action == "back" && _current_topic_index > 0)
   {
-    std::string topic = _topic_history[--_current_topic];
-    _back_item->set_enabled(_current_topic > 0);
+    std::string topic = _topic_history[--_current_topic_index];
+    _back_item->set_enabled(_current_topic_index > 0);
     _forward_item->set_enabled(true);
-    show_help_topic(topic);
+    show_help_text_for_topic(topic);
     return;
   }
 
-  if (action == "forward" && _current_topic < (int)_topic_history.size() - 1)
+  if (action == "forward" && _current_topic_index < (int)_topic_history.size() - 1)
   {
-    std::string topic = _topic_history[++_current_topic];
+    std::string topic = _topic_history[++_current_topic_index];
     _back_item->set_enabled(true);
-    _forward_item->set_enabled(_current_topic < (int)_topic_history.size() - 1);
-    show_help_topic(topic);
+    _forward_item->set_enabled(_current_topic_index < (int)_topic_history.size() - 1);
+    show_help_text_for_topic(topic);
     return;
   }
 
@@ -682,20 +770,32 @@ void QuerySidePalette::help_toolbar_item_activated(ToolBarItem* item)
   {
     std::string topic = _quick_jump_item->get_text();
     if (update_help_history(topic))
-      show_help_topic(topic);
+      show_help_text_for_topic(topic);
     return;
   }
 
-  if (_current_topic >= 0)
+  if (action == "toggle-auto-context-help")
+  {
+    _automatic_help = item->get_checked();
+    _manual_help_item->set_enabled(!_automatic_help);
+    _grtm->set_app_option("DbSqlEditor:DisableAutomaticContextHelp", grt::IntegerRef(_automatic_help ? 0 : 1));
+
+    return;
+  }
+
+  if (action == "manual-context-help")
+    find_context_help(NULL);
+
+  if (_current_topic_index >= 0)
   {
     if (action == "copy_to_clipboard")
     {
-      std::pair<std::string, std::string> entry = _topic_cache[_topic_history[_current_topic]];
+      std::pair<std::string, std::string> entry = _topic_cache[_topic_history[_current_topic_index]];
       mforms::Utilities::set_clipboard_text(entry.first);
     }
     if (action == "copy_html_to_clipboard")
     {
-      std::pair<std::string, std::string> entry = _topic_cache[_topic_history[_current_topic]];
+      std::pair<std::string, std::string> entry = _topic_cache[_topic_history[_current_topic_index]];
       mforms::Utilities::set_clipboard_text(entry.second);
     }
   }
@@ -854,8 +954,10 @@ std::string QuerySidePalette::format_help_as_html(const std::string &text)
     // Replace triple hyphen by typographic dash.
     pattern = "---";
     pattern.GlobalReplace("&mdash;", &line);
-    
-    result += " " + line;
+
+    if (!result.empty())
+      result += " ";
+    result += line;
   }
 
   // Replace "URL: <link>" by "See Also: <link>"
