@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -73,7 +73,7 @@ void SqlEditorForm::auto_save()
 // Save all script buffers, including scratch buffers
 void SqlEditorForm::save_workspace(const std::string &workspace_name, bool is_autosave)
 {
-  GMutexLock sql_editors_mutex(_sql_editors_mutex);
+  MutexLock sql_editors_mutex(_sql_editors_mutex);
   
   if (_sql_editors.size() < 1 || (_sql_editors.size() == 1 && _sql_editors[0]->is_scratch && _sql_editors[0]->editor->empty()))
     return;
@@ -519,16 +519,16 @@ int SqlEditorForm::add_sql_editor(bool scratch, bool start_collapsed)
   sql_editor->set_auto_completion_cache(_auto_completion_cache);
   sql_editor->sql_mode(_sql_mode);
   sql_editor->set_current_schema(active_schema());
+  sql_editor->register_file_drop_for(this);
   scoped_connect(sql_editor->text_change_signal(),
     boost::bind(&SqlEditorForm::do_partial_ui_refresh, this, (int)RefreshEditorTitle));
   int sql_editor_index;
   {
-    GMutexLock sql_editors_mutex(_sql_editors_mutex);
+    MutexLock sql_editors_mutex(_sql_editors_mutex);
     Sql_editor_info::Ref info(new Sql_editor_info());
     info->toolbar = setup_editor_toolbar(sql_editor);
     info->editor = sql_editor;
     info->is_scratch = scratch;
-    info->recordset_mutex = g_mutex_new();
     info->start_collapsed = start_collapsed;
     if (!scratch)
       info->caption = strfmt("SQL File %i", ++_sql_editors_serial);
@@ -547,6 +547,59 @@ int SqlEditorForm::add_sql_editor(bool scratch, bool start_collapsed)
   return sql_editor_index;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+mforms::DragOperation SqlEditorForm::drag_over(mforms::View *sender, base::Point p,
+  const std::vector<std::string> &formats)
+{
+  // We can accept dropped files.
+  if (std::find(formats.begin(), formats.end(), mforms::DragFormatFileName) != formats.end())
+    return mforms::DragOperationCopy; // Copy to indicate we don't do anything with the files.
+  return mforms::DragOperationNone;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+mforms::DragOperation SqlEditorForm::files_dropped(mforms::View *sender, base::Point p,
+  const std::vector<std::string> &file_names)
+{
+#if _WIN32
+  bool case_sensitive = false;
+#else
+  bool case_sensitive = true;
+#endif
+  Sql_editors editors_copy;
+  {
+    MutexLock sql_editors_mutex(_sql_editors_mutex);
+    editors_copy = _sql_editors;
+  }
+  for (size_t i = 0; i < file_names.size(); ++i)
+  {
+    // First see if we have already a tab with that file open.
+    bool found = false;
+    for (size_t j = 0; j < editors_copy.size(); ++j)
+    {
+      if (base::same_string(editors_copy[j]->filename, file_names[i], case_sensitive))
+      {
+        found = true;
+
+        // Ignore this file name, but if this is the only one dropped the activate the particular
+        // sql editor.
+        if (file_names.size() == 1)
+          active_sql_editor_index(j); // TODO: signal the UI that we changed the active sql editor.
+        break;
+      }
+    }
+
+    if (!found)
+      open_file(file_names[i], true);
+  }
+
+  return mforms::DragOperationCopy;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Helper to rename all auto-save files under the from index to the to index, provided that:
  * - A file with that name exists and
@@ -602,7 +655,7 @@ void SqlEditorForm::remove_sql_editor(int editor_index)
   info->editor->stop_processing();
   {
     // Notify the UI that recordsets must be closed.
-    GMutexLock  lock(info->recordset_mutex);
+    MutexLock  lock(info->recordset_mutex);
     RecordsetsRef rsets = info->recordsets;
     if (rsets != NULL)
     {
@@ -614,12 +667,10 @@ void SqlEditorForm::remove_sql_editor(int editor_index)
       }
     }
   }
-  g_mutex_free(info->recordset_mutex);
-  info->recordset_mutex = NULL;
   
   Sql_editor::Ref active_sql_editor;
   {
-    GMutexLock sql_editors_mutex(_sql_editors_mutex);
+    MutexLock sql_editors_mutex(_sql_editors_mutex);
     if ((editor_index < 0) || (editor_index >= (int)_sql_editors.size()))
       return;
     
@@ -653,7 +704,7 @@ void SqlEditorForm::remove_sql_editor(int editor_index)
 
 int SqlEditorForm::sql_editor_count()
 {
-  GMutexLock sql_editors_mutex(_sql_editors_mutex);
+  MutexLock sql_editors_mutex(_sql_editors_mutex);
   return _sql_editors.size();
 }
 
@@ -683,7 +734,7 @@ Sql_editor::Ref SqlEditorForm::active_sql_editor_or_new_scratch()
 
 Sql_editor::Ref SqlEditorForm::sql_editor(int new_index)
 {
-  GMutexLock sql_editors_mutex(_sql_editors_mutex);
+  MutexLock sql_editors_mutex(_sql_editors_mutex);
   if (new_index >= 0 && new_index < (int)_sql_editors.size())
     return _sql_editors.at(new_index)->editor;
   return Sql_editor::Ref();
@@ -696,7 +747,7 @@ bool SqlEditorForm::sql_editor_reorder(Sql_editor::Ref editor, int new_index)
   if (old_index < 0 || old_index == new_index || new_index < 0)
     return false;
 
-  GMutexTryLock editor_lock(_sql_editors_mutex);
+  MutexTryLock editor_lock(_sql_editors_mutex);
   if (editor_lock.locked())
   {
     // First reorder auto-save files so file names are correct in case of errors.
@@ -747,13 +798,13 @@ bool SqlEditorForm::sql_editor_will_close(int new_index)
 {
   int edited_recordsets = 0;
   {
-    GMutexLock sql_editors_mutex(_sql_editors_mutex);
+    MutexLock sql_editors_mutex(_sql_editors_mutex);
     if (new_index < 0 || new_index >= (int)_sql_editors.size() || _sql_editors.at(new_index)->busy)
       return false;
   }
   
   // the recordset lock will prevent the editor from being closed elsewhere
-  GMutexTryLock recordset_mutex(_sql_editors[new_index]->recordset_mutex);
+  MutexTryLock recordset_mutex(_sql_editors[new_index]->recordset_mutex);
   if (!recordset_mutex.locked())
     return false;
   {
@@ -875,7 +926,7 @@ void SqlEditorForm::sql_editor_caption(int new_index, std::string caption)
 
 //--------------------------------------------------------------------------------------------------
 
-int SqlEditorForm::sql_editor_index_for_recordset(long long rset)
+int SqlEditorForm::sql_editor_index_for_recordset(long rset)
 {
   int i = 0;
   for (Sql_editors::iterator iter = _sql_editors.begin(); iter != _sql_editors.end(); ++iter)
@@ -908,25 +959,47 @@ RecordsetsRef SqlEditorForm::sql_editor_recordsets(const int index)
   return rsets;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 int SqlEditorForm::sql_editor_index(Sql_editor::Ref editor)
 {
-  GMutexLock ed_lock(_sql_editors_mutex);
-  for (int i= 0; i < (int)_sql_editors.size(); i++)
+  MutexLock ed_lock(_sql_editors_mutex);
+  for (size_t i = 0; i < _sql_editors.size(); i++)
   {
     if (_sql_editors[i]->editor == editor)
-      return i;
+      return (int)i;
   }
   return -1;
 }
 
+//--------------------------------------------------------------------------------------------------
 
-void SqlEditorForm::set_sql_editor_text(const char *sql)
+/**
+ * Helper to determine if a given editor belongs to this form if you *don't* have a shared_ptr.
+ * Otherwise just use the sql_editor_index() function.
+ */
+bool SqlEditorForm::contains_editor(Sql_editor *editor)
 {
-  if (active_sql_editor())
+  MutexLock ed_lock(_sql_editors_mutex);
+  for (size_t i = 0; i < _sql_editors.size(); i++)
   {
-    active_sql_editor()->set_refresh_enabled(true);
-    active_sql_editor()->sql(sql);
+    if (_sql_editors[i]->editor.get() == editor)
+      return true;
+  }
+  return false;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void SqlEditorForm::set_sql_editor_text(const char *sql, int editor_index)
+{
+  if (editor_index < 0)
+    editor_index = _active_sql_editor_index;
+
+  if (editor_index >= 0 && editor_index < (int)_sql_editors.size())
+  {
+    _sql_editors[editor_index]->editor->set_refresh_enabled(true);
+    _sql_editors[editor_index]->editor->sql(sql);
   }
 }
 
@@ -1010,9 +1083,6 @@ void SqlEditorForm::sql_editor_open_file(int index, const std::string &file_path
   gsize length = base_get_file_size(file_path.c_str());
   GError *error = NULL;
   
-  if (length == 0)
-    return;
-
   if (length > EDITOR_TEXT_LIMIT)
   {
     // File is larger than 100 MB. Tell the user we are going to switch off code folding and
@@ -1065,7 +1135,7 @@ void SqlEditorForm::sql_editor_open_file(int index, const std::string &file_path
   }  
   
   _updating_sql_editor++;
-  set_sql_editor_text(utf8_contents.c_str());
+  set_sql_editor_text(utf8_contents.c_str(), index);
   _updating_sql_editor--;
   
   Sql_editor_info::Ref editor(_sql_editors[index]);
