@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -49,14 +49,11 @@ DEFAULT_LOG_DOMAIN("AutoCCache");
 //--------------------------------------------------------------------------------------------------
 
 AutoCompleteCache::AutoCompleteCache(const std::string &connection_id,
-  boost::function<base::GMutexLock (sql::Dbc_connection_handler::Ref &)> get_connection,
+  boost::function<base::RecMutexLock (sql::Dbc_connection_handler::Ref &)> get_connection,
   const std::string &cache_dir, boost::function<void (bool)> feedback)
-  : _refresh_thread(0), _connection_id(connection_id), _get_connection(get_connection), _schema_list_fetched(false), _shutdown(false)
+  : _refresh_thread(0), _cache_working(1), _connection_id(connection_id), _get_connection(get_connection), _schema_list_fetched(false), _shutdown(false)
 {
   _feedback = feedback;
-  _pending_mutex = g_mutex_new();
-  _shutdown_mutex = g_mutex_new();
-  _sqconn_mutex = g_mutex_new();
   _sqconn = new sqlite::connection(make_path(cache_dir, _connection_id)+".cache");
   sqlite::execute(*_sqconn, "PRAGMA temp_store=MEMORY", true);
   sqlite::execute(*_sqconn, "PRAGMA synchronous=NORMAL", true);
@@ -98,19 +95,19 @@ AutoCompleteCache::AutoCompleteCache(const std::string &connection_id,
 
 void AutoCompleteCache::shutdown()
 {
-  base::GMutexLock sd_lock(_shutdown_mutex);
+  base::MutexLock sd_lock(_shutdown_mutex);
   _shutdown = true;
 
   {
-    base::GMutexLock lock(_pending_mutex);
+    base::MutexLock lock(_pending_mutex);
     _pending_refresh_schema.clear();
     _feedback = NULL;
   }
-  GThread *thread = _refresh_thread;
+
   if (_refresh_thread)
   {
     log_debug2("Waiting for worker thread to finish...\n");
-    g_thread_join(thread);
+    g_thread_join(_refresh_thread);
     _refresh_thread = NULL;
     log_debug2("Worker thread finished.\n");
   }
@@ -123,9 +120,6 @@ AutoCompleteCache::~AutoCompleteCache()
   g_assert(_shutdown);
 
   delete _sqconn;
-  g_mutex_free(_sqconn_mutex);
-  g_mutex_free(_pending_mutex);
-  g_mutex_free(_shutdown_mutex);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -138,8 +132,8 @@ std::vector<std::string>  AutoCompleteCache::get_matching_schema_names(const std
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::GMutexLock sd_lock(_shutdown_mutex);
-    base::GMutexLock lock(_sqconn_mutex);
+    base::MutexLock sd_lock(_shutdown_mutex);
+    base::MutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM schemas WHERE name LIKE ? ESCAPE '\\'");
     q.bind(1, base::escape_sql_string(prefix, true) + "%");
     if (q.emit())
@@ -175,8 +169,8 @@ std::vector<std::string> AutoCompleteCache::get_matching_table_names(const std::
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::GMutexLock sd_lock(_shutdown_mutex);
-    base::GMutexLock lock(_sqconn_mutex);
+    base::MutexLock sd_lock(_shutdown_mutex);
+    base::MutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM tables WHERE schema LIKE ? ESCAPE '\\' AND name LIKE ? ESCAPE '\\'");
     q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
     q.bind(2, base::escape_sql_string(prefix, true) + "%");
@@ -213,8 +207,8 @@ std::vector<std::string> AutoCompleteCache::get_matching_column_names(const std:
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::GMutexLock sd_lock(_shutdown_mutex);
-    base::GMutexLock lock(_sqconn_mutex);
+    base::MutexLock sd_lock(_shutdown_mutex);
+    base::MutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM columns WHERE schema LIKE ? ESCAPE '\\' "
       "AND tabl LIKE ? ESCAPE '\\' AND name LIKE ? ESCAPE '\\'");
     q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
@@ -255,8 +249,8 @@ std::vector<std::string> AutoCompleteCache::get_matching_procedure_names(const s
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::GMutexLock sd_lock(_shutdown_mutex);
-    base::GMutexLock lock(_sqconn_mutex);
+    base::MutexLock sd_lock(_shutdown_mutex);
+    base::MutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM routines WHERE schema LIKE ? ESCAPE '\\' "
       "AND name LIKE ? ESCAPE '\\' AND is_function=0");
     q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
@@ -295,8 +289,8 @@ std::vector<std::string> AutoCompleteCache::get_matching_function_names(const st
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::GMutexLock sd_lock(_shutdown_mutex);
-    base::GMutexLock lock(_sqconn_mutex);
+    base::MutexLock sd_lock(_shutdown_mutex);
+    base::MutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM routines WHERE schema LIKE ? ESCAPE '\\' "
       "AND name LIKE ? ESCAPE '\\' AND is_function=1");
     q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
@@ -335,8 +329,8 @@ bool AutoCompleteCache::refresh_schema_cache_if_needed(const std::string &schema
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::GMutexLock sd_lock(_shutdown_mutex);
-    base::GMutexLock lock(_sqconn_mutex);
+    base::MutexLock sd_lock(_shutdown_mutex);
+    base::MutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT last_refresh FROM schemas WHERE name LIKE ? ESCAPE '\\' ");
     q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
     if (q.emit())
@@ -380,14 +374,15 @@ void AutoCompleteCache::refresh_table_cache(const std::string &schema, const std
 void AutoCompleteCache::refresh_cache_thread()
 {
   log_debug2("entering worker thread\n");
+
+//  while(!_refresh_thread); //wait until _rfresh_thread is assigned
+
   while (!_shutdown)
   {
     std::string schema_to_fetch;
 
     if (!get_pending_refresh(schema_to_fetch))
     {
-        // signal the main thread that the thread is (about to be) gone
-        _refresh_thread = NULL;
         break;
     }
     
@@ -409,7 +404,7 @@ void AutoCompleteCache::refresh_cache_thread()
         if (s == std::string::npos)
         {
           refresh_tables_w(schema_to_fetch);
-    
+
           refresh_routines_w(schema_to_fetch);
         }
         else
@@ -425,11 +420,18 @@ void AutoCompleteCache::refresh_cache_thread()
       }
     }
   }
-  
+
+
+
+  // signal the main thread that the thread is (about to be) gone
+  _refresh_thread = NULL;
+  _cache_working.post();
+
   if (_feedback && !_shutdown)
     _feedback(false);
 
   log_debug2("leaving worker thread\n");
+
 }
 
 
@@ -446,9 +448,10 @@ void AutoCompleteCache::refresh_schemas_w()
   std::vector<std::string> schemas;
   {
     sql::Dbc_connection_handler::Ref conn;
-    base::GMutexLock lock(_get_connection(conn)); 
+    base::RecMutexLock lock(_get_connection(conn));
     {
-      std::auto_ptr<sql::ResultSet> rs(conn->ref->createStatement()->executeQuery("SHOW DATABASES"));
+      std::auto_ptr<sql::Statement> stmt(conn->ref->createStatement());
+      std::auto_ptr<sql::ResultSet> rs(stmt->executeQuery("SHOW DATABASES"));
       if (rs.get())
       {
         while (rs->next() && !_shutdown)
@@ -472,7 +475,7 @@ void AutoCompleteCache::refresh_tables_w(const std::string &schema)
     sql::Dbc_connection_handler::Ref conn;
 
     // TODO: is it possible that the connection can be locked even it was already deleted???
-    base::GMutexLock lock(_get_connection(conn));
+    base::RecMutexLock lock(_get_connection(conn));
     {
       std::auto_ptr<sql::ResultSet> rs(conn->ref->createStatement()->executeQuery(std::string(base::sqlstring("SHOW FULL TABLES FROM !", 0) << schema)));
       if (rs.get())
@@ -501,7 +504,7 @@ void AutoCompleteCache::refresh_routines_w(const std::string &schema)
   std::vector<std::pair<std::string, bool> > routines;
   {
     sql::Dbc_connection_handler::Ref conn;
-    base::GMutexLock lock(_get_connection(conn));    
+    base::RecMutexLock lock(_get_connection(conn));
     {
       std::auto_ptr<sql::ResultSet> rs(conn->ref->createStatement()->executeQuery(std::string(base::sqlstring("SHOW PROCEDURE STATUS WHERE Db=?", 0) << schema)));
       if (rs.get())
@@ -528,9 +531,10 @@ void AutoCompleteCache::refresh_columns_w(const std::string &schema, const std::
   std::vector<std::string> columns;
   {
     sql::Dbc_connection_handler::Ref conn;
-    base::GMutexLock lock(_get_connection(conn));    
+    base::RecMutexLock lock(_get_connection(conn));
     {
-      std::auto_ptr<sql::ResultSet> rs(conn->ref->createStatement()->executeQuery(std::string(base::sqlstring("SHOW COLUMNS FROM !.!", 0) << schema << table)));
+      std::auto_ptr<sql::Statement> statement(conn->ref->createStatement());
+      std::auto_ptr<sql::ResultSet> rs(statement->executeQuery(std::string(base::sqlstring("SHOW COLUMNS FROM !.!", 0) << schema << table)));
       if (rs.get())
       {
         while (rs->next() && !_shutdown)
@@ -627,7 +631,7 @@ void AutoCompleteCache::init_db()
 
 bool AutoCompleteCache::is_schema_list_fetch_done()
 {
-  base::GMutexLock lock(_sqconn_mutex);
+  base::MutexLock lock(_sqconn_mutex);
   sqlite::query q(*_sqconn, "select * from schemas");
   if (q.emit())
     return true;
@@ -637,7 +641,7 @@ bool AutoCompleteCache::is_schema_list_fetch_done()
 
 bool AutoCompleteCache::is_schema_tables_fetch_done(const std::string &schema)
 {
-  base::GMutexLock lock(_sqconn_mutex);
+  base::MutexLock lock(_sqconn_mutex);
   sqlite::query q(*_sqconn, "select * from tables where schema = ?");
   q.bind(1, schema);
   if (q.emit())
@@ -647,7 +651,7 @@ bool AutoCompleteCache::is_schema_tables_fetch_done(const std::string &schema)
 
 bool AutoCompleteCache::is_schema_table_columns_fetch_done(const std::string &schema, const std::string &table)
 {
-  base::GMutexLock lock(_sqconn_mutex);
+  base::MutexLock lock(_sqconn_mutex);
   sqlite::query q(*_sqconn, "select * from columns where schema = ? and tabl = ?");
   q.bind(1, schema);
   q.bind(2, table);
@@ -658,7 +662,7 @@ bool AutoCompleteCache::is_schema_table_columns_fetch_done(const std::string &sc
 
 bool AutoCompleteCache::is_schema_routines_fetch_done(const std::string &schema)
 {
-  base::GMutexLock lock(_sqconn_mutex);
+  base::MutexLock lock(_sqconn_mutex);
   sqlite::query q(*_sqconn, "select * from routines where schema = ?");
   q.bind(1, schema);
   if (q.emit())
@@ -694,8 +698,8 @@ void AutoCompleteCache::update_schemas(const std::vector<std::string> &schemas)
     if (!_shutdown)
     {
       // Ensures shutdown is not done while processing
-      base::GMutexLock sd_lock(_shutdown_mutex);
-      base::GMutexLock lock(_sqconn_mutex);
+      base::MutexLock sd_lock(_shutdown_mutex);
+      base::MutexLock lock(_sqconn_mutex);
  
       std::map<std::string, int> old_schema_update_times;
       {
@@ -758,14 +762,12 @@ void AutoCompleteCache::update_schema_tables(const std::string &schema,
     if (!_shutdown)
     {
       // Ensures shutdown is not done while processing
-      base::GMutexLock sd_lock(_shutdown_mutex);
-      base::GMutexLock lock(_sqconn_mutex);
+      base::MutexLock sd_lock(_shutdown_mutex);
+      base::MutexLock lock(_sqconn_mutex);
  
       touch_schema_record(schema);
     
       sqlide::Sqlite_transaction_guarder  trans(_sqconn, false); //will be commited when will go out of the scope
-
-
       // Clear records for this schema if we are not just appending.
       if (!just_append)
       {
@@ -773,6 +775,7 @@ void AutoCompleteCache::update_schema_tables(const std::string &schema,
         del.bind(1, schema);
         del.emit();
       }
+
       sqlite::execute insert(*_sqconn, "insert into tables (schema, name, is_view) values (?, ?, ?)");
       for (std::vector<std::pair<std::string, bool> >::const_iterator t = tables.begin(); t != tables.end(); ++t)
       {
@@ -807,8 +810,8 @@ void AutoCompleteCache::update_schema_routines(const std::string &schema,
     if (!_shutdown)
     {
       // Ensures shutdown is not done while processing
-      base::GMutexLock sd_lock(_shutdown_mutex);
-      base::GMutexLock lock(_sqconn_mutex);
+      base::MutexLock sd_lock(_shutdown_mutex);
+      base::MutexLock lock(_sqconn_mutex);
     
       touch_schema_record(schema);
 
@@ -856,11 +859,11 @@ void AutoCompleteCache::update_table_columns(const std::string &schema, const st
     if (!_shutdown)
     {
       // Ensures shutdown is not done while processing
-      base::GMutexLock sd_lock(_shutdown_mutex);
-      base::GMutexLock lock(_sqconn_mutex);
-      sqlide::Sqlite_transaction_guarder  trans(_sqconn, false); //will be commited when will go out of the scope
+      base::MutexLock sd_lock(_shutdown_mutex);
+      base::MutexLock lock(_sqconn_mutex);
       try
       {
+        sqlide::Sqlite_transaction_guarder  trans(_sqconn, false); //will be commited when will go out of the scope
         // clear records for this schema
         {
           sqlite::execute del(*_sqconn, "delete from columns where schema = ? and tabl = ?");
@@ -869,6 +872,7 @@ void AutoCompleteCache::update_table_columns(const std::string &schema, const st
           del.emit();
         }
 
+
         sqlite::query insert(*_sqconn, "insert into columns (schema, tabl, name) values (?, ?, ?)");
         for (std::vector<std::string>::const_iterator t = columns.begin(); t != columns.end(); ++t)
         {
@@ -901,8 +905,8 @@ void AutoCompleteCache::add_pending_refresh(const std::string& task)
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::GMutexLock sd_lock(_shutdown_mutex);
-    base::GMutexLock lock(_pending_mutex);
+    base::MutexLock sd_lock(_shutdown_mutex);
+    base::MutexLock lock(_pending_mutex);
     if ((!task.empty() || !_schema_list_fetched) && std::find(_pending_refresh_schema.begin(), _pending_refresh_schema.end(), task)
         == _pending_refresh_schema.end())
     {
@@ -922,8 +926,8 @@ bool AutoCompleteCache::get_pending_refresh(std::string &task)
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::GMutexLock sd_lock(_shutdown_mutex);
-    base::GMutexLock lock(_pending_mutex);
+    base::MutexLock sd_lock(_shutdown_mutex);
+    base::MutexLock lock(_pending_mutex);
 
     if (!_pending_refresh_schema.empty())
     {
@@ -939,13 +943,20 @@ bool AutoCompleteCache::get_pending_refresh(std::string &task)
 void AutoCompleteCache::create_worker_thread()
 {
   // Fire up thread to start caching.
-  if (!_refresh_thread)
-  {
+  //if (!_refresh_thread)
+  if(!_cache_working.try_wait()) // If there is already working thread, just do nothing and exit.
+    return;
+
+  _refresh_thread = NULL;
+//  if (!_refresh_thread)
+//  {
     if (!_shutdown)
     {
       GError *error = NULL;
       log_debug3("creating worker thread\n");
-      _refresh_thread = g_thread_create(&AutoCompleteCache::_refresh_cache_thread, this, 1, &error);
+//      if (_refresh_thread)
+//        throw std::logic_error("Starting new AutoCompleteCache worker, but the previous one didn't die");
+      _refresh_thread = base::create_thread(&AutoCompleteCache::_refresh_cache_thread, this, &error);
       if (!_refresh_thread)
       {
         log_error("Error creating autocompletion worker thread: %s\n", error ? error->message : "out of mem?");
@@ -955,5 +966,5 @@ void AutoCompleteCache::create_worker_thread()
         if (_feedback)
           _feedback(true);
     }
-  } 
+//  }
 }
