@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,6 +28,7 @@
 #include "grtui/binary_data_editor.h"
 
 #include "mforms/menubar.h"
+#include "mforms/toolbar.h"
 #include "mforms/utilities.h"
 #include "mforms/filechooser.h"
 
@@ -42,21 +43,18 @@
 using namespace bec;
 using namespace base;
 
-#define CATCH_AND_DISPATCH_EXCEPTION(rethrow, context, res) \
+#define CATCH_AND_DISPATCH_EXCEPTION(rethrow, context) \
 catch (sql::SQLException &e)\
 {\
   rethrow ? throw : task->send_msg(grt::ErrorMsg, strfmt("Error Code: %i\n%s", e.getErrorCode(), e.what()), context);\
-  res= false;\
 }\
 catch (sqlite::database_exception &e)\
 {\
   rethrow ? throw : task->send_msg(grt::ErrorMsg, e.what(), context);\
-  res= false;\
 }\
 catch (std::exception &e)\
 {\
   rethrow ? throw : task->send_msg(grt::ErrorMsg, e.what(), context);\
-  res= false;\
 }
 
 
@@ -64,6 +62,11 @@ const std::string ERRMSG_PENDING_CHANGES= _("There are pending changes. Please c
 std::string Recordset::_add_change_record_statement= "insert into `changes` (`record`, `action`, `column`) values (?, ?, ?)";
 
 
+Recordset::ClientData::~ClientData()
+{
+}
+
+
 Recordset::Ref Recordset::create(bec::GRTManager *grtm)
 {
   Ref instance(new Recordset(grtm));
@@ -80,8 +83,10 @@ Recordset::Ref Recordset::create(GrtThreadedTask::Ref parent_task)
 static gint next_id = 0;
 
 Recordset::Recordset(GRTManager *grtm)
-  : VarGridModel(grtm), _show_apply_buttons(false), task(GrtThreadedTask::create(grtm))
+  : VarGridModel(grtm), _inserts_editor(false), task(GrtThreadedTask::create(grtm))
 {
+  _toolbar = NULL;
+  _client_data = NULL;
   _context_menu = 0;
   _id = g_atomic_int_get(&next_id);
   g_atomic_int_inc(&next_id);
@@ -94,13 +99,16 @@ Recordset::Recordset(GRTManager *grtm)
 
 
 Recordset::Recordset(GrtThreadedTask::Ref parent_task)
-  : VarGridModel(parent_task->grtm()), _show_apply_buttons(false), task(GrtThreadedTask::create(parent_task))
+  : VarGridModel(parent_task->grtm()), _inserts_editor(false), task(GrtThreadedTask::create(parent_task))
 {
+  _toolbar = NULL;
+  _client_data = NULL;
   _context_menu = 0;
   _id = g_atomic_int_get(&next_id);
   g_atomic_int_inc(&next_id);
 
   task->send_task_res_msg(false);
+  apply_changes= boost::bind(&Recordset::apply_changes_, this);
   register_default_actions();
   reset();
 }
@@ -108,6 +116,7 @@ Recordset::Recordset(GrtThreadedTask::Ref parent_task)
 
 Recordset::~Recordset()
 {
+  delete _client_data;
   delete _context_menu;
 }
 
@@ -118,7 +127,7 @@ bool Recordset::reset(Recordset_data_storage::Ptr data_storage_ptr, bool rethrow
 
   boost::shared_ptr<sqlite::connection> data_swap_db= this->data_swap_db();
 
-  bool res= true;
+  bool res= false;
 
   _aux_column_count= 0;
   _rowid_column= 0;
@@ -168,8 +177,9 @@ bool Recordset::reset(Recordset_data_storage::Ptr data_storage_ptr, bool rethrow
       _readonly= data_storage->readonly();
 
       _readonly_reason = data_storage->readonly_reason();
+      res = true;
     }
-    CATCH_AND_DISPATCH_EXCEPTION(rethrow, "Reset recordset", res)
+    CATCH_AND_DISPATCH_EXCEPTION(rethrow, "Reset recordset")
   }
 
   refresh_ui_status_bar();
@@ -346,6 +356,7 @@ Recordset::Cell Recordset::cell(RowId row, ColumnId column)
     for (ColumnId col= 0; _column_count > col; ++col, ++new_cell)
       *(new_cell)= sqlite::null_t();
     _data[_data.size() - _column_count + _rowid_column]= (int)rowid;
+
   }
 
   return VarGridModel::cell(row, column);
@@ -363,9 +374,9 @@ void Recordset::after_set_field(const NodeId &node, int column, const sqlite::va
 
 void Recordset::mark_dirty(RowId row, ColumnId column, const sqlite::variant_t &new_value)
 {
-  GStaticRecMutexLock data_mutex(_data_mutex);
+  base::RecMutexLock data_mutex(_data_mutex);
 
-  RowId rowid;
+  RowId rowid(row);
   NodeId node(row);
   if (get_field_(node, _rowid_column, (int&)rowid))
   {
@@ -412,7 +423,7 @@ bool Recordset::delete_node(const bec::NodeId &node)
 bool Recordset::delete_nodes(std::vector<bec::NodeId> &nodes)
 {
   {
-    GStaticRecMutexLock data_mutex(_data_mutex);
+    base::RecMutexLock data_mutex(_data_mutex);
 
     {
       std::sort(nodes.begin(), nodes.end());
@@ -491,6 +502,9 @@ bool Recordset::delete_nodes(std::vector<bec::NodeId> &nodes)
     nodes.clear();
   }
 
+  if (rows_changed)
+    rows_changed();
+
   refresh_ui_status_bar();
 
   return true;
@@ -554,15 +568,13 @@ grt::StringRef Recordset::do_apply_changes(grt::GRT *grt, Ptr self_ptr, Recordse
 {
   RETVAL_IF_FAIL_TO_RETAIN_WEAK_PTR (Recordset, self_ptr, self, grt::StringRef(""))
   RETVAL_IF_FAIL_TO_RETAIN_WEAK_PTR (Recordset_data_storage, data_storage_ptr, data_storage, grt::StringRef(""))
-
-  bool res= true;
   try
   {
     data_storage->apply_changes(self_ptr);
     task->send_msg(grt::InfoMsg, _("Commit complete"), _("Commit recordset changes"));
     reset(data_storage_ptr, false);
   }
-  CATCH_AND_DISPATCH_EXCEPTION(false, "Commit recordset changes", res)
+  CATCH_AND_DISPATCH_EXCEPTION(false, "Commit recordset changes")
 
   return grt::StringRef("");
 }
@@ -620,6 +632,8 @@ void Recordset::rollback_and_gather_messages(std::string &messages)
 int Recordset::on_apply_changes_finished()
 {
   task->finish_cb(GrtThreadedTask::Finish_cb());
+  if (rows_changed)
+    rows_changed();
   refresh_ui_status_bar();
   return refresh_ui();
 }
@@ -746,6 +760,9 @@ std::vector<Recordset_storage_info> Recordset::data_storages_for_export()
 
 void Recordset::sort_by(ColumnId column, int direction, bool retaining)
 {
+  if (_column_count == 0)
+    return;
+
   if (!retaining)
   {
     _sort_columns.clear();
@@ -884,7 +901,7 @@ void Recordset::reset_data_search_string()
 void Recordset::rebuild_data_index(sqlite::connection *data_swap_db, bool do_cache_data_frame, bool do_refresh_ui)
 {
   {
-    GStaticRecMutexLock data_mutex(_data_mutex);
+    base::RecMutexLock data_mutex(_data_mutex);
 
     std::string where_clause;
     {
@@ -952,6 +969,10 @@ void Recordset::rebuild_data_index(sqlite::connection *data_swap_db, bool do_cac
         case DatetimeType:
           column_expr= strfmt("cast(_%u as numeric)", (unsigned int) sort_column.first);
           break;
+        case StringType:
+          column_expr= strfmt("_%u COLLATE NOCASE", (unsigned int) sort_column.first);
+          break;
+
         default:
           column_expr= strfmt("_%u", (unsigned int) sort_column.first);
           break;
@@ -963,7 +984,7 @@ void Recordset::rebuild_data_index(sqlite::connection *data_swap_db, bool do_cac
         case -1: dir= "DESC"; break;
         default: dir= ""; break;
         }
-        orderby_clause+= strfmt("%s %s, ", column_expr.c_str(), dir);
+        orderby_clause += strfmt("%s %s, ", column_expr.c_str(), dir);
       }
       if (!orderby_clause.empty())
       {
@@ -996,7 +1017,7 @@ void Recordset::rebuild_data_index(sqlite::connection *data_swap_db, bool do_cac
 
     recalc_row_count(data_swap_db);
 
-    if (do_cache_data_frame)
+    if (do_cache_data_frame && _column_count > 0)
       cache_data_frame(0, true);
   }
 
@@ -1053,6 +1074,10 @@ void Recordset::paste_rows_from_clipboard(int dest_row)
                                              "Data must be in the same format used by the Copy Row Content command.",
                                              (int)parts.size(), (int)get_column_count()),
                                       "OK");
+
+        if (rows_changed && row != rows.begin())
+          rows_changed();
+
         return;
       }
       size_t i = 0;
@@ -1071,6 +1096,9 @@ void Recordset::paste_rows_from_clipboard(int dest_row)
       dest_row++;
     }
   }
+
+  if (rows_changed)
+    rows_changed();
 }
 
 
@@ -1084,6 +1112,8 @@ mforms::ContextMenu *Recordset::get_context_menu()
 
 void Recordset::update_selection_for_menu(const std::vector<int> &rows, int clicked_column)
 {
+  // TODO: lift the restriction to a single column.
+  //       We need to support multiple cells (in multiple columns) on all platforms.
   _selected_rows = rows;
   _selected_column = clicked_column;
 
@@ -1098,7 +1128,7 @@ void Recordset::update_selection_for_menu(const std::vector<int> &rows, int clic
     item = _context_menu->add_item_with_title(ro ? "Open Value in Viewer" : "Open Value in Editor",
                                               boost::bind(&Recordset::activate_menu_item, this, "edit_cell", rows, clicked_column),
                                               "edit_cell");
-    item->set_enabled((rows.size() > 0) && (clicked_column >= 0));
+    item->set_enabled((rows.size() == 1) && (clicked_column >= 0));
     if (item->get_enabled())
     {
       switch (get_real_column_type(clicked_column))
@@ -1117,13 +1147,17 @@ void Recordset::update_selection_for_menu(const std::vector<int> &rows, int clic
     item = _context_menu->add_item_with_title("Set Field(s) to NULL",
                                        boost::bind(&Recordset::activate_menu_item, this, "set_to_null", rows, clicked_column),
                                        "set_to_null");
+
+    // On Windows we can select individual cells, so it is perfectly ok to allow acting on multiple
+    // cells. The other platforms only select entire rows in the multi-selection case.
+    // So we to have disallow acting on them to avoid changing unrelated entries.
   #ifdef _WIN32
-    item->set_enabled(clicked_column >= 0 && !ro); //XXX why is this?
+    item->set_enabled(clicked_column >= 0 && !ro);
   #else
     item->set_enabled(clicked_column >= 0 && rows.size() == 1 && !ro);
   #endif
 
-    item = _context_menu->add_item_with_title("Mark Field Value as a Function",
+    item = _context_menu->add_item_with_title("Mark Field Value as a Function/Literal",
                                       boost::bind(&Recordset::activate_menu_item, this, "set_to_function", rows, clicked_column),
                                       "set_to_function");
   #ifdef _WIN32
@@ -1192,35 +1226,33 @@ void Recordset::update_selection_for_menu(const std::vector<int> &rows, int clic
 
 void Recordset::activate_menu_item(const std::string &action, const std::vector<int> &rows, int clicked_column)
 {
-  bool result = false;
   bool need_ui_refresh = false;
 
+  // TODO: the tests here for rows count and clicked_column are all unnecessary. This has already be done.
   if (action == "edit_cell")
   {
     if (rows.size() == 1 && clicked_column >= 0)
     {
       open_field_data_editor(rows[0], clicked_column);
-      result = true;
     }
   }
   else if (action == "set_to_null")
   {
-    if (rows.size() == 1 && clicked_column >= 0)
+    for (size_t i = 0; i < rows.size(); ++i)
     {
       bec::NodeId node;
-      node.append(rows[0]);
+      node.append(rows[i]);
       set_field_null(node, clicked_column);
-      result = true;
     }
   }
   else if (action == "set_to_function")
   {
-    if (rows.size() == 1 && clicked_column >= 0)
+    for (size_t i = 0; i < rows.size(); ++i)
     {
       bec::NodeId node;
       Cell cell;
       
-      node.append(rows[0]);
+      node.append(rows[i]);
       std::string function;
       if (!get_cell(cell, node, clicked_column, false))
         function = "";
@@ -1228,7 +1260,6 @@ void Recordset::activate_menu_item(const std::string &action, const std::vector<
         function = boost::apply_visitor(_var_to_str, *cell);
       if (!g_str_has_prefix(function.c_str(), "\\func"))
         set_field(node, clicked_column, std::string("\\func ")+function);
-      result = true;
     }
   }  
   else if (action == "delete_row")
@@ -1242,7 +1273,6 @@ void Recordset::activate_menu_item(const std::string &action, const std::vector<
         nodes.push_back(bec::NodeId(*iter));
       delete_nodes(nodes);
       need_ui_refresh = true;
-      result = true;
     }
   }
   else if (action == "save_to_file")
@@ -1252,7 +1282,6 @@ void Recordset::activate_menu_item(const std::string &action, const std::vector<
       bec::NodeId node;
       node.append(rows[0]);
       save_to_file(node, clicked_column);
-      result = true;
     }
   }
   else if (action == "load_from_file")
@@ -1262,7 +1291,6 @@ void Recordset::activate_menu_item(const std::string &action, const std::vector<
       bec::NodeId node;
       node.append(rows[0]);
       load_from_file(node, clicked_column);
-      result = true;
     }
   }  
   else if (action == "copy_row")
@@ -1270,33 +1298,28 @@ void Recordset::activate_menu_item(const std::string &action, const std::vector<
     if (rows.size() > 0)
     {
       copy_rows_to_clipboard(rows, ", ");
-      result = true;
     }
   }
   else if (action == "copy_row_with_names")
   {
     copy_rows_to_clipboard(rows, ", ", true, true);
-    result = true;
   }
   else if (action == "copy_row_unquoted")
   {
     if (rows.size() > 0)
     {
       copy_rows_to_clipboard(rows, ", ", false);
-      result = true;
     }
   }
   else if (action == "copy_row_unquoted_with_names")
   {
     copy_rows_to_clipboard(rows, ", ", false, true);
-    result = true;
   }
   else if (action == "copy_row_tabsep")
   {
     if (rows.size() > 0)
     {
       copy_rows_to_clipboard(rows, "\t", false);
-      result = true;
     }
   }  
   else if (action == "copy_field")
@@ -1304,7 +1327,6 @@ void Recordset::activate_menu_item(const std::string &action, const std::vector<
     if (rows.size() == 1 && clicked_column >= 0)
     {
       copy_field_to_clipboard(rows[0], clicked_column);
-      result = true;
     }
   }
   else if (action == "copy_field_unquoted")
@@ -1312,13 +1334,12 @@ void Recordset::activate_menu_item(const std::string &action, const std::vector<
     if (rows.size() == 1 && clicked_column >= 0)
     {
       copy_field_to_clipboard(rows[0], clicked_column, false);
-      result = true;
     }
   }
   else if (action == "paste_row")
   {
     paste_rows_from_clipboard(rows.empty() ? -1 : rows[0]);
-    result = true;
+    need_ui_refresh = true;
   }
 
   if (need_ui_refresh)
@@ -1397,45 +1418,13 @@ void Recordset::copy_field_to_clipboard(int row, int column, bool quoted)
   mforms::Utilities::set_clipboard_text(text);
 }
 
-static void add_toolbar_action_item(bec::ToolbarItemList &items, bec::IconManager *im, const std::string &item_icon, const std::string &item_name, const std::string &item_tooltip)
-{
-  bec::ToolbarItem item;
-  item.name = item_name;
-  item.icon = im->get_icon_id(item_icon);
-  item.caption = item_tooltip;
-  item.command = item.name;
-  item.tooltip = item.caption;
-  item.type = bec::ToolbarAction;
-  item.enabled = true;
-  items.push_back(item);
-}
-
-static void add_toolbar_action_item(bec::ToolbarItemList &items, bec::IconManager *im, const std::string &item_name, const std::string &item_tooltip)
-{
-  add_toolbar_action_item(items, im, item_name + ".png", item_name, item_tooltip);
-}
-
-static void add_toolbar_separator_item(bec::ToolbarItemList &items)
-{
-  bec::ToolbarItem item;
-  item.type = bec::ToolbarSeparator;
-  items.push_back(item);
-}
-
-static void add_toolbar_label_item(bec::ToolbarItemList &items, const std::string &label)
-{
-  bec::ToolbarItem item;
-  item.caption = label;
-  item.type = bec::ToolbarLabel;
-  items.push_back(item);
-}
 
 std::string Recordset::status_text()
 {
   std::string limit_text;
 
   if (limit_rows_applicable() && limit_rows())
-    limit_text = ", more available";
+    limit_text = ", row LIMIT active";
   else
     limit_text = "";
   
@@ -1465,62 +1454,130 @@ std::string Recordset::status_text()
   return status_text;
 }
 
-bec::ToolbarItemList Recordset::get_toolbar_items()
+static mforms::ToolBarItem *add_toolbar_action_item(mforms::ToolBar *toolbar, bec::IconManager *im, const std::string &item_icon, const std::string &item_name, const std::string &item_tooltip)
 {
-  bec::ToolbarItemList items;
-  bec::ToolbarItem item;
-  bec::IconManager *im = bec::IconManager::get_instance();
-
-  add_toolbar_label_item(items, "Result Set Filter:");
-  {
-    bec::ToolbarItem item;
-    item.command = "filter";
-    item.type = bec::ToolbarSearch;
-    items.push_back(item);
-  }
-  if (!_data_storage || _data_storage->reloadable())
-    add_toolbar_action_item(items, im, "record_refresh.png", "record_refresh", "Refresh data from data source");
-
-  if (!is_readonly())
-  {
-    add_toolbar_separator_item(items);
-    add_toolbar_label_item(items, "Edit:");
-    add_toolbar_action_item(items, im, "record_edit", "Edit current row");
-    add_toolbar_action_item(items, im, "record_add", "Insert new row");
-    add_toolbar_action_item(items, im, "record_del", "Delete selected rows");
-   
-    if (_show_apply_buttons)
-    {
-      add_toolbar_separator_item(items);
-      add_toolbar_action_item(items, im, "record_save", "Apply changes to data");
-      add_toolbar_action_item(items, im, "record_discard", "Discard changes to data");
-    }
-  }
-  add_toolbar_separator_item(items);
-  if (!is_readonly())
-    add_toolbar_label_item(items, "Export/Import:");
-  else
-    add_toolbar_label_item(items, "Export:");
-  add_toolbar_action_item(items, im, "record_export", "Export recordset to an external file");
-  if (!is_readonly())
-    add_toolbar_action_item(items, im, "record_import", "Import records from an external file");
-
-#ifndef __APPLE__
-  add_toolbar_separator_item(items);
-  add_toolbar_label_item(items, "Wrap Cell Content:");
-  add_toolbar_action_item(items, im, "record_wrap_vertical", "Toggle wrapping of cell contents");
-#endif
-  
-  if (limit_rows_applicable())
-  {
-    add_toolbar_separator_item(items);
-    add_toolbar_label_item(items, "Fetch rows:");
-    add_toolbar_action_item(items, im, "record_fetch_prev.png", "scroll_rows_frame_backward", "Fetch previous frame of records from the data source");
-    add_toolbar_action_item(items, im, "record_fetch_next.png", "scroll_rows_frame_forward", "Fetch next frame of records from the data source");
-  }
-  return items;
+  mforms::ToolBarItem *item = mforms::manage(new mforms::ToolBarItem(mforms::ActionItem));
+  item->set_name(item_name);
+  item->set_icon(im->get_icon_path(item_icon));
+  item->set_tooltip(item_tooltip);
+  toolbar->add_item(item);
+  return item;
 }
 
+static mforms::ToolBarItem *add_toolbar_action_item(mforms::ToolBar *toolbar, bec::IconManager *im, const std::string &item_name, const std::string &item_tooltip)
+{
+  return add_toolbar_action_item(toolbar, im, item_name + ".png", item_name, item_tooltip);
+}
+
+static void add_toolbar_label_item(mforms::ToolBar *toolbar, const std::string &label)
+{
+  mforms::ToolBarItem *item = mforms::manage(new mforms::ToolBarItem(mforms::LabelItem));;
+  item->set_text(label);
+  toolbar->add_item(item);
+}
+
+void Recordset::search_activated(mforms::ToolBarItem *item)
+{
+  std::string text;
+  if ((text = item->get_text()).empty())
+    reset_data_search_string();
+  else
+    set_data_search_string(text);
+}
+
+void Recordset::rebuild_toolbar()
+{
+  if (_toolbar)
+  {
+    _toolbar->remove_all();
+    // hack so that this label only appears for resultset grids and not inserts editor
+    if (!_inserts_editor)
+    {
+      mforms::ToolBarItem *item = mforms::manage(new mforms::ToolBarItem(mforms::TitleItem));;
+      item->set_text("Result Grid");
+      _toolbar->add_item(item);
+      _toolbar->add_separator_item();
+    }
+
+    mforms::ToolBarItem *item;
+    bec::IconManager *im = bec::IconManager::get_instance();
+
+    item = add_toolbar_action_item(_toolbar, im, "record_sort_reset.png", "record_sort_reset", "Resets all sorted columns");
+    item->signal_activated()->connect(boost::bind(&Recordset::sort_by, this, 0, 0, false));
+
+    if (!_data_storage || _data_storage->reloadable())
+    {
+      item = add_toolbar_action_item(_toolbar, im, "record_refresh.png", "record_refresh", "Refresh data re-executing the original query");
+      item->signal_activated()->connect(boost::bind(&Recordset::refresh, this));
+    }
+
+    add_toolbar_label_item(_toolbar, "Filter Rows:");
+
+    item = mforms::manage(new mforms::ToolBarItem(mforms::SearchFieldItem));
+    item->signal_activated()->connect(boost::bind(&Recordset::search_activated, this, _1));
+    _toolbar->add_item(item);
+
+    if (!is_readonly() || _inserts_editor)
+    {
+      _toolbar->add_separator_item();
+      add_toolbar_label_item(_toolbar, "Edit:");
+      add_toolbar_action_item(_toolbar, im, "record_edit", "Edit current row"); // connect in frontend
+      add_toolbar_action_item(_toolbar, im, "record_add", "Insert new row"); // connect in frontend
+      add_toolbar_action_item(_toolbar, im, "record_del", "Delete selected rows"); // connect in frontend
+    }
+    _toolbar->add_separator_item();
+    if (!is_readonly() || _inserts_editor)
+      add_toolbar_label_item(_toolbar, "Export/Import:");
+    else
+      add_toolbar_label_item(_toolbar, "Export:");
+    add_toolbar_action_item(_toolbar, im, "record_export", "Export recordset to an external file");
+    if (!is_readonly() || _inserts_editor)
+      add_toolbar_action_item(_toolbar, im, "record_import", "Import records from an external file");
+    
+#ifndef __APPLE__
+    _toolbar->add_separator_item();
+    add_toolbar_label_item(_toolbar, "Wrap Cell Content:");
+    add_toolbar_action_item(_toolbar, im, "record_wrap_vertical", "Toggle wrapping of cell contents"); // connect in frontend
+#endif
+    
+    if (limit_rows_applicable())
+    {
+      _toolbar->add_separator_item();
+      add_toolbar_label_item(_toolbar, "Fetch rows:");
+      item = add_toolbar_action_item(_toolbar, im, "record_fetch_prev.png", "scroll_rows_frame_backward", "Fetch previous frame of records from the data source");
+      item->signal_activated()->connect(boost::bind(&Recordset::scroll_rows_frame_backward, this));
+      item = add_toolbar_action_item(_toolbar, im, "record_fetch_next.png", "scroll_rows_frame_forward", "Fetch next frame of records from the data source");
+      item->signal_activated()->connect(boost::bind(&Recordset::scroll_rows_frame_forward, this));
+    }
+
+    if (_inserts_editor/* && !is_readonly()*/)
+    {
+      _toolbar->add_separator_item();
+      add_toolbar_label_item(_toolbar, "Apply changes:");
+      item = add_toolbar_action_item(_toolbar, im, "record_save", "Apply changes to data");
+      item->signal_activated()->connect(boost::bind(&Recordset::call_apply_changes, this));
+      item = add_toolbar_action_item(_toolbar, im, "record_discard", "Discard changes to data");
+      item->signal_activated()->connect(boost::bind(&Recordset::rollback, this));
+    }
+  }
+}
+
+mforms::ToolBar *Recordset::get_toolbar()
+{
+  if (!_toolbar)
+  {
+    _toolbar = mforms::manage(new mforms::ToolBar(mforms::SecondaryToolBar));
+    rebuild_toolbar();
+  }
+  
+  return _toolbar;
+}
+
+
+void Recordset::call_apply_changes()
+{
+  apply_changes();
+}
 
 ActionList & Recordset::action_list()
 {
@@ -1583,9 +1640,8 @@ public:
 };
 void Recordset::open_field_data_editor(RowId row, ColumnId column)
 {
-  GStaticRecMutexLock data_mutex(_data_mutex);
+  base::RecMutexLock data_mutex(_data_mutex);
 
-  bool res= true;
   try
   {
     sqlite::variant_t blob_value;
@@ -1622,7 +1678,7 @@ void Recordset::open_field_data_editor(RowId row, ColumnId column)
     data_editor->signal_saved.connect(boost::bind(&Recordset::set_field_value,this, row, column, data_editor));
     data_editor->show(true);
   }
-  CATCH_AND_DISPATCH_EXCEPTION(false, "Open field editor", res)
+  CATCH_AND_DISPATCH_EXCEPTION(false, "Open field editor")
 }
 
 
@@ -1718,7 +1774,7 @@ public:
 
 void Recordset::save_to_file(const bec::NodeId &node, int column, const std::string &file)
 {
-  GStaticRecMutexLock data_mutex(_data_mutex);
+  base::RecMutexLock data_mutex(_data_mutex);
       
   sqlite::variant_t blob_value;
   sqlite::variant_t *value;
@@ -1727,8 +1783,8 @@ void Recordset::save_to_file(const bec::NodeId &node, int column, const std::str
   {
     if (!_data_storage)
       return;
-    RowId rowid;
-    if (!get_field_(node, _rowid_column, (int&)rowid))
+    int rowid;
+    if (!get_field_(node, _rowid_column, rowid))
       return;
     boost::shared_ptr<sqlite::connection> data_swap_db= this->data_swap_db();
     _data_storage->fetch_blob_value(this, data_swap_db.get(), rowid, column, blob_value);
@@ -1758,11 +1814,10 @@ void Recordset::save_to_file(const bec::NodeId &node, int column)
   
   if (chooser.run_modal())
   {
-    bool res = false;
     try
     {      
       save_to_file(node, column, chooser.get_path());
     }
-    CATCH_AND_DISPATCH_EXCEPTION(false, "Save field to file", res)
+    CATCH_AND_DISPATCH_EXCEPTION(false, "Save field to file")
   }
 }
