@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,10 +17,8 @@
  * 02110-1301  USA
  */
 
-#ifndef __WB_H__
-#define __WB_H__
+#pragma once
 
-#include "workbench/wb_context.h"
 #include "workbench/wb_context_ui.h"
 #include "model/wb_model_diagram_form.h"
 #include "model/wb_context_model.h"
@@ -38,30 +36,15 @@
 #include "Overview.h"
 #include "ModelDiagramFormWrapper.h"
 
-// XXX: What is this supposed to fix or help with?
-#if WBContext_VERSION > 5
-# error outdated wrapper Wb.h
-#endif
-
-using namespace System;
-using namespace System::Collections::Generic;
-using namespace System::Runtime::InteropServices;
-
-using namespace Aga::Controls::Tree;
-
-using namespace MySQL::Base;
-using namespace MySQL::GUI::Mdc;
-using namespace MySQL::Forms;
-
 namespace MySQL {
 namespace Workbench {
 
 public enum class Msg_type
 {
-  MT_warning=  grt::WarningMsg,
-  MT_error=    grt::ErrorMsg,
-  MT_info=     grt::InfoMsg,
-  MT_progress= grt::ProgressMsg,
+  MT_warning  = grt::WarningMsg,
+  MT_error    = grt::ErrorMsg,
+  MT_info     = grt::InfoMsg,
+  MT_progress = grt::ProgressMsg,
 };
 
 //--------------------------------------------------------------------------------------------------
@@ -163,13 +146,12 @@ public:
 
 public ref class WbContext
 {
-  ::wb::WBContextUI *inner;
+  wb::WBContextUI *inner;
 
-  ::MySQL::Grt::GrtManager^ manager;
-  ::MySQL::Workbench::Overview^ physical_overview;
-  ::MySQL::Grt::GrtValueTree^ catalog_tree;
+  MySQL::Grt::GrtManager ^manager;
+  MySQL::Workbench::Overview ^physical_overview;
 
-  ::System::Collections::ArrayList open_editor_slot_wrappers;
+  System::Collections::ArrayList open_editor_slot_wrappers;
 
   DelegateSlot0<void,void> ^undo_delegate;
   DelegateSlot0<bool,bool> ^can_undo_delegate;
@@ -191,7 +173,7 @@ public ref class WbContext
   DelegateSlot0<bool,bool> ^can_find_replace_delegate;
 
 public:
-  delegate void VoidStrUIFormDelegate(String^ str1, UIForm^ form);
+  delegate void VoidStrUIFormDelegate(String^ str1, MySQL::Base::UIForm^ form);
 
 private:
   // void (string, bec::UIFrom)
@@ -209,87 +191,30 @@ private:
   void create_main_form_view_wrapper(const std::string& view_name, boost::shared_ptr<bec::UIForm> form_be);
 
 public:
-  WbContext(bool verbose)
-  {
-    inner = new ::wb::WBContextUI(verbose);
-    manager = nullptr;
-    physical_overview = nullptr;
-    catalog_tree = nullptr;
-
-    Logger::LogDebug("WBContext managed", 1, "Creating WbContext\n");
-  }
-
-  ~WbContext()
-  {
-    delete catalog_tree;
-    delete physical_overview;
-    delete inner;
-    delete manager;
-
-    Logger::LogDebug("WBContext managed", 1, "Destroying WbContext\n");
-  }
+  WbContext(bool verbose);
+  ~WbContext();
 
   bool is_commercial() { return inner->get_wb()->is_commercial(); };
-  ::wb::WBContextUI *get_unmanaged_object() 
-  { return inner; }
+  wb::WBContextUI *get_unmanaged_object() { return inner; }
 
   bool init(MySQL::Workbench::WbFrontendCallbacks^ callbacks, WbOptions^ options,
-    VoidStrUIFormDelegate^ create_main_form_view)
-  {
-    set_create_main_form_view(callbacks, create_main_form_view);
-    return inner->init(
-      callbacks->get_callbacks(), 
-      options->get_unmanaged_object());
-  }
+    VoidStrUIFormDelegate^ create_main_form_view);
 
-  bool opengl_rendering_enforced()
-  {
-    return inner->get_wb()->opengl_rendering_enforced();
-  }
+  bool opengl_rendering_enforced() { return inner->get_wb()->opengl_rendering_enforced(); }
+  bool software_rendering_enforced() { return inner->get_wb()->software_rendering_enforced(); }
+  bool is_busy() { return inner->get_wb()->get_grt_manager()->get_dispatcher()->get_busy(); }
+  bool request_quit() { return inner->request_quit(); }
+  void perform_quit() { inner->perform_quit(); }
+  bool is_quitting() { return inner->is_quitting(); }
+  void finalize() { inner->finalize(); }
 
-  bool software_rendering_enforced()
-  {
-    return inner->get_wb()->software_rendering_enforced();
-  }
+  GrtManager^ get_grt_manager();
 
-  bool is_busy()
-  {
-    return inner->get_wb()->get_grt_manager()->get_dispatcher()->get_busy();
-  }
-
-  bool request_quit()
-  {
-    return inner->request_quit();
-  }
-
-  void perform_quit()
-  {
-    inner->perform_quit();
-  }
-
-  bool is_quitting()
-  {
-    return inner->is_quitting();
-  }
-
-  void finalize()
-  {
-    inner->finalize();
-  }
-
-  GrtManager^ get_grt_manager()
-  {
-    if (manager == nullptr)
-      manager= gcnew GrtManager(inner->get_wb()->get_grt_manager());
-
-    return manager; 
-  }
-
-  MenuStrip^ menu_for_form(MySQL::Base::UIForm^ form);
-  MenuStrip^ menu_for_appview(MySQL::Forms::AppViewDockContent ^content);
+  Windows::Forms::MenuStrip^ menu_for_form(MySQL::Base::UIForm^ form);
+  Windows::Forms::MenuStrip^ menu_for_appview(MySQL::Forms::AppViewDockContent ^content);
   void validate_menu_for_form(MySQL::Base::UIForm^ form);
-  ToolStrip^ toolbar_for_form(MySQL::Base::UIForm^ form);
-  Control ^shared_secondary_sidebar();
+  Windows::Forms::ToolStrip^ toolbar_for_form(MySQL::Base::UIForm^ form);
+  Windows::Forms::Control ^shared_secondary_sidebar();
 
   void focus_search_box(MySQL::Base::UIForm^ form);
   String^ get_search_string(MySQL::Base::UIForm^ form);
@@ -367,8 +292,8 @@ public:
 
   bool edit_can_paste()
   {
-    if (inner->get_active_form() && inner->get_active_form()->can_paste())
-      return true;
+    if (inner->get_active_form())
+      return inner->get_active_form()->can_paste();
     return false;
   }
 
@@ -489,16 +414,16 @@ public:
 
   // Overview.
   Overview^ get_physical_overview();
-  ::MySQL::Grt::GrtValueTree^ get_catalog_tree();
-  TreeViewAdv^ get_history_tree();
-  TreeViewAdv^ get_usertypes_tree();
-  ::MySQL::Grt::GrtValueInspector^ get_inspector_for_selection(UIForm^ form, [Out] List<String^>^ %items);
+  Aga::Controls::Tree::TreeViewAdv^ get_catalog_tree();
+  Aga::Controls::Tree::TreeViewAdv^ get_history_tree();
+  Aga::Controls::Tree::TreeViewAdv^ get_usertypes_tree();
+  MySQL::Grt::GrtValueInspector^ get_inspector_for_selection(MySQL::Base::UIForm ^form, [Out] List<String^>^ %items);
   bool are_lists_equal(GrtValue^ v1, GrtValue^ v2);
-  String^ get_description_for_selection(UIForm^ form, [Out] GrtValue^ %activeObjList, [Out] List<String^>^ %items);
+  String^ get_description_for_selection(MySQL::Base::UIForm ^form, [Out] GrtValue^ %activeObjList, [Out] List<String^>^ %items);
   String^ get_description_for_selection([Out] GrtValue^ %activeObjList, [Out] List<String^>^ %items);
   void set_description_for_selection(GrtValue^ activeObjList, String^ val);
   ::MySQL::Workbench::ModelDiagramFormWrapper^ get_diagram_form_for_diagram(String^ id);
-  void set_active_form(UIForm^ uiform);
+  void set_active_form(MySQL::Base::UIForm ^uiform);
   void set_active_form_from_appview(MySQL::Forms::AppViewDockContent^ form);
   String ^get_active_context();
   void show_output();
@@ -641,5 +566,3 @@ public:
 
 } // namespace Workbench
 } // namespace MySQL
-
-#endif // __WB_H__
