@@ -45,10 +45,12 @@ namespace Aga.Controls.Tree
 		private AbortableThreadPool _threadPool = new AbortableThreadPool();
 
 		#region Public Events
+    
+    // ml: Made all events protected and virtual.
 
 		[Category("Action")]
 		public event ItemDragEventHandler ItemDrag;
-		private void OnItemDrag(MouseButtons buttons, object item)
+    protected virtual void OnItemDrag(MouseButtons buttons, object item)
 		{
 			if (ItemDrag != null)
 				ItemDrag(this, new ItemDragEventArgs(buttons, item));
@@ -56,7 +58,7 @@ namespace Aga.Controls.Tree
 
 		[Category("Behavior")]
 		public event EventHandler<TreeNodeAdvMouseEventArgs> NodeMouseDoubleClick;
-		private void OnNodeMouseDoubleClick(TreeNodeAdvMouseEventArgs args)
+    protected virtual void OnNodeMouseDoubleClick(TreeNodeAdvMouseEventArgs args)
 		{
 			if (NodeMouseDoubleClick != null)
 				NodeMouseDoubleClick(this, args);
@@ -64,7 +66,7 @@ namespace Aga.Controls.Tree
 
 		[Category("Behavior")]
 		public event EventHandler<TreeColumnEventArgs> ColumnWidthChanged;
-		internal void OnColumnWidthChanged(TreeColumn column)
+		protected internal virtual void OnColumnWidthChanged(TreeColumn column)
 		{
 			if (ColumnWidthChanged != null)
 				ColumnWidthChanged(this, new TreeColumnEventArgs(column));
@@ -72,7 +74,7 @@ namespace Aga.Controls.Tree
 
 		[Category("Behavior")]
 		public event EventHandler<TreeColumnEventArgs> ColumnReordered;
-		internal void OnColumnReordered(TreeColumn column)
+		protected internal virtual void OnColumnReordered(TreeColumn column)
 		{
 			if (ColumnReordered != null)
 				ColumnReordered(this, new TreeColumnEventArgs(column));
@@ -80,7 +82,7 @@ namespace Aga.Controls.Tree
 
 		[Category("Behavior")]
 		public event EventHandler<TreeColumnEventArgs> ColumnClicked;
-		internal void OnColumnClicked(TreeColumn column)
+		protected internal virtual void OnColumnClicked(TreeColumn column)
 		{
 			if (ColumnClicked != null)
 				ColumnClicked(this, new TreeColumnEventArgs(column));
@@ -88,7 +90,7 @@ namespace Aga.Controls.Tree
 
 		[Category("Behavior")]
 		public event EventHandler SelectionChanged;
-		internal void OnSelectionChanged()
+		protected internal virtual void OnSelectionChanged()
 		{
 			if (SuspendSelectionEvent)
 				_fireSelectionEvent = true;
@@ -102,7 +104,7 @@ namespace Aga.Controls.Tree
 
 		[Category("Behavior")]
 		public event EventHandler<TreeViewAdvEventArgs> Collapsing;
-		private void OnCollapsing(TreeNodeAdv node)
+    protected virtual void OnCollapsing(TreeNodeAdv node)
 		{
 			if (Collapsing != null)
 				Collapsing(this, new TreeViewAdvEventArgs(node));
@@ -110,7 +112,7 @@ namespace Aga.Controls.Tree
 
 		[Category("Behavior")]
 		public event EventHandler<TreeViewAdvEventArgs> Collapsed;
-		private void OnCollapsed(TreeNodeAdv node)
+    protected virtual void OnCollapsed(TreeNodeAdv node)
 		{
 			if (Collapsed != null)
 				Collapsed(this, new TreeViewAdvEventArgs(node));
@@ -118,7 +120,7 @@ namespace Aga.Controls.Tree
 
 		[Category("Behavior")]
 		public event EventHandler<TreeViewAdvEventArgs> Expanding;
-		private void OnExpanding(TreeNodeAdv node)
+    protected virtual void OnExpanding(TreeNodeAdv node)
 		{
 			if (Expanding != null)
 				Expanding(this, new TreeViewAdvEventArgs(node));
@@ -126,23 +128,23 @@ namespace Aga.Controls.Tree
 
 		[Category("Behavior")]
 		public event EventHandler<TreeViewAdvEventArgs> Expanded;
-		private void OnExpanded(TreeNodeAdv node)
+    protected virtual void OnExpanded(TreeNodeAdv node)
 		{
 			if (Expanded != null)
 				Expanded(this, new TreeViewAdvEventArgs(node));
 		}
 
-        [Category("Behavior")]
-        public event EventHandler GridLineStyleChanged;
-		private void OnGridLineStyleChanged()
-        {
-			if (GridLineStyleChanged != null)
-				GridLineStyleChanged(this, EventArgs.Empty);
-        }
+    [Category("Behavior")]
+    public event EventHandler GridLineStyleChanged;
+    protected virtual void OnGridLineStyleChanged()
+    {
+      if (GridLineStyleChanged != null)
+        GridLineStyleChanged(this, EventArgs.Empty);
+    }
 
     [Category("Drawing")]
     public event EventHandler<TreeViewAdvDrawRowEventArgs> BeforeDrawNode;
-    private void OnBeforeNodeDrawing(TreeNodeAdv node, DrawContext context)
+    protected virtual void OnBeforeNodeDrawing(TreeNodeAdv node, DrawContext context)
     {
       if (BeforeDrawNode != null)
         BeforeDrawNode(this, new TreeViewAdvDrawRowEventArgs(node, context));
