@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -49,7 +49,6 @@
 #include "linux_utilities/gtk_helpers.h"
 #include "linux_utilities/image_cache.h"
 
-#include "linux_utilities/toolbar_manager.h"
 
 #include "base/string_utilities.h"
 #include "base/geometry.h"
@@ -70,6 +69,7 @@ static void set_window_icons(Gtk::Window *window)
 
   icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbench-16.png", false));
   icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbench-32.png", false));
+  icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbench-48.png", false));
   icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbench-128.png", false));
 
   window->set_default_icon_list(icons);
@@ -146,6 +146,12 @@ MainForm::MainForm(wb::WBContextUI* ctx)
 //------------------------------------------------------------------------------
 MainForm::~MainForm()
 {
+
+  _sig_flush_idle.disconnect();
+  _sig_set_current_page.disconnect();
+  _sig_close_tab.disconnect();
+  _sig_change_status.disconnect();
+
   base::NotificationCenter::get()->remove_observer(this);
 
   const size_t n_slots = _slots.size();
@@ -154,12 +160,10 @@ MainForm::~MainForm()
   _slots.clear();
 
 
-  for (std::map<Gtk::Widget*,mforms::Menu*>::iterator it = _menu_list.begin(); it != _menu_list.end(); ++it)
-  {
-    if (it->second)
-      delete it->second;
-  }
-  _menu_list.clear();
+  std::map<mdc::CanvasView*,ModelDiagramPanel*>::iterator it;
+  for (it = _diagram_panel_list.begin(); it != _diagram_panel_list.end(); it++)
+    delete it->second;
+  _diagram_panel_list.clear();
 
   notify_callbacks(); // disconnect all signals
 
@@ -340,7 +344,7 @@ void MainForm::show_status_text_becb(const std::string& text)
     change_status(status, text);
   else
     // execute when idle in case we're being called from the worker thread
-    Glib::signal_idle().connect(sigc::bind<Gtk::Statusbar*,std::string>(sigc::ptr_fun(change_status), status, text));
+    _sig_change_status = Glib::signal_idle().connect(sigc::bind<Gtk::Statusbar*,std::string>(sigc::ptr_fun(change_status), status, text));
 }
 
 
@@ -523,7 +527,7 @@ void MainForm::call_find()
     mforms::ToolBar *toolbar = form->get_form()->get_toolbar();
     if (toolbar)
     {
-      Gtk::Entry *entry = dynamic_cast<Gtk::Entry*>(mforms::gtk::widget_for_toolbar_item_named(toolbar, "find"));
+      Gtk::Entry *entry = dynamic_cast<Gtk::Entry*>(mforms::widget_for_toolbar_item_named(toolbar, "find"));
       if (entry)
       {
         entry->grab_focus();
@@ -552,8 +556,6 @@ void MainForm::call_find_replace()
 void MainForm::call_undo()
 {
     mforms::CodeEditor *editor = get_focused_code_editor(get_mainwindow());
-    Gtk::Widget *focused= get_mainwindow()->get_focus();
-
     if (editor)
     {
         editor->undo();
@@ -568,8 +570,6 @@ void MainForm::call_undo()
 void MainForm::call_redo()
 {
   mforms::CodeEditor *editor = get_focused_code_editor(get_mainwindow());
-  Gtk::Widget *focused= get_mainwindow()->get_focus();
-
   if (editor)
   {
     editor->redo();
@@ -966,8 +966,10 @@ mdc::CanvasView* MainForm::create_view_becb(const model_DiagramRef &diagram)
 
   add_form_pane(model_panel, tabState);
  // model_panel->set_data("model_panel", model_panel);
+  mdc::CanvasView* view = model_panel->get_canvas();
+  _diagram_panel_list.insert(std::pair<mdc::CanvasView*, ModelDiagramPanel*>(view, model_panel));
 
-  return model_panel->get_canvas();
+  return view;
 }
 
 
@@ -991,6 +993,15 @@ void MainForm::destroy_view_becb(mdc::CanvasView* view)
       break;
     }
   }
+
+  std::map<mdc::CanvasView*,ModelDiagramPanel*>::iterator it;
+  it = _diagram_panel_list.find(view);
+  if (it != _diagram_panel_list.end())
+  {
+    delete it->second;
+    _diagram_panel_list.erase(it);
+  }
+
 }
 
 //------------------------------------------------------------------------------
@@ -1077,7 +1088,6 @@ void MainForm::handle_notification(const std::string &name, void *sender, base::
 const char* RefreshTypeStr[] = {
   "RefreshNeeded",
     "RefreshNothing",
-    "RefreshSchemaList",
     "RefreshSchemaNoReload",
     "RefreshNewDiagram",
     "RefreshSelection",
@@ -1106,7 +1116,7 @@ void MainForm::refresh_gui_becb(wb::RefreshType type, const std::string& arg_id,
   {
   case wb::RefreshNeeded:
     {
-      Glib::signal_idle().connect(sigc::bind_return(sigc::mem_fun(_wbui_context->get_wb(), &wb::WBContext::flush_idle_tasks), false));
+      _sig_flush_idle = Glib::signal_idle().connect(sigc::bind_return(sigc::mem_fun(_wbui_context->get_wb(), &wb::WBContext::flush_idle_tasks), false));
       break;
     }
 
@@ -1123,14 +1133,6 @@ void MainForm::refresh_gui_becb(wb::RefreshType type, const std::string& arg_id,
   case wb::RefreshNothing:
     break;
 
-  case wb::RefreshSchemaList:
-    {
-      ModelDiagramPanel *panel = dynamic_cast<ModelDiagramPanel*>(get_active_pane());
-      if (panel)
-        panel->refresh_catalog(true);
-      break;
-    }
-
   case wb::RefreshSchemaNoReload:
     {
       ModelDiagramPanel *panel = dynamic_cast<ModelDiagramPanel*>(get_active_pane());
@@ -1362,8 +1364,8 @@ void MainForm::bring_plugin_pane(PluginEditorBase *pane)
   if (page)
   {
     FormViewBase *form = reinterpret_cast<FormViewBase*>(page->get_data("FormViewBase"));
-    
-    if (form && old_form != form)
+
+    if (form && old_form != form && old_form->get_form()->get_form_context_name() != WB_CONTEXT_QUERY)
     {
       pane->reference();
       if (old_form)
@@ -1414,7 +1416,6 @@ void MainForm::close_main_tab()
     if (curpagenum >= 0)
     {
       Gtk::Widget *page = upper->get_nth_page(curpagenum);
-      FormViewBase *panel = reinterpret_cast<FormViewBase*>(page->get_data("FormViewBase"));
       bec::UIForm *form = reinterpret_cast<bec::UIForm*>(page->get_data("uiform"));
       if (form && form->get_form_context_name() == "home")
         return;
@@ -1509,13 +1510,7 @@ bool MainForm::close_tab(Gtk::Notebook *note, Gtk::Widget *page)
       {
         if (form)
           form->dispose();
-        std::map<Gtk::Widget*,mforms::Menu*>::iterator it;
-        it = _menu_list.find(page);
-        if (it != _menu_list.end())
-        {
-          delete it->second;
-          _menu_list.erase(it);
-        }
+
         note->remove_page(*page);
         page->unreference();
 
@@ -1571,7 +1566,7 @@ void MainForm::append_tab_page(Gtk::Notebook *note, Gtk::Widget *widget, const s
       *title_label_ret = label;
 
     mforms::Menu* menu = init_tab_menu(widget);
-    label->set_menu(menu);
+    label->set_menu(menu, true);
     menu->set_handler(sigc::bind(sigc::mem_fun(this, &MainForm::tab_menu_handler), label, widget, note));
 
     if (g_object_is_floating(G_OBJECT(widget->gobj())))
@@ -1581,7 +1576,7 @@ void MainForm::append_tab_page(Gtk::Notebook *note, Gtk::Widget *widget, const s
     if (tabState == TabOpenActive)
     {
       // switch the current page when we're in idle
-      Glib::signal_idle().connect(sigc::bind_return(sigc::bind(sigc::mem_fun(note, &Gtk::Notebook::set_current_page),
+      _sig_set_current_page = Glib::signal_idle().connect(sigc::bind_return(sigc::bind(sigc::mem_fun(note, &Gtk::Notebook::set_current_page),
                                                                note->get_n_pages()-1), false));
     }
     if (tabState != TabClosed)
@@ -1592,18 +1587,13 @@ void MainForm::append_tab_page(Gtk::Notebook *note, Gtk::Widget *widget, const s
 //------------------------------------------------------------------------------
 mforms::Menu* MainForm::init_tab_menu(Gtk::Widget *widget)
 {
-  std::map<Gtk::Widget*,mforms::Menu*>::iterator it;
-  it = _menu_list.find(widget);
-  if (it == _menu_list.end())
   {
     mforms::Menu *m = new mforms::Menu();
     m->add_item("Close Tab", "close tab");
     m->add_item("Close Other Tabs", "close other tabs");
     m->add_item("Close Other Tabs of This Type", "close similar");
-    _menu_list.insert(std::pair<Gtk::Widget*,mforms::Menu*>(widget,m));
     return m;
   }
-  return it->second;
 }
 
 //------------------------------------------------------------------------------
@@ -1614,7 +1604,7 @@ void MainForm::tab_menu_handler(const std::string& action, ActiveLabel* label, G
   {
     if (action == "close tab")
     {
-      Glib::signal_idle().connect(sigc::bind_return(sigc::bind(sigc::mem_fun(this, &MainForm::close_tab), note, widget), false));
+      _sig_close_tab = Glib::signal_idle().connect(sigc::bind_return(sigc::bind(sigc::mem_fun(this, &MainForm::close_tab), note, widget), false));
     }
     else if (action == "close other tabs")
     {
@@ -1779,7 +1769,13 @@ static std::string get_executable_path(mforms::App* app, const std::string& file
   const char *basedir = getenv("MWB_BASE_DIR");
   if (basedir)
   {
-    char *p = g_strdup_printf("%s/bin/%s", basedir, file.c_str());
+    char *p = g_strdup_printf("%s/libexec/mysql-workbench/%s", basedir, file.c_str());
+    path = p;
+    g_free(p);
+    if (g_file_test(path.c_str(), G_FILE_TEST_EXISTS))
+      return path;    
+    
+    p = g_strdup_printf("%s/bin/%s", basedir, file.c_str());
     path = p;
     g_free(p);
     if (g_file_test(path.c_str(), G_FILE_TEST_EXISTS))
@@ -1872,6 +1868,8 @@ static base::Color get_system_color(mforms::SystemColor type)
       }
       break;
     }
+    case mforms::SystemColorContainer:
+      break;
   }
 
   return ret;
@@ -1992,7 +1990,7 @@ Gtk::Widget *MainForm::decorate_widget(Gtk::Widget *panel, bec::UIForm *form)
   Gtk::VBox *top_box = Gtk::manage(new Gtk::VBox(false, 0));
   if (menu)
   {
-    Gtk::Widget *w = mforms::gtk::widget_for_menubar(menu);
+    Gtk::Widget *w = mforms::widget_for_menubar(menu);
     w->set_name(form->get_form_context_name());
     top_box->pack_start(*w, false, true);
     w->show();
@@ -2000,7 +1998,7 @@ Gtk::Widget *MainForm::decorate_widget(Gtk::Widget *panel, bec::UIForm *form)
 
   if (toolbar)
   {
-    Gtk::Widget *w = mforms::gtk::widget_for_toolbar(toolbar);
+    Gtk::Widget *w = mforms::widget_for_toolbar(toolbar);
     top_box->pack_start(*w, false, true);
     w->show();
   }
@@ -2020,17 +2018,18 @@ static gpointer delete_appview(mforms::AppView *appview)
   return 0;
 }
 
-void MainForm::get_size(int *w, int *h)
+std::pair<int, int> MainForm::get_size()
 {
   Gtk::Widget *note = get_upper_note();
-  *w = note->get_width();
-  *h = note->get_height();
+  int w = note->get_width();
+  int h = note->get_height();
+  return std::pair<int, int>(w, h);
 }
 
 void MainForm::dock_view(mforms::AppView *view, const std::string &position, int)
 {
   g_return_if_fail(view != NULL);
-  Gtk::Widget *w= mforms::gtk::ViewImpl::get_widget_for_view(view);
+  Gtk::Widget *w= mforms::widget_for_view(view);
   g_return_if_fail(w != NULL);
 
   if (position == "maintab" || position.empty())
@@ -2097,7 +2096,7 @@ bool MainForm::select_view(mforms::AppView *view)
   for (int i= 0; i < upper_note->get_n_pages(); i++)
   {
     Gtk::Widget *page = upper_note->get_nth_page(i);
-    if (page == mforms::widget_for_view(view))
+    if (page && reinterpret_cast<mforms::AppView*>(page->get_data("AppView")) == view)
     {
       upper_note->set_current_page(i);
       return true;
@@ -2110,7 +2109,7 @@ bool MainForm::select_view(mforms::AppView *view)
 void MainForm::undock_view(mforms::AppView *view)
 {
   g_return_if_fail(view != NULL);
-  Gtk::Widget *w= mforms::gtk::ViewImpl::get_widget_for_view(view);
+  Gtk::Widget *w= mforms::widget_for_view(view);
   g_return_if_fail(w != NULL);
   
   Gtk::Notebook *note= reinterpret_cast<Gtk::Notebook*>(w->get_data("containerNote"));
@@ -2125,7 +2124,7 @@ void MainForm::undock_view(mforms::AppView *view)
 void MainForm::set_view_title(mforms::AppView *view, const std::string &title)
 {
   g_return_if_fail(view != NULL);
-  Gtk::Widget *w= mforms::gtk::ViewImpl::get_widget_for_view(view);
+  Gtk::Widget *w= mforms::widget_for_view(view);
   g_return_if_fail(w != NULL);
 
   ActiveLabel *label= reinterpret_cast<ActiveLabel*>(w->get_data("tablabel"));
