@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -184,9 +184,8 @@ static NativeHandle windowOpenPlugin(bec::GRTManager *grtm,
     {
       NSLog(@"plugin bundle %@ for plugin %s could not be loaded", bundlePath, path.c_str());
       NSRunAlertPanel(NSLocalizedString(@"Error Opening Plugin", @"plugin open error"),
-                      [NSString stringWithFormat:NSLocalizedString(@"The plugin %s could not be loaded.", @"plugin open error"),
-                       shlib.c_str()],
-                      NSLocalizedString(@"OK", @"dialog window"), nil, nil);
+                      NSLocalizedString(@"The plugin %s could not be loaded.", @"plugin open error"),
+                      nil, nil, nil, shlib.c_str());
       return 0;
     }
     /* debug msg
@@ -201,9 +200,8 @@ static NativeHandle windowOpenPlugin(bec::GRTManager *grtm,
       NSLog(@"plugin class %s was not found in bundle %@", class_name.c_str(), bundlePath);
       
       NSRunAlertPanel(NSLocalizedString(@"Error Opening Plugin", @"plugin open error"),
-                      [NSString stringWithFormat:@"The plugin %s does not contain the published object %s",
-                       shlib.c_str(), class_name.c_str()],
-                      NSLocalizedString(@"OK", @"dialog window"), nil, nil);
+                      @"The plugin %s does not contain the published object %s",
+                      nil, nil, nil, shlib.c_str(), class_name.c_str());
       return 0;
     }
     
@@ -627,9 +625,8 @@ static bool validate_toggle_fullscreen(WBMainWindow *mainWindow)
 static bool validate_find_replace()
 {
   id firstResponder = [[NSApp keyWindow] firstResponder];
-  
   if ([firstResponder isKindOfClass: [ScintillaView class]] || [firstResponder isKindOfClass: [NSTextView class]] ||
-      ([firstResponder isKindOfClass: [NSView class]] && [[firstResponder superview] isKindOfClass: [ScintillaView class]]))
+      [firstResponder isKindOfClass: [SCIContentView class]])
     return true;
   return false;
 }
@@ -641,9 +638,11 @@ static void call_find_replace(bool do_replace)
   if (validate_find_replace())
   {
     id firstResponder = [[NSApp keyWindow] firstResponder];
-    if ([firstResponder isKindOfClass: [InnerView class]])
-      firstResponder = [firstResponder superview];
-    
+    if ([firstResponder isKindOfClass: [SCIContentView class]])
+    {
+      while (firstResponder && ![firstResponder isKindOfClass: [ScintillaView class]])
+        firstResponder = [firstResponder superview];
+    }
     if ([firstResponder isKindOfClass: [MFCodeEditor class]])
       [firstResponder showFindPanel: do_replace];
   }
@@ -656,9 +655,11 @@ static void call_find_replace(bool do_replace)
 static void call_find(WBMainWindow *mainWindow)
 {
   id firstResponder = [[NSApp keyWindow] firstResponder];
-  if ([firstResponder isKindOfClass: [InnerView class]])
-    firstResponder = [firstResponder superview];
-
+  if ([firstResponder isKindOfClass: [SCIContentView class]])
+  {
+    while (firstResponder && ![firstResponder isKindOfClass: [ScintillaView class]])
+      firstResponder = [firstResponder superview];
+  }
   if ([firstResponder isKindOfClass: [MFCodeEditor class]])
     [firstResponder showFindPanel: NO];
   else
@@ -698,7 +699,7 @@ static bool validate_undo(WBMainWindow *mainWindow)
     return [firstResponder canUndo];
   else if ([firstResponder isKindOfClass: [NSTextView class]])
     return [[firstResponder undoManager] canUndo];
-  else if ([firstResponder isKindOfClass: [InnerView class]])
+  else if ([firstResponder isKindOfClass: [SCIContentView class]])
     return true;
   else
     if (wbui->get_active_main_form())
@@ -729,7 +730,7 @@ static bool validate_redo(WBMainWindow *mainWindow)
     return [firstResponder canRedo];
   else if ([firstResponder isKindOfClass: [NSTextView class]])
     return [[firstResponder undoManager] canRedo];
-  else if ([firstResponder isKindOfClass: [InnerView class]])
+  else if ([firstResponder isKindOfClass: [SCIContentView class]])
     return true;
   else
     if (wbui->get_active_main_form())
@@ -1052,6 +1053,7 @@ static void init_mforms()
 {
   log_info("Shutting down Workbench\n");
 
+  [NSObject cancelPreviousPerformRequestsWithTarget: self];
   [[NSNotificationCenter defaultCenter] removeObserver: self];
   
   [_editorWindows release];
@@ -1141,9 +1143,8 @@ static void init_mforms()
     }
     catch (const std::exception &exc)
     {
-      NSRunAlertPanel(@"Unhandled Exception",
-                      [NSString stringWithFormat: @"An unhandled exception has occurred: %s", exc.what()],
-                      @"OK", nil, nil);
+      NSRunAlertPanel(@"Unhandled Exception", @"An unhandled exception has occurred: %s",
+                      @"OK", nil, nil, exc.what());
     }
     _showingUnhandledException = NO;
   }
