@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -276,7 +276,6 @@ void setup_mforms_app(WBMainWindow *mwin);
   const char* RefreshTypeStr[] = {
     "RefreshNeeded",
     "RefreshNothing",
-    "RefreshSchemaList",
     "RefreshSchemaNoReload",
     "RefreshNewView",
     "RefreshSelection",
@@ -367,14 +366,6 @@ void setup_mforms_app(WBMainWindow *mwin);
       break;
     }
       
-    case wb::RefreshSchemaList:
-    {
-      id panel = [self selectedTopPanel];
-      if ([panel respondsToSelector:@selector(refreshCatalogTree)])
-        [panel refreshCatalogTree];
-      break;
-    }
-
     case wb::RefreshSchemaNoReload:
       break;
 
@@ -443,9 +434,8 @@ void setup_mforms_app(WBMainWindow *mwin);
                           name:(const char*)name
 {
   wb::ModelDiagramForm *dform= _wbui->get_wb()->get_model_context()->get_diagram_form_for_diagram_id(oid);
-  WBModelDiagramPanel *panel= [[WBModelDiagramPanel alloc] initWithId:[NSString stringWithUTF8String:oid]
-                                                               formBE:dform
-                                              sharedSidebarController:[_physicalOverview sidebarController]];
+  WBModelDiagramPanel *panel= [[WBModelDiagramPanel alloc] initWithId: [NSString stringWithUTF8String:oid]
+                                                               formBE: dform];
 
   NSTabViewItem* item= [self addTopPanel:panel];
   [panel release];
@@ -628,7 +618,10 @@ void setup_mforms_app(WBMainWindow *mwin);
 - (BOOL)closePanel:(WBBasePanel*)panel
 {
   id identifier = [[[panel identifier] retain] autorelease];
-  
+
+  // Remove first responder status from current first responder to trigger any pending processing
+  // done on end-editing. Necessary for instance to make code editors store their content in their backend.
+  [self.window makeFirstResponder:  nil];
   if ([panel respondsToSelector:@selector(willClose)] && ![panel willClose])
     return NO;
   if ([topTabView indexOfTabViewItemWithIdentifier: identifier] != NSNotFound)
@@ -1090,7 +1083,7 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 - (BOOL)closeAllPanels
 {
-  id panelsCopy = [_panels copy];
+  id panelsCopy = [[_panels copy] autorelease];
   id homeTabId = [[topTabView tabViewItemAtIndex: 0] identifier];
   for (id panel in [panelsCopy objectEnumerator])
   {
@@ -1101,7 +1094,6 @@ void setup_mforms_app(WBMainWindow *mwin);
     if (![self closePanel: panel])
       return NO;
   }
-  [panelsCopy release];
   [_panels removeAllObjects];
 
   return YES;
@@ -1111,6 +1103,7 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 - (void)dealloc
 {
+  [NSObject cancelPreviousPerformRequestsWithTarget: self];
   delete _backendObserver;
   
   [_closedTopTabs release];
@@ -1212,11 +1205,10 @@ public:
       NSLog(@"set_view_title called for undocked view");
   }
   
-  virtual void get_size(int *w, int *h)
+  virtual std::pair<int, int> get_size()
   {
     NSRect rect = [[self window] frame];
-    *w = NSWidth(rect);
-    *h = NSHeight(rect);
+    return std::make_pair(NSWidth(rect), NSHeight(rect));
   }
 };
 
@@ -1382,7 +1374,7 @@ static base::Color get_system_color(mforms::SystemColor color)
     }
     case mforms::SystemColorContainer:
     {
-      NSColor *c = [[NSColor controlBackgroundColor] colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
+      NSColor *c = [[NSColor colorWithDeviceWhite: 240/255.0 alpha: 1.0] colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
       return base::Color([c redComponent], [c greenComponent], [c blueComponent]);
     }
   }
