@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -98,7 +98,7 @@ namespace MySQL.GUI.Workbench
       tabImageList = new ImageList();
       tabImageList.ColorDepth = ColorDepth.Depth32Bit;
       tabImageList.ImageSize = new Size(18, 16);
-      ImageListHelper.Add(ApplicationCommand(AppCommand.AppGetResourcePath, null, "WB_Home.png"), tabImageList);
+      ImageListHelper.Add(ApplicationCommand(AppCommand.AppGetResourcePath, "WB_Home.png"), tabImageList);
       contentTabControl.ImageList = tabImageList;
 
       // Create a timer to be triggered when the backend needs
@@ -134,6 +134,9 @@ namespace MySQL.GUI.Workbench
       host.Alignment = ToolStripItemAlignment.Right;
       mainStatusStrip.Items.Add(host);
 
+      // Listen to system color changes.
+      Microsoft.Win32.SystemEvents.UserPreferenceChanged += new Microsoft.Win32.UserPreferenceChangedEventHandler(PreferenceChangedHandler);
+
       ManagedNotificationCenter.AddObserver(this, "GNColorsChanged");
       ManagedNotificationCenter.AddObserver(this, "GNFocusChanged");
     }
@@ -141,6 +144,7 @@ namespace MySQL.GUI.Workbench
     void Destroy()
     {
       ManagedNotificationCenter.RemoveObserver(this, "");
+      Microsoft.Win32.SystemEvents.UserPreferenceChanged -= new Microsoft.Win32.UserPreferenceChangedEventHandler(PreferenceChangedHandler);
 
       // All documents are closed in Form_Closing.
     }
@@ -201,6 +205,27 @@ namespace MySQL.GUI.Workbench
 
     #region Callbacks
 
+    public void PreferenceChangedHandler(object sender, Microsoft.Win32.UserPreferenceChangedEventArgs e)
+    {
+      if (e.Category == Microsoft.Win32.UserPreferenceCategory.VisualStyle     // For changing from high contrast to normal style.
+        || e.Category == Microsoft.Win32.UserPreferenceCategory.Accessibility) // For changing between high contrast schemes.
+      {
+        if (SystemInformation.HighContrast)
+          Conversions.SetColorScheme(ColorScheme.ColorSchemeHighContrast);
+        else
+        {
+          Conversions.SetColorScheme(ColorScheme.ColorSchemeStandard);
+
+          // Re-establish our glass frame.
+          int topAreaHeight = 0;
+          if (contentTabControl.ActiveDocument != null)
+            topAreaHeight += contentTabControl.ActiveDocument.ToolbarHeight;
+
+          AdjustGlassFrame(topAreaHeight);
+        }
+      }
+    }
+
     public void ShowStatusText(String message)
     {
       StatusBarText = message;
@@ -299,7 +324,7 @@ namespace MySQL.GUI.Workbench
       return RequestInputPressedOk;
     }
 
-    public String ApplicationCommand(AppCommand command, MySQL.Forms.AppViewImpl view, String str)
+    public String ApplicationCommand(AppCommand command, String str)
     {
       if (IsDisposed || Disposing)
         return "";
@@ -316,12 +341,6 @@ namespace MySQL.GUI.Workbench
         case AppCommand.AppSetStatusText:
           StatusBarText = str;
           break;
-        case AppCommand.AppGetBounds:
-          // Returns the current bounds of the main window, encoded in a string (we only have one return type: string).
-          // This can be used to center messages over the application window, regardless on which screen/monitor
-          // the app windows currently is.
-          result = String.Format("{0} {1} {2} {3} ", Left, Top, Width, Height);
-          break;
       }
       return result;
     }
@@ -376,7 +395,6 @@ namespace MySQL.GUI.Workbench
           ForwardRefreshToAllDocuments(refresh, str, ptr);
           break;
 
-        case RefreshType.RefreshSchemaList:
         case RefreshType.RefreshSchemaNoReload:
         case RefreshType.RefreshSelection:
         case RefreshType.RefreshZoom:
@@ -621,14 +639,14 @@ namespace MySQL.GUI.Workbench
 
     #region DockingPointDelegate
 
-    public bool select_view(AppViewImpl view)
+    public bool select_view(AppViewDockContent view)
     {
       foreach (ITabDocument document in contentTabControl.Documents)
       {
         if (document is AppViewDockContent)
         {
           AppViewDockContent content = (document as AppViewDockContent);
-          if (content == view.GetHost())
+          if (content == view)
           {
             content.Activate();
             return true;
@@ -638,17 +656,15 @@ namespace MySQL.GUI.Workbench
       return false;
     }
 
-    public void set_view_title(AppViewImpl view, String str)
+    public void set_view_title(AppViewDockContent view, String str)
     {
-      ITabDocument document = view.GetHost();
-      if (contentTabControl.IndexFromDocument(document) > 0)
-        view.GetHost().TabText = str;
+      if (contentTabControl.IndexFromDocument(view) > 0)
+        view.TabText = str;
     }
 
-    public void get_size(out int w, out int h)
+    public System.Drawing.Size get_size()
     {
-      w = contentTabControl.Width;
-      h = contentTabControl.Height;
+      return contentTabControl.Size;
     }
 
     #endregion
@@ -1309,7 +1325,7 @@ namespace MySQL.GUI.Workbench
       else
       {
         AppViewDockContent dockContent = document as AppViewDockContent;
-        toolbar = dockContent.GetAppView().GetToolBar();
+        toolbar = dockContent.GetToolBar();
         if (toolbar == null)
           toolbar = wbContext.toolbar_for_form(null); // Currently never returns a toolbar.
         menu = wbContext.menu_for_appview(dockContent);
@@ -1399,7 +1415,7 @@ namespace MySQL.GUI.Workbench
         AppViewDockContent content = documents[i] as AppViewDockContent;
         if (content != null)
         {
-          MenuStrip menu = content.GetAppView().GetMenuBar();
+          MenuStrip menu = content.GetMenuBar();
           if (menu != null)
           {
             menu.Enabled = (contentTabControl.SelectedIndex == i);
@@ -1446,16 +1462,14 @@ namespace MySQL.GUI.Workbench
             // If the current View is changed, update ModelCatalogForm
             if (activeDocument is ModelDiagramForm)
             {
+              // First focus the canvas to make it the active control before the menu items
+              // are revalidated (on set_active_form).
               ModelDiagramForm form = activeDocument as ModelDiagramForm;
-
-              wbContext.set_active_form(form.BackendForm);
-
               form.FocusCanvasControl();
+              wbContext.set_active_form(form.BackendForm);
             }
             else
-            {
               wbContext.set_active_form(wbDoc.BackendForm);
-            }
           }
           else if (activeDocument is Plugins.DockablePlugin)
           {
@@ -1524,12 +1538,6 @@ namespace MySQL.GUI.Workbench
     /// dock it to the currently active main document.</param>
     public void DockDocument(ITabDocument document, bool main, bool activate)
     {
-      if (document is IWorkbenchDocument)
-        (document as IWorkbenchDocument).UpdateColors();
-      else
-        if (document is AppViewDockContent)
-          (document as AppViewDockContent).UpdateColors();
-
       if (main)
       {
         int index = -1;
@@ -1564,6 +1572,14 @@ namespace MySQL.GUI.Workbench
             if (host is SqlIdeForm)
               (host as SqlIdeForm).DockDocument(document, activate);
       }
+
+      // Do the color updates after everything set up as this depends on the structure.
+      if (document is IWorkbenchDocument)
+        (document as IWorkbenchDocument).UpdateColors();
+      else
+        if (document is AppViewDockContent)
+          (document as AppViewDockContent).UpdateColors();
+
     }
 
     /// <summary>
@@ -1645,7 +1661,7 @@ namespace MySQL.GUI.Workbench
             if (document is MySQL.Forms.AppViewDockContent)
             {
               MySQL.Forms.AppViewDockContent content = document as MySQL.Forms.AppViewDockContent;
-              e.Cancel = !content.GetAppView().DocumentClosing();
+              e.Cancel = !content.DocumentClosing();
             }
 
           if (e.Cancel)
@@ -1687,7 +1703,7 @@ namespace MySQL.GUI.Workbench
           if (documents[i] is MySQL.Forms.AppViewDockContent)
           {
             MySQL.Forms.AppViewDockContent content = documents[i] as MySQL.Forms.AppViewDockContent;
-            content.GetAppView().DocumentClosing();
+            content.DocumentClosing();
           }
           else
             documents[i].Close();
@@ -1763,6 +1779,10 @@ namespace MySQL.GUI.Workbench
 
     private void MainForm_Load(object sender, EventArgs e)
     {
+      // Switch to high contrast mode if that is active on the system but not set in our settings.
+      if (!Conversions.InHighContrastMode() && SystemInformation.HighContrast)
+        Conversions.SetColorScheme(ColorScheme.ColorSchemeHighContrast);
+
       LoadFormState();
 
       BringToFront();
@@ -1807,7 +1827,7 @@ namespace MySQL.GUI.Workbench
         if (document is MySQL.Forms.AppViewDockContent)
         {
           MySQL.Forms.AppViewDockContent content = document as MySQL.Forms.AppViewDockContent;
-          e.canClose = content.GetAppView().DocumentClosing();
+          e.canClose = content.DocumentClosing();
         }
     }
 
@@ -1888,9 +1908,15 @@ namespace MySQL.GUI.Workbench
       }
     }
 
+    private void mainStatusStrip_Paint(object sender, PaintEventArgs e)
+    {
+      using (SolidBrush brush = new SolidBrush(mainStatusStrip.BackColor))
+        e.Graphics.FillRectangle(brush, e.ClipRectangle);
+    }
+
     #endregion
 
-    public class MainPageDockDelegate : DockingPointDelegateWrapper
+    public class MainPageDockDelegate : ManagedDockDelegate
     {
       private MainForm _owner = null;
 
@@ -1905,30 +1931,31 @@ namespace MySQL.GUI.Workbench
         return "MainWindow";
       }
 
-      public override void dock_view(Object representedObject, AppViewImpl view, String arg1, int arg2)
+      public override void dock_view(Object representedObject, AppViewDockContent view, String arg1, int arg2)
       {
-        _owner.DockDocument(view.GetHost(), true, true);
+        _owner.DockDocument(view, true, true);
       }
 
-      public override void undock_view(Object representedObject, AppViewImpl view)
+      public override void undock_view(Object representedObject, AppViewDockContent view)
       {
-        _owner.UndockDocument(view.GetHost());
+        _owner.UndockDocument(view);
       }
 
-      public override bool select_view(Object representedObject, AppViewImpl view)
+      public override bool select_view(Object representedObject, AppViewDockContent view)
       {
         return _owner.select_view(view);
       }
 
-      public override void set_view_title(Object representedObject, AppViewImpl view, String str)
+      public override void set_view_title(Object representedObject, AppViewDockContent view, String str)
       {
         _owner.set_view_title(view, str);
       }
 
-      public override void get_size(Object representedObject, out int w, out int h)
+      public override System.Drawing.Size get_size(Object representedObject)
       {
-        _owner.get_size(out w, out h);
+        return _owner.get_size();
       }
     };
+
   }
 }
