@@ -20,13 +20,14 @@
 #include "tut_stdafx.h"
 #include "base/file_utilities.h"
 #include "base/string_utilities.h"
+#include "base/threading.h"
 
 using namespace base;
 
 #ifdef _WIN32
 #define FILE_SEPARATOR    "\\"
 #define INVALID_NAME      "__test_file01*?"
-#define RESERVED_NAME     "PRN"
+#define RESERVED_NAME     "com1"
 #define MKDIR             "mkdir "
 #define READONLY          "attrib +R "
 #define READWRITE         "attrib -R "
@@ -34,7 +35,7 @@ using namespace base;
 #define FILE_SEPARATOR    "/"
 #define INVALID_NAME      "__test_file01/?"
 #define RESERVED_NAME     "."
-#define MKDIR             "mkdir - p "
+#define MKDIR             "mkdir -p "
 #define READONLY          "chmod 0444 "
 #define READWRITE         "chmod 0777 "
 #endif
@@ -50,6 +51,11 @@ using namespace base;
 #define FILE_PATTERN "__test_"
 #define EMPTY_NAME ""
 
+/*
+ * Some tests on Windows specific behavior (like reserved file names, file sharing violation exceptions, etc) 
+ * are desabled in other platforms
+ * */
+
 BEGIN_TEST_DATA_CLASS(file_utilities_test)
 
   protected:
@@ -69,7 +75,7 @@ TEST_DATA_CONSTRUCTOR(file_utilities_test)
 
   file_unicode_basename.clear();
   file_unicode_basename += "__test_file_";
-  file_unicode_basename += "\xE3\x8A\xA8"; // Valid Unicode character.
+  file_unicode_basename += "\xE3\x8F\xA3"; // Valid Unicode character.
 
   file_unicode_name.clear();
   file_unicode_name = file_unicode_basename;
@@ -94,7 +100,8 @@ TEST_DATA_CONSTRUCTOR(file_utilities_test)
   remove(file_unicode_name);
 
   // Initialize the thread system
-  if (!g_thread_supported ()) g_thread_init (NULL);
+  if (!g_thread_supported ()) 
+    base::threading_init();
 }
 
 TEST_DATA_DESTRUCTOR(file_utilities_test)
@@ -196,8 +203,8 @@ TEST_FUNCTION(5)
     // Remove an existing file
     try
     {
-      base::FileHandle test_file(TEST_FILE_NAME01, "w+"); // Create file
-      test_file.dispose(); // Close file
+      base::FileHandle test_file_scoped(TEST_FILE_NAME01, "w+"); // Create file
+      test_file_scoped.dispose(); // Close file
 
       if (!base::remove(TEST_FILE_NAME01))
       {
@@ -283,7 +290,7 @@ TEST_FUNCTION(10)
       if (!(base::create_directory(dir_unicode_name, 0700)))
       {
           // return false means dir already exists
-          fail(strfmt("TEST 10.4: Directory \"%s\" already exists",dir_unicode_name));
+          fail(strfmt("TEST 10.4: Directory \"%s\" already exists",dir_unicode_name.c_str()));
       }
     }
     catch (base::file_error &exc)
@@ -291,7 +298,8 @@ TEST_FUNCTION(10)
       throw grt::os_error(strfmt("Cannot create directory for document: %s", exc.what()));
     }
 
-    // Create a directory -- Reserved name
+#ifdef _WIN32
+    // Create a directory -- Reserved name. This will not fail in linux, returning false (already exists)
     try
     {
       base::create_directory(RESERVED_NAME, 0700);
@@ -310,27 +318,29 @@ TEST_FUNCTION(10)
       fail(strfmt("TEST 10.5: Directory name \"%s\" threw an unexpected error: %s",
           RESERVED_NAME, exc.what()));
     }
+#endif
 
     // Create a file -- Unicode name
     try
     {
-      base::FileHandle test_file(file_unicode_name.c_str(), "w+"); // Create file
+      base::FileHandle test_file_scoped(file_unicode_name.c_str(), "w+"); // Create file
       if (!file_exists(file_unicode_name))
       {
           // return false means file does not exist
-          fail(strfmt("TEST 10.6: File creation of \"%s\" failed",file_unicode_name));
+          fail(strfmt("TEST 10.6: File creation of \"%s\" failed",file_unicode_name.c_str()));
       }
     }
     catch (base::file_error &exc)
     {
       throw grt::os_error(strfmt("Cannot create file for document: %s", exc.what()));
     }
-
+#ifdef _WIN32
     // Remove a file/directory -- Invalid name
     try
     {
       base::remove(INVALID_NAME);
       fail(strfmt("TEST 10.7: Directory name \"%s\" did not throw an error",INVALID_NAME));
+
     }
     catch (const base::file_error &exc)
     {
@@ -365,6 +375,7 @@ TEST_FUNCTION(10)
       fail(strfmt("TEST 10.8: Directory name \"%s\" threw an unexpected error: %s",
           EMPTY_NAME, exc.what()));
     }
+#endif
 
     // Remove a file/directory -- Too long name
     try
@@ -390,7 +401,7 @@ TEST_FUNCTION(10)
       if (!base::remove(dir_unicode_name))
       {
           // return false means dir does not exist
-          fail(strfmt("TEST 10.10: Directory \"%s\" does not exist",dir_unicode_name));
+          fail(strfmt("TEST 10.10: Directory \"%s\" does not exist",dir_unicode_name.c_str()));
       }
     }
     catch (base::file_error &exc)
@@ -404,7 +415,7 @@ TEST_FUNCTION(10)
       if (!base::remove(file_unicode_name))
       {
           // return false means file does not exist
-          fail(strfmt("TEST 10.11: File \"%s\" does not exist",file_unicode_name));
+          fail(strfmt("TEST 10.11: File \"%s\" does not exist",file_unicode_name.c_str()));
       }
     }
     catch (base::file_error &exc)
@@ -512,19 +523,12 @@ TEST_FUNCTION(25)
   {
       // Miscellaneous variables
       base::FileHandle test_file01(TEST_FILE_NAME01, "w+");
-      base::FileHandle test_file02(test_file01.operator FILE *()); //alias for test_file01
       base::FileHandle test_file03(TEST_FILE_NAME03, "w+");
       base::FileHandle test_file04;
 
       FILE* test_result = NULL;
       FILE* expected_result = NULL;
 
-      // Testing overridden operator FILE *()
-      test_result = test_file02.operator FILE *();
-      expected_result = test_file01.operator FILE *();
-
-      ensure_equals("TEST 25.1: Unexpected result calling operator FILE *()",test_result,expected_result);
-
       // Testing overridden operator bool()
       if (test_file01.operator bool())
       {
@@ -539,17 +543,11 @@ TEST_FUNCTION(25)
       }
 
       // Testing overridden operator =(FileHandle &fh)
-      expected_result = test_file03;
+      expected_result = test_file03.file();
       test_file04 = test_file03;
-      test_result = test_file04;
+      test_result = test_file04.file();
 
       ensure_equals("TEST 25.4: Unexpected result calling operator =(FileHandle &fh)",test_result,expected_result);
-
-      // Testing overridden operator ->()
-      test_result = test_file04.operator->();
-      expected_result = test_file04;
-
-      ensure_equals("TEST 25.5: Unexpected result calling operator ->()",test_result,expected_result);
   }
   catch (base::file_error &exc)
   {
@@ -566,8 +564,6 @@ TEST_FUNCTION(25)
  * - swap(FileHandle &fh)
  * - file()
  * - dispose()
- * - open_file(const char *filename, const char *mode, bool throw_on_fail= true)
- * - close_file(FILE *f)
  */
 TEST_FUNCTION(30)
 {
@@ -582,21 +578,18 @@ TEST_FUNCTION(30)
       FILE* expected_result = NULL;
 
       // Testing swap(FileHandle &fh)
-      expected_result = test_file02;
+      expected_result = test_file02.file();
       test_file01.swap(test_file02);
-      test_result = test_file01;
+      test_result = test_file01.file();
 
       ensure_equals("TEST 30.1: Unexpected result calling function swap(FileHandle &fh)",test_result,expected_result);
 
       // Testing file()
-      test_result = test_file01.file();
-      expected_result = test_file01;
-
-      ensure_equals("TEST 30.2: Unexpected result calling function file()",test_result,expected_result);
+      ensure("TEST 30.2: Unexpected result calling function file()", test_file01.file() != NULL);
 
       // Testing dispose()
       test_file01.dispose();
-      test_result = test_file01;
+      test_result = test_file01.file();
       expected_result = NULL;
 
       ensure_equals("TEST 30.3: Unexpected result calling function dispose()",test_result,expected_result);
@@ -607,20 +600,20 @@ TEST_FUNCTION(30)
       try
       {
           // throw error (file does not exist)
-          test_file03.open_file(TEST_FILE_NAME03, "r");
-          fail("TEST 30.4: Unexpected result calling open_file()");
+          base::FileHandle test_file_scoped(TEST_FILE_NAME03, "r");
+          fail("TEST 30.4: Unexpected result calling FileHandle c-tor");
       }
       catch(...)
       {
           //Nothing to do, just catch the error and continue
       }
 
-      // test case for 'throw_on_fail' FALSE value
+      // Testing FileHandle c-tor
       try
       {
         // doesn't throw error (even if file does not exist)
-        test_file03.open_file(TEST_FILE_NAME03, "r", false);
-        test_result = test_file03;
+        base::FileHandle test_file03(TEST_FILE_NAME03, "r", false);
+        test_result = test_file03.file();
         expected_result = NULL;
 
         ensure_equals("TEST 30.5: Unexpected result calling open_file()",test_result,expected_result);
@@ -629,31 +622,20 @@ TEST_FUNCTION(30)
       {
         fail("TEST 30.5: Unexpected error calling open_file()");
       }
-
-      // Testing open_file()
+      
       try
       {
         base::FileHandle test_file04(TEST_FILE_NAME04, "w+");
-        test_result = test_file04.operator FILE *();
+        test_result = test_file04.file();
 
         if(!test_result)
         {
-          fail("TEST 30.6: Unexpected result calling open_file()");
+          fail("TEST 30.6: Unexpected result calling FileHandle c-tor");
         }
       }
       catch(...)
       {
-        fail("TEST 30.6: Unexpected error calling open_file()");
-      }
-
-      // Testing close_file()
-      try
-      {
-        test_file01.close_file(test_file02.operator FILE *());
-      }
-      catch(...)
-      {
-        fail("TEST 30.7: Unexpected error calling close_file()");
+        fail("TEST 30.6: Unexpected error calling FileHandle c-tor");
       }
 
   }
@@ -674,7 +656,6 @@ TEST_FUNCTION(30)
  * - file()
  * - dispose()
  * - open_file(const char *filename, const char *mode, bool throw_on_fail= true)
- * - close_file(FILE *f)
  * -- Read-only permissions -- 
  */
 TEST_FUNCTION(33)
@@ -711,21 +692,18 @@ TEST_FUNCTION(33)
   try
   {
       // Testing swap(FileHandle &fh)
-      expected_result = test_file02;
+      expected_result = test_file02.file();
       test_file01.swap(test_file02);
-      test_result = test_file01;
+      test_result = test_file01.file();
 
       ensure_equals("TEST 33.1: Unexpected result calling function swap(FileHandle &fh)",test_result,expected_result);
 
       // Testing file()
-      test_result = test_file01.file();
-      expected_result = test_file01;
-
-      ensure_equals("TEST 33.2: Unexpected result calling function file()",test_result,expected_result);
+      ensure("TEST 33.2: Unexpected result calling function file()", test_file01.file() != NULL);
 
       // Testing dispose()
       test_file01.dispose();
-      test_result = test_file01;
+      test_result = test_file01.file();
       expected_result = NULL;
 
       ensure_equals("TEST 33.3: Unexpected result calling function dispose()",test_result,expected_result);
@@ -736,7 +714,7 @@ TEST_FUNCTION(33)
       try
       {
           // throw error (file is read-only)
-          test_file.open_file(TEST_FILE_NAME01, "w");
+          base::FileHandle test_file_scoped(TEST_FILE_NAME01, "w");
           fail(strfmt("TEST 33.4: Read-only file \"%s\" did not throw an error",TEST_FILE_NAME01));
       }
       catch (const base::file_error &exc)
@@ -757,13 +735,13 @@ TEST_FUNCTION(33)
       try
       {
           // doesn't throw error (even if file is read-only)
-          test_file.open_file(TEST_FILE_NAME01, "w", false);
-          test_result = test_file.operator FILE *();
+          base::FileHandle test_file_scoped(TEST_FILE_NAME01, "w", false);
+          test_result = test_file.file();
           expected_result = NULL;
 
           if(test_result != expected_result)
           {
-              fail("TEST 33.5: Unexpected result calling open_file()");
+              fail("TEST 33.5: Unexpected result calling FileHandle c-tor");
           }
       }
       catch(std::exception &exc)
@@ -771,16 +749,6 @@ TEST_FUNCTION(33)
           fail(strfmt("TEST 33.5: Read-only file \"%s\" threw an unexpected error: %s",
               TEST_FILE_NAME01, exc.what()));
       }
-
-      // Testing close_file()
-      try
-      {
-          test_file01.close_file(test_file02.operator FILE *());
-      }
-      catch(std::exception &exc)
-      {
-          fail(strfmt("TEST 33.6: Unexpected error calling close_file(): %s",exc.what()));
-      }
   }
   catch (base::file_error &exc)
   {
@@ -824,7 +792,7 @@ TEST_FUNCTION(35)
 {
   try
   {
-      // Miscellaneous variables
+          // Miscellaneous variables
       base::FileHandle test_file;
 
       // Testing open_file(const char *filename, const char *mode, bool throw_on_fail= true)
@@ -833,7 +801,7 @@ TEST_FUNCTION(35)
       // -- Invalid name
       try
       {
-          test_file.open_file(INVALID_NAME, "r");
+          base::FileHandle test_file_scoped(INVALID_NAME, "r");
           fail(strfmt("TEST 35.1: File name \"%s\" did not throw an error",INVALID_NAME));
       }
       catch (const base::file_error &exc)
@@ -849,12 +817,12 @@ TEST_FUNCTION(35)
           fail(strfmt("TEST 35.1: File name \"%s\" threw an unexpected error: %s",
               INVALID_NAME, exc.what()));
       }
-
+      
       // test case for 'throw_on_fail' default value (i.e. TRUE)
       // -- Empty name
       try
       {
-          test_file.open_file(EMPTY_NAME, "r");
+          base::FileHandle test_file_scoped(EMPTY_NAME, "r");
           fail(strfmt("TEST 35.2: File name \"%s\" did not throw an error",EMPTY_NAME));
       }
       catch (const base::file_error &exc)
@@ -875,7 +843,7 @@ TEST_FUNCTION(35)
       // -- Too long name
       try
       {
-          test_file.open_file(too_long_name.c_str(), "r");
+          base::FileHandle test_file_scoped(too_long_name.c_str(), "r");
           fail("TEST 35.3: File with too long name did not throw an error");
       }
       catch (const base::file_error &exc)
@@ -889,13 +857,15 @@ TEST_FUNCTION(35)
       {
           fail(strfmt("TEST 35.3: File with too long name threw an unexpected error: %s",exc.what()));
       }
-
+      
+#ifdef _WIN32
       // test case for 'throw_on_fail' default value (i.e. TRUE)
       // -- Unicode name (file doesn't exist)
       try
       {
-          test_file.open_file(file_unicode_name.c_str(), "r");
+          base::FileHandle test_file_scoped(file_unicode_name.c_str(), "r");
           fail("TEST 35.4: File with unicode name did not throw an error");
+
       }
       catch (const base::file_error &exc)
       {
@@ -908,7 +878,7 @@ TEST_FUNCTION(35)
       {
           fail(strfmt("TEST 35.4: File with unicode name threw an unexpected error: %s",exc.what()));
       }
-
+#endif
       // test case for 'throw_on_fail' default value (i.e. TRUE)
       // -- Reserved name
       //try
@@ -935,7 +905,7 @@ TEST_FUNCTION(35)
       try
       {
         // doesn't throw error (even if file does not exist)
-        test_file.open_file(INVALID_NAME, "r", false);
+        base::FileHandle test_file_scoped(INVALID_NAME, "r", false);
       }
       catch(...)
       {
@@ -947,7 +917,7 @@ TEST_FUNCTION(35)
       try
       {
         // doesn't throw error (even if file does not exist)
-        test_file.open_file(EMPTY_NAME, "r", false);
+        base::FileHandle test_file_scoped(EMPTY_NAME, "r", false);
       }
       catch(...)
       {
@@ -959,7 +929,7 @@ TEST_FUNCTION(35)
       try
       {
         // doesn't throw error (even if file does not exist)
-        test_file.open_file(too_long_name.c_str(), "r", false);
+        base::FileHandle test_file_scoped(too_long_name.c_str(), "r", false);
       }
       catch(...)
       {
@@ -971,7 +941,7 @@ TEST_FUNCTION(35)
       try
       {
         // doesn't throw error (even if file does not exist)
-        test_file.open_file(file_unicode_name.c_str(), "r", false);
+        base::FileHandle test_file_scoped(file_unicode_name.c_str(), "r", false);
       }
       catch(...)
       {
@@ -983,7 +953,7 @@ TEST_FUNCTION(35)
       try
       {
         // doesn't throw error (even if file does not exist)
-        test_file.open_file(RESERVED_NAME, "r", false);
+        base::FileHandle test_file_scoped(RESERVED_NAME, "r", false);
       }
       catch(...)
       {
@@ -1011,14 +981,12 @@ TEST_FUNCTION(40)
       // constructors & miscellaneous variables
       base::FileHandle c_tor_no_name;
       base::FileHandle c_tor_test_filename(TEST_FILE_NAME01, "w+");
-      base::FileHandle c_tor_FILE_ptr(c_tor_test_filename.operator FILE *());
-
       base::FileHandle c_tor_temp(TEST_FILE_NAME02, "w+");
-      FILE* original_c_tor_temp_ptr = c_tor_temp.operator FILE *();
+      FILE* original_c_tor_temp_ptr = c_tor_temp.file();
       base::FileHandle c_tor_FileHandle_ref(c_tor_temp);
 
       // test case for empty name string
-      FILE* test_result = c_tor_no_name;
+      FILE* test_result = c_tor_no_name.file();
       FILE* expected_result = NULL;
 
       ensure_equals("TEST 40.1: Unexpected result calling FileHandle constructor",test_result,expected_result);
@@ -1040,7 +1008,7 @@ TEST_FUNCTION(40)
       {
         // doesn't throw error (even if file does not exist)
         base::FileHandle c_tor_throw_on_fail(TEST_FILE_NAME03, "r", false);
-        test_result = c_tor_throw_on_fail;
+        test_result = c_tor_throw_on_fail.file();
         expected_result = NULL;
 
         ensure_equals("TEST 40.3: Unexpected result calling FileHandle constructor",test_result,expected_result);
@@ -1050,22 +1018,15 @@ TEST_FUNCTION(40)
         fail("TEST 40.3: Unexpected result calling FileHandle constructor");
       }
 
-      // test case for FILE * value
-      test_result = c_tor_FILE_ptr.operator FILE *();
-      test_result = c_tor_FILE_ptr;
-      expected_result = c_tor_test_filename;
-
-      ensure_equals("TEST 40.4: Unexpected result calling FileHandle constructor",test_result,expected_result);
-
       // test case for FileHandle& value
-      test_result = c_tor_FileHandle_ref;
+      test_result = c_tor_FileHandle_ref.file();
       expected_result = original_c_tor_temp_ptr;
   
       ensure_equals("TEST 40.5: Unexpected result calling FileHandle constructor",test_result,expected_result);
 
       // test case for d-tor
       c_tor_FileHandle_ref.~FileHandle();
-      test_result = c_tor_FileHandle_ref;
+      test_result = c_tor_FileHandle_ref.file();
       expected_result = NULL;
 
       ensure_equals("TEST 40.6: Unexpected result calling FileHandle destructor",test_result,expected_result);
@@ -1150,13 +1111,14 @@ TEST_FUNCTION(45)
       {
           fail(strfmt("TEST 45.3: File with too long name threw an unexpected error: %s",exc.what()));
       }
-
+#ifdef _WIN32
       // test case for 'throw_on_fail' default value (i.e. TRUE)
       // -- Unicode name (file doesn't exist)
       try
       {
           base::FileHandle c_tor(file_unicode_name.c_str(), "r");
           fail("TEST 45.4: File with unicode name did not throw an error");
+
       }
       catch (const base::file_error &exc)
       {
@@ -1170,12 +1132,15 @@ TEST_FUNCTION(45)
           fail(strfmt("TEST 45.4: File with unicode name threw an unexpected error: %s",exc.what()));
       }
 
+#if 0 // this test is not reliable, in some systems/setups this works fine and in some others it'll throw an exception
+
       // test case for 'throw_on_fail' default value (i.e. TRUE)
       // -- Reserved name
       try
       {
-        base::FileHandle c_tor(RESERVED_NAME, "r");
+        base::FileHandle c_tor(RESERVED_NAME, "w+");
         fail(strfmt("TEST 45.5.1: File name \"%s\" did not throw an error",RESERVED_NAME));
+
       }
       catch (const base::file_error &exc)
       {
@@ -1190,7 +1155,8 @@ TEST_FUNCTION(45)
           fail(strfmt("TEST 45.5.3: File name \"%s\" threw an unexpected error: %s",
               RESERVED_NAME, exc.what()));
       }
-
+#endif
+#endif
       // test case for 'throw_on_fail' FALSE value
       // -- Invalid name
       try
@@ -1303,7 +1269,7 @@ TEST_FUNCTION(50)
   try
   {
       // Create file
-      base::FileHandle test_file(TEST_FILE_NAME01, "w+");
+      base::FileHandle test_file_scoped(TEST_FILE_NAME01, "w+");
   }
   catch (base::file_error &exc)
   {
@@ -1428,7 +1394,7 @@ TEST_FUNCTION(53)
   try
   {
       // Create file
-      base::FileHandle test_file(TEST_FILE_NAME01, "w+");
+      base::FileHandle test_file_scoped(TEST_FILE_NAME01, "w+");
   }
   catch (base::file_error &exc)
   {
@@ -1500,7 +1466,8 @@ TEST_FUNCTION(53)
           // return true means file/dir exists
           fail(strfmt("TEST 53.8: 'is_directory()' returned TRUE for a unicode name"));
       }
-
+      
+#ifdef _WIN32
       // -- Reserved name
       if (base::is_directory(RESERVED_NAME))
       {
@@ -1515,7 +1482,7 @@ TEST_FUNCTION(53)
       test_result.clear();
       test_result = base::dirname(INVALID_NAME);
       ensure_equals("TEST 53.10: Unexpected result getting the directory name", test_result, ".");
-
+#endif
       // -- Empty name
       test_result.clear();
       test_result = base::dirname(EMPTY_NAME);
@@ -1821,12 +1788,12 @@ TEST_FUNCTION(53)
           fail(strfmt("TEST 53.31: Non-existing file \"%s\" threw an unexpected error: %s",
               TEST_FILE_NAME03, exc.what()));
       }
-
+#ifdef _WIN32
       // -- Target File already exists
       try
       {
         // Create file
-        base::FileHandle test_file(TEST_FILE_NAME03, "w+");
+        base::FileHandle test_file_scoped(TEST_FILE_NAME03, "w+");
 
         base::rename(TEST_FILE_NAME01, TEST_FILE_NAME03);
         fail(strfmt("TEST 53.32: Already-existing file \"%s\" did not throw an error",TEST_FILE_NAME03));
@@ -1844,7 +1811,7 @@ TEST_FUNCTION(53)
           fail(strfmt("TEST 53.32: Already-existing file \"%s\" threw an unexpected error: %s",
               TEST_FILE_NAME03, exc.what()));
       }
-
+#endif
       // -- Source & Target files are the same, and non-existing
       try
       {
@@ -1896,11 +1863,13 @@ TEST_FUNCTION(53)
       test_result = base::extension(file_unicode_name.c_str());
       ensure_equals("TEST 53.38: Unexpected result getting file extension", test_result, ".txt");
 
+#ifdef _WIN32
       // -- Reserved name
       test_result.clear();
       test_result = base::extension(RESERVED_NAME);
       ensure_equals("TEST 53.39: Unexpected result getting file extension", test_result, "");
-
+#endif
+      
       // -- No extension
       test_result.clear();
       test_result = base::extension("filename_no_ext");
@@ -1946,11 +1915,13 @@ TEST_FUNCTION(53)
       ensure_equals("TEST 53.47: Unexpected result calling strip_extension()", test_result, 
           file_unicode_basename.c_str());
 
+#ifdef _WIN32
       // -- Reserved name
       test_result.clear();
       test_result = base::strip_extension(RESERVED_NAME);
-      ensure_equals("TEST 53.48: Unexpected result calling strip_extension()", test_result, RESERVED_NAME);
 
+      ensure_equals("TEST 53.48: Unexpected result calling strip_extension()", test_result, RESERVED_NAME);
+#endif
       // -- No extension
       test_result.clear();
       test_result = base::strip_extension("filename_with_no_extension");
@@ -2004,13 +1975,14 @@ TEST_FUNCTION(53)
           // return false means dir does not exist
           fail("TEST 53.56: Directory with Unicode name does not exist");
       }
-
+#ifdef _WIN32
       // -- Reserved name
       if (base::remove_recursive(RESERVED_NAME))
       {
           // return true means dir exists
           fail(strfmt("TEST 53.57: Directory \"%s\" exists",RESERVED_NAME));
       }
+#endif
   }
   catch (base::file_error &exc)
   {
@@ -2032,6 +2004,7 @@ TEST_FUNCTION(53)
  * - remove_recursive(const std::string &path)
  * -- Read-only permissions -- 
  */
+#ifdef _WIN32
 TEST_FUNCTION(54)
 {
   std::string command_line;
@@ -2062,7 +2035,7 @@ TEST_FUNCTION(54)
   try
   {
       // Create file
-      base::FileHandle test_file(TEST_FILE_NAME01, "w+");
+      base::FileHandle test_file_scoped(TEST_FILE_NAME01, "w+");
 
       // Change file permission to read-only
       command_line.clear();
@@ -2109,7 +2082,7 @@ TEST_FUNCTION(54)
       // Remove a read-only directory
       try
       {
-          base::remove(TEST_DIR_NAME02);
+          bool result = base::remove(TEST_DIR_NAME02);
           fail(strfmt("TEST 54.2: Read-only directory \"%s\" did not throw an error",TEST_DIR_NAME02));
       }
       catch (const base::file_error &exc)
@@ -2192,6 +2165,7 @@ TEST_FUNCTION(54)
   base::remove(TEST_FILE_NAME01);
   base::remove(TEST_FILE_NAME02);
 }
+#endif
 
 /*
  * Testing public API
@@ -2339,6 +2313,7 @@ TEST_FUNCTION(65)
       // and destructor
       try
       {
+        {
           base::LockFile lock_file01(TEST_FILE_NAME01);
           base::FileHandle test_file02(TEST_FILE_NAME02, "w+");
 
@@ -2347,14 +2322,8 @@ TEST_FUNCTION(65)
 
           if (LockFile::check(TEST_FILE_NAME02) != LockFile::NotLocked)
               fail(strfmt("TEST 65.2: File \"%s\" locked",TEST_FILE_NAME02));
-
-          lock_file01.~LockFile();
-#ifdef _WIN32
-          lock_file01.handle = NULL;
-#else
-          lock_file01.fd = -1;
-#endif
-          ensure("TEST 65.3: Failed d-tor call", !base::file_exists(TEST_FILE_NAME01));
+        }
+        ensure("TEST 65.3: Failed d-tor call", !base::file_exists(TEST_FILE_NAME01));
       }
       catch (base::file_error &exc)
       {
@@ -2379,6 +2348,7 @@ TEST_FUNCTION(65)
   remove(TEST_FILE_NAME02);
 }
 
+#ifdef _WIN32
 /*
  * Child thread function
  */
@@ -2389,140 +2359,148 @@ gpointer _child_thread_func(gpointer data)
     FILE* expected_result = NULL;
     base::FileHandle test_file;
 
-    // Testing rename(const std::string &from, const std::string &to)
+    //  If the tread thows an exception it will SIGABRT the process. So we'll catch the exception
+    //  and return gracefully.
     try
     {
-        base::rename(TEST_FILE_NAME01, TEST_FILE_NAME02);
-        fail(strfmt("TEST 70.1: Locked file \"%s\" did not throw an error",TEST_FILE_NAME01));
-    }
-    catch (const base::file_error &exc)
-    {
-        if (0 != std::string(exc.what()).find("Could not rename file "))
-        {
-            fail(strfmt("TEST 70.1: Locked file \"%s\" threw an unexpected error: %s",
-                TEST_FILE_NAME01, exc.what()));
-        }
+      // Testing rename(const std::string &from, const std::string &to)
+      try
+      {
+          base::rename(TEST_FILE_NAME01, TEST_FILE_NAME02);
+          fail(strfmt("TEST 70.1: Locked file \"%s\" did not throw an error",TEST_FILE_NAME01));
+      }
+      catch (const base::file_error &exc)
+      {
+          if (0 != std::string(exc.what()).find("Could not rename file "))
+          {
+              fail(strfmt("TEST 70.1: Locked file \"%s\" threw an unexpected error: %s",
+                  TEST_FILE_NAME01, exc.what()));
+          }
+      }
+      catch(std::exception &exc)
+      {
+          fail(strfmt("TEST 70.1: Locked file \"%s\" threw an unexpected error: %s",
+              TEST_FILE_NAME01, exc.what()));
+      }
+
+      // Testing remove(const std::string &path)
+      try
+      {
+          base::remove(TEST_FILE_NAME01);
+          fail(strfmt("TEST 70.2: Locked file \"%s\" did not throw an error",TEST_FILE_NAME01));
+      }
+      catch (const base::file_error &exc)
+      {
+          if (0 != std::string(exc.what()).find("Could not delete file "))
+          {
+              fail(strfmt("TEST 70.2: Locked file \"%s\" threw an unexpected error: %s",
+                  TEST_FILE_NAME01, exc.what()));
+          }
+      }
+      catch(std::exception &exc)
+      {
+          fail(strfmt("TEST 70.2: Locked file \"%s\" threw an unexpected error: %s",
+              TEST_FILE_NAME01, exc.what()));
+      }
+
+      // Testing open_file(const char *filename, const char *mode, bool throw_on_fail= true)
+
+      // test case for 'throw_on_fail' default value (i.e. TRUE)
+      try
+      {
+          // throw error (file is locked)
+          base::FileHandle test_file_scoped(TEST_FILE_NAME01, "w");
+          fail(strfmt("TEST 70.3: Locked file \"%s\" did not throw an error",TEST_FILE_NAME01));
+      }
+      catch (const base::file_error &exc)
+      {
+          if (0 != std::string(exc.what()).find("Failed to open file \""))
+          {
+              fail(strfmt("TEST 70.3: Locked file \"%s\" threw an unexpected error: %s",
+                  TEST_FILE_NAME01, exc.what()));
+          }
+      }
+      catch(std::exception &exc)
+      {
+          fail(strfmt("TEST 70.3: Locked file \"%s\" threw an unexpected error: %s",
+              TEST_FILE_NAME01, exc.what()));
+      }
+
+      // test case for 'throw_on_fail' FALSE value
+      try
+      {
+          // doesn't throw error (even if file is locked)
+          base::FileHandle test_file_scoped(TEST_FILE_NAME01, "w", false);
+          test_result = test_file.file();
+          expected_result = NULL;
+
+          ensure_equals("TEST 70.4: Unexpected result calling FileHandle c-tor",test_result,expected_result);
+      }
+      catch(std::exception &exc)
+      {
+          fail(strfmt("TEST 70.4: Read-only file \"%s\" threw an unexpected error: %s",
+              TEST_FILE_NAME01, exc.what()));
+      }
+
+      // Testing FileHandle c-tor
+      try
+      {
+          base::FileHandle c_tor_test_filename(TEST_FILE_NAME01, "w+");
+          fail(strfmt("TEST 70.5: Locked file \"%s\" did not throw an error",TEST_FILE_NAME01));
+      }
+      catch (const base::file_error &exc)
+      {
+          if (0 != std::string(exc.what()).find("Failed to open file \""))
+          {
+              fail(strfmt("TEST 70.5: Locked file \"%s\" threw an unexpected error: %s",
+                  TEST_FILE_NAME01, exc.what()));
+          }
+      }
+      catch(std::exception &exc)
+      {
+          fail(strfmt("TEST 70.5: Locked file \"%s\" threw an unexpected error: %s",
+              TEST_FILE_NAME01, exc.what()));
+      }
+
+      // Testing LockFile c-tor
+      try
+      {
+          base::LockFile lock_file(TEST_FILE_NAME01);
+          fail(strfmt("TEST 70.6: Locked file \"%s\" did not throw an error",TEST_FILE_NAME01));
+      }
+      catch (const base::file_locked_error)
+      {
+          //Nothing to do, just catch the error and continue
+      }
+      catch(std::exception &exc)
+      {
+          fail(strfmt("TEST 70.6: Locked file \"%s\" threw an unexpected error: %s",
+              TEST_FILE_NAME01, exc.what()));
+      }
+
+      // Testing LockFile::check in child thread
+      try
+      {
+          if (LockFile::check(TEST_FILE_NAME01) != LockFile::LockedSelf)
+              fail(strfmt("TEST 70.7: File \"%s\" not locked",TEST_FILE_NAME01));
+      }
+      catch (base::file_error &exc)
+      {
+          throw grt::os_error(strfmt("File error: %s", exc.what()));
+      }
+      catch (std::invalid_argument &exc)
+      {
+          throw grt::os_error(strfmt("Invalid argument error: %s", exc.what()));
+      }
+      catch (std::runtime_error &exc)
+      {
+          throw grt::os_error(strfmt("Runtime/file-locked error: %s", exc.what()));
+      }
     }
     catch(std::exception &exc)
     {
-        fail(strfmt("TEST 70.1: Locked file \"%s\" threw an unexpected error: %s",
-            TEST_FILE_NAME01, exc.what()));
+      fail(exc.what());
     }
-
-    // Testing remove(const std::string &path)
-    try
-    {
-        base::remove(TEST_FILE_NAME01);
-        fail(strfmt("TEST 70.2: Locked file \"%s\" did not throw an error",TEST_FILE_NAME01));
-    }
-    catch (const base::file_error &exc)
-    {
-        if (0 != std::string(exc.what()).find("Could not delete file "))
-        {
-            fail(strfmt("TEST 70.2: Locked file \"%s\" threw an unexpected error: %s",
-                TEST_FILE_NAME01, exc.what()));
-        }
-    }
-    catch(std::exception &exc)
-    {
-        fail(strfmt("TEST 70.2: Locked file \"%s\" threw an unexpected error: %s",
-            TEST_FILE_NAME01, exc.what()));
-    }
-
-    // Testing open_file(const char *filename, const char *mode, bool throw_on_fail= true)
-
-    // test case for 'throw_on_fail' default value (i.e. TRUE)
-    try
-    {
-        // throw error (file is locked)
-        test_file.open_file(TEST_FILE_NAME01, "w");
-        fail(strfmt("TEST 70.3: Locked file \"%s\" did not throw an error",TEST_FILE_NAME01));
-    }
-    catch (const base::file_error &exc)
-    {
-        if (0 != std::string(exc.what()).find("Failed to open file \""))
-        {
-            fail(strfmt("TEST 70.3: Locked file \"%s\" threw an unexpected error: %s",
-                TEST_FILE_NAME01, exc.what()));
-        }
-    }
-    catch(std::exception &exc)
-    {
-        fail(strfmt("TEST 70.3: Locked file \"%s\" threw an unexpected error: %s",
-            TEST_FILE_NAME01, exc.what()));
-    }
-
-    // test case for 'throw_on_fail' FALSE value
-    try
-    {
-        // doesn't throw error (even if file is locked)
-        test_file.open_file(TEST_FILE_NAME01, "w", false);
-        test_result = test_file;
-        expected_result = NULL;
-
-        ensure_equals("TEST 70.4: Unexpected result calling open_file()",test_result,expected_result);
-    }
-    catch(std::exception &exc)
-    {
-        fail(strfmt("TEST 70.4: Read-only file \"%s\" threw an unexpected error: %s",
-            TEST_FILE_NAME01, exc.what()));
-    }
-
-    // Testing FileHandle c-tor
-    try
-    {
-        base::FileHandle c_tor_test_filename(TEST_FILE_NAME01, "w+");
-        fail(strfmt("TEST 70.5: Locked file \"%s\" did not throw an error",TEST_FILE_NAME01));
-    }
-    catch (const base::file_error &exc)
-    {
-        if (0 != std::string(exc.what()).find("Failed to open file \""))
-        {
-            fail(strfmt("TEST 70.5: Locked file \"%s\" threw an unexpected error: %s",
-                TEST_FILE_NAME01, exc.what()));
-        }
-    }
-    catch(std::exception &exc)
-    {
-        fail(strfmt("TEST 70.5: Locked file \"%s\" threw an unexpected error: %s",
-            TEST_FILE_NAME01, exc.what()));
-    }
-
-    // Testing LockFile c-tor
-    try
-    {
-        base::LockFile lock_file(TEST_FILE_NAME01);
-        fail(strfmt("TEST 70.6: Locked file \"%s\" did not throw an error",TEST_FILE_NAME01));
-    }
-    catch (const base::file_locked_error)
-    {
-        //Nothing to do, just catch the error and continue
-    }
-    catch(std::exception &exc)
-    {
-        fail(strfmt("TEST 70.6: Locked file \"%s\" threw an unexpected error: %s",
-            TEST_FILE_NAME01, exc.what()));
-    }
-
-    // Testing LockFile::check in child thread
-    try
-    {
-        if (LockFile::check(TEST_FILE_NAME01) != LockFile::LockedSelf)
-            fail(strfmt("TEST 70.7: File \"%s\" not locked",TEST_FILE_NAME01));
-    }
-    catch (base::file_error &exc)
-    {
-        throw grt::os_error(strfmt("File error: %s", exc.what()));
-    }
-    catch (std::invalid_argument &exc)
-    {
-        throw grt::os_error(strfmt("Invalid argument error: %s", exc.what()));
-    }
-    catch (std::runtime_error &exc)
-    {
-        throw grt::os_error(strfmt("Runtime/file-locked error: %s", exc.what()));
-    }
-
     return NULL;
 }
 
@@ -2573,9 +2551,11 @@ TEST_FUNCTION(70)
     }
 
     // Kick off the child thread
-    _child_thread = g_thread_create(_child_thread_func, this, TRUE, NULL);
+    _child_thread = base::create_thread(_child_thread_func, this);
     // Wait for _main_thread to finish
     g_thread_join(_child_thread);
 }
+#endif
+
 
 END_TESTS
