@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,7 +18,10 @@
  */
 
 #include "stdafx.h"
-
+#include <map>
+#include <string>
+#include "SciLexer.h"
+#include "tinyxml.h"
 #include "base/log.h"
 #include "base/drawing.h"
 #include "base/string_utilities.h"
@@ -276,6 +279,7 @@ CodeEditor::CodeEditor(void *host)
 
 CodeEditor::~CodeEditor()
 {
+  auto_completion_cancel();
   for (std::map<int, void*>::iterator iterator = _images.begin(); iterator != _images.end(); ++iterator)
     free(iterator->second);
 }
@@ -464,6 +468,9 @@ const std::string CodeEditor::get_text_in_range(int start, int end)
   range.lpstrText = (char*)malloc(end - start + 1); // Don't forget the 0 terminator.
   _code_editor_impl->send_editor(this, SCI_GETTEXTRANGE, 0, (sptr_t)&range);
 
+  if (range.lpstrText == NULL)
+    return std::string();
+
   std::string result(range.lpstrText, end - start);
   free(range.lpstrText);
 
@@ -920,6 +927,7 @@ void CodeEditor::on_notify(SCNotification* notification)
   case SCN_DWELLEND:
     _dwell_event(false, 0, 0, 0);
     break;
+
   case SCN_UPDATEUI:
     switch (notification->updated)
     {
@@ -934,20 +942,20 @@ void CodeEditor::on_notify(SCNotification* notification)
       break;
     }
     break;
+
   case SCN_CHARADDED:
     _char_added_event(notification->ch);
     break;
+  case SCN_FOCUSOUT:
+    _signal_lost_focus();
+    break;
   }
 }
 //--------------------------------------------------------------------------------------------------
 
-void CodeEditor::on_command(int command) {
-  switch(command)
-  {
-  case SCEN_KILLFOCUS:
-    _signal_lost_focus();
-  break;
-  }
+void CodeEditor::on_command(int command)
+{
+  // TODO: removal candidate.
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -981,22 +989,8 @@ void CodeEditor::set_features(CodeEditorFeature features, bool flag)
   }
 
   if ((features & mforms::FeatureReadOnly) != 0)
-  {
     _code_editor_impl->send_editor(this, SCI_SETREADONLY, flag, 0);
 
-    Color backColor;
-    if (flag)
-      backColor = mforms::App::get()->get_system_color(mforms::SystemColorDisabled);
-    else
-      backColor = mforms::App::get()->get_system_color(mforms::SystemColorEditor);
-
-    // Scintilla uses a BGR component order.
-    int rawColor = (int)(255 * backColor.red) + ((int)(255 * backColor.green) << 8) +
-      ((int)(255 * backColor.blue) << 16);
-    for (int i = 0; i <= 32; i++)
-      _code_editor_impl->send_editor(this, SCI_STYLESETBACK, i, rawColor);
-  }
-
   if ((features & mforms::FeatureShowSpecial) != 0)
   {
     _code_editor_impl->send_editor(this, SCI_SETVIEWEOL, flag, 0);
@@ -1349,7 +1343,7 @@ int CodeEditor::find_and_replace_text(const std::string& search_text, const std:
         break;
 
       replace_count++;
-      result = _code_editor_impl->send_editor(this, SCI_REPLACETARGET, new_text.size(), (sptr_t)new_text.c_str());
+      /*result =*/ _code_editor_impl->send_editor(this, SCI_REPLACETARGET, new_text.size(), (sptr_t)new_text.c_str());
 
       // The replacement changes the target range to the replaced text. Continue after that until the end.
       // The text length might be changed by the replacement so make sure the target end is the actual
@@ -1367,7 +1361,7 @@ int CodeEditor::find_and_replace_text(const std::string& search_text, const std:
 
     if (replace_count > 0)
     {
-      result = _code_editor_impl->send_editor(this, SCI_REPLACETARGET, new_text.size(), (sptr_t)new_text.c_str());
+      /*result =*/ _code_editor_impl->send_editor(this, SCI_REPLACETARGET, new_text.size(), (sptr_t)new_text.c_str());
 
       // For a single replace we set the new selection to the replaced text.
       _code_editor_impl->send_editor(this, SCI_SETSELECTIONSTART,
@@ -1522,17 +1516,19 @@ void CodeEditor::auto_completion_register_images(const std::vector<std::pair<int
           free(entry->second);
         
         unsigned char *target = (unsigned char*)malloc(4 * width * height);
-        _images[images[i].first] = target;
-        int j = 0;
-        while (j < 4 * width * height)
+        if (target != NULL)
         {
-          target[j] = data[j + 2];
-          target[j + 1] = data[j + 1];
-          target[j + 2] = data[j];
-          target[j + 3] = data[j + 3];
-          j += 4;
+          _images[images[i].first] = target;
+          int j = 0;
+          while (j < 4 * width * height)
+          {
+            target[j] = data[j + 2];
+            target[j + 1] = data[j + 1];
+            target[j + 2] = data[j];
+            target[j + 3] = data[j + 3];
+            j += 4;
+          }
         }
-        
         _code_editor_impl->send_editor(this, SCI_REGISTERRGBAIMAGE, images[i].first, (sptr_t)target);
         cairo_surface_destroy(image);
       }
