@@ -32,8 +32,7 @@
 #include <stdexcept>
 #include <boost/function.hpp>
 #include <libxml/xmlmemory.h>
- 		  	
-#include <glib.h>
+#include "base/threading.h"
 
 #include <string>
 
@@ -60,23 +59,7 @@
 #define THROW(...) throw (__VA_ARGS__)
 #endif
 
-#ifdef _WIN32
-#include <hash_set>
-typedef stdext::hash_set<std::string> string_hash_set;
-#else
-#include <ext/hash_set>
-
-struct string_hash : public std::unary_function<std::string, size_t> 
-{
-  size_t operator() (const std::string &v) const
-  {
-    return __gnu_cxx::hash<const char*>()(v.c_str());
-  }
-};
-
-typedef __gnu_cxx::hash_set<std::string, string_hash> string_hash_set;
-#endif
-
+#include <set>
 
 #define GRT_VERSION "4.1.0"
 
@@ -188,15 +171,9 @@ namespace grt {
 
 
 #include "grtpp_value.h"
-//this is too unstable and overcomplicated
-//#define USE_EXPRERIMENTAL_REFS
-#ifdef USE_EXPRERIMENTAL_REFS
-#include "grtpp_references.h"
-#endif
 
 namespace grt {
 
-#ifndef USE_EXPRERIMENTAL_REFS
   class GRT;
   
   namespace internal
@@ -1872,7 +1849,6 @@ namespace grt {
         throw type_error(DictType, content().content_type(), ListType);
     }
   };
-#endif
 
 //--------------------------------------------------------------------------------------------------
 
@@ -2058,7 +2034,7 @@ namespace grt {
     bool foreach_member(TPred pred)
     {
       // set of already seen members (only overriden ones)
-      string_hash_set seen;
+      std::set<std::string> seen;
       MetaClass *mc= this;
 
       do
@@ -2095,7 +2071,7 @@ namespace grt {
     bool foreach_method(TPred pred)
     {
       // set of already seen methods (only overriden ones)
-      string_hash_set seen;
+      std::set<std::string> seen;
       MetaClass *mc= this;
 
       do
@@ -2130,7 +2106,7 @@ namespace grt {
     bool foreach_signal(TPred pred)
     {
       // set of already seen methods (only overriden ones)
-      string_hash_set seen;
+      std::set<std::string> seen;
       MetaClass *mc= this;
 
       do
@@ -2780,7 +2756,7 @@ namespace grt {
     
     std::vector<std::pair<float,float> > _progress_step_stack;
 
-    GStaticRecMutex _message_mutex;
+    base::RecMutex _message_mutex;
 
     std::list<ModuleLoader*> _loaders;
     std::vector<Module*> _modules;
