@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -394,7 +394,7 @@ bool Layouter::Node::is_linked_to(const int node) const
     }
   }
 
-  return false;
+  return found;
 }
 
 //------------------------------------------------------------------------------
@@ -1030,12 +1030,12 @@ static workbench_physical_DiagramRef create_view_for_object_count(workbench_phys
 {
   int xpages= 2, ypages= 1;
   int pages;
-  // guesstimate about 20 objects per page
-  pages= object_count / 20;
+  // guesstimate about 15 objects per page
+  pages= (int)ceil(object_count / 15.0);
   ypages= (int)sqrt((float)pages);
   if (ypages < 1) 
     ypages= 1;
-  xpages= pages / ypages;
+  xpages= (int)ceil((float)pages / ypages);
   if (xpages < 1) 
     xpages= 1;
 
@@ -1087,8 +1087,8 @@ int WbModelImpl::createDiagramWithCatalog(workbench_physical_ModelRef model, db_
   {
     db_SchemaRef schema= schemata[n];
     object_count+= schema->tables().count();
-    object_count+= schema->views().count()/4; // views and routine groups
-    object_count+= schema->routineGroups().count()/3; // take less space
+    object_count+= schema->views().count()/3; // views and routine groups
+    object_count+= schema->routineGroups().count()/2; // take less space
   }
   if (object_count > 250)
     throw logic_error("Cannot create diagram: too many objects to place.\nTry dividing your model into several sub-diagrams with less than 200 objects each.");
