@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -15,12 +15,14 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 # 02110-1301  USA
 
+import re
 import sys
 import subprocess
 import os
 import threading
 import zipfile
 import tempfile
+import shlex
 
 # import the wb module
 from wb import DefineModule, wbinputs
@@ -28,12 +30,16 @@ from wb import DefineModule, wbinputs
 import grt
 import mforms
 
+from grt import log_warning
+from workbench.log import log_info
+
 # define this Python module as a GRT module
 ModuleInfo = DefineModule(name= "PyWbUtils", author= "Sun Microsystems Inc.", version="1.0")
 
 
 # wb_model_utils contains a few more plugins that will get registered when imported
 import wb_model_utils # noqa
+import wb_catalog_utils # noqa
 
 def get_linux_terminal_program():
     paths = os.getenv("PATH").split(":")
@@ -116,37 +122,50 @@ def restoreConnections():
                               'or the file is corrupted: %s.' % error.message)
                 return
 
-            instances   = grt.unserialize(instances_file.name)
             connections = grt.unserialize(connections_file.name)
 
-            if not isinstance(instances, grt.List) or not isinstance(connections, grt.List):
+            if not isinstance(connections, grt.List):
                 mforms.Utilities.show_error('Restore Connections Error', 'The selected file is not a valid backup file '
                                             'or the file is corrupted.',
                                             'OK', '', '')
-                grt.log_error('restoreConnections', 'The selected file is not a valid backup file or the file is corrupted.')
+                grt.log_error('restoreConnections', 'The selected archive does not have a valid connection backup file.\n')
                 return
 
             inserted_connections = {}
             existent_connection_names = set(conn.name for conn in grt.root.wb.rdbmsMgmt.storedConns)
+            existent_connection_ids = set(conn.__id__ for conn in grt.root.wb.rdbmsMgmt.storedConns)
+            duplicate_connection_count = 0
             for candidate_connection in connections:
-                new_name = generate_unique_name(candidate_connection.name, existent_connection_names)
-                if new_name != candidate_connection.name:
-                    candidate_connection.name = new_name
+                if candidate_connection.__id__ in existent_connection_ids:
+                    duplicate_connection_count = duplicate_connection_count + 1
+                    continue
+                    
+                candidate_connection.name = generate_unique_name(candidate_connection.name, existent_connection_names)
+
                 existent_connection_names.add(candidate_connection.name)
-                old_id = candidate_connection.__id__
-                candidate_connection = candidate_connection.shallow_copy()
-                inserted_connections[old_id] = candidate_connection
+                candidate_connection.owner = grt.root.wb.rdbmsMgmt
+                inserted_connections[candidate_connection.__id__] = candidate_connection
                 grt.root.wb.rdbmsMgmt.storedConns.append(candidate_connection)
 
+            instances   = grt.unserialize(instances_file.name)
+            
+            if not isinstance(instances, grt.List):
+                mforms.Utilities.show_error('Restore Connections Error', 'The selected file is not a valid backup file '
+                                            'or the file is corrupted.',
+                                            'OK', '', '')
+                grt.log_error('restoreConnections', 'Workbench restored %i valid connections but server configuration data coul not be found or is not valid.\n' % len(connections))
+                return
+     
             existent_instance_names = set(instance.name for instance in grt.root.wb.rdbmsMgmt.storedInstances)
             previous_instances_conns = set()
+            duplicated_instance_count = 0
             for candidate_instance in instances:
                 if candidate_instance.connection.__id__ in previous_instances_conns:
+                    duplicated_instance_count = duplicated_instance_count + 1
                     continue  # Skip instances whose connections are associated to previously processed instances
                 previous_instances_conns.add(candidate_instance.connection.__id__)
-                new_name = generate_unique_name(candidate_instance.name, existent_instance_names)
-                if new_name != candidate_instance.name:
-                    candidate_instance.name = new_name
+                candidate_instance.name = generate_unique_name(candidate_instance.name, existent_instance_names)
+
                 existent_instance_names.add(candidate_instance.name)
                 new_conn = inserted_connections.get(candidate_instance.connection.__id__, None)
                 candidate_instance = candidate_instance.shallow_copy()
@@ -156,33 +175,210 @@ def restoreConnections():
             grt.modules.Workbench.refreshHomeConnections()
             grt.modules.Workbench.saveConnections()
             grt.modules.Workbench.saveInstances()
+            
+            if duplicate_connection_count > 0 or duplicated_instance_count > 0:
+                message = []
+                message.append('Workbench detected ')
+                if duplicate_connection_count > 0:
+                    message.append('%i duplicated connections' % duplicate_connection_count)
+                if duplicated_instance_count > 0:
+                    if duplicate_connection_count > 0:
+                        message.append(' and ')
+                    message.append('%i duplicated instances' % duplicated_instance_count)
+                message.append(', which were not restored.')
+                mforms.Utilities.show_warning('Restore Connections', ''.join(message), 'OK', '', '')
+            
         except zipfile.BadZipfile, error:
             mforms.Utilities.show_error('Restore Connections Error', 'The selected file is not a valid backup file '
                                         'or the file is corrupted.',
                                         'OK', '', '')
-            grt.log_error('restoreConnections', 'The selected file is not a valid backup file or the file is corrupted: %s' % error)
+            grt.log_error('restoreConnections', 'The selected file is not a valid backup file or the file is corrupted: %s\n' % error)
         except IOError, error:
             mforms.Utilities.show_error('Restore Connections Error', 'Cannot read from file. Please check this file '
                                         'permissions and try again.',
                                         'OK', '', '')
-            grt.log_error('restoreConnections', str(error))
+            grt.log_error('restoreConnections', '%s\n' % str(error))
     return 0
 
 
+@ModuleInfo.export(grt.STRING, grt.classes.db_mgmt_Connection)
+def connectionStringFromConnection(conn):
+    #<user>[:<password>]@<host>[:<port>][:<socket>]
+    connstr = ""
+    if conn.driver.name == "MysqlNative":
+        connstr = "%s@%s:%s" % (conn.parameterValues["userName"], conn.parameterValues["hostName"], conn.parameterValues["port"])
+    elif conn.driver.name == "MysqlNativeSocket":
+        connstr = "%s@%s::%s" % (conn.parameterValues["userName"], conn.parameterValues["hostName"], conn.parameterValues["socket"])
+    elif conn.driver.name == "MysqlNativeSSH":
+        #XXX this is incomplete, need some way to encode the ssh params
+        connstr = "%s@%s::%s" % (conn.parameterValues["userName"], conn.parameterValues["hostName"], conn.parameterValues["port"])
+    return connstr
+
+
+@ModuleInfo.export(grt.classes.db_mgmt_Connection, grt.STRING)
+def connectionFromString(connstr):
+    valid = False
+    
+    def get_driver(name):
+        for d in grt.root.wb.rdbmsMgmt.rdbms[0].drivers:
+            if d.name == name:
+                return d
+        return None
+    
+    # parse as a one of our connection strings
+    g = re.match("(.*?)(?::(.*))?@(.*?)(?::([0-9]+|)(?::(.+|))?)?$", connstr)
+    if g:
+        user, password, host, port, socket = g.groups()
+        valid = True
+    else:
+        user, password, host, port, socket = None, None, None, None, None
+        # check if this is a mysql cmdline client command
+        tokens = shlex.split(connstr.strip())
+        if tokens:
+            print tokens
+            if tokens[0].endswith("mysql") or tokens[0].endswith("mysql.exe"):
+                i = 1
+                valid = True
+                while i < len(tokens):
+                    if tokens[i] == "-u":
+                        i += 1
+                        user = tokens[i]
+                    elif tokens[i].startswith("-u"):
+                        user = tokens[i][2:]
+                    elif tokens[i] == "-h":
+                        i += 1
+                        host = tokens[i]
+                    elif tokens[i].startswith("-h"):
+                        host = tokens[i][2:]
+                    elif tokens[i] == "-p":
+                        i += 1
+                        password = tokens[i]
+                    elif tokens[i].startswith("-p"):
+                        password = tokens[i][2:] # noqa
+                    elif tokens[i] == "-P":
+                        i += 1
+                        port = tokens[i]
+                    elif tokens[i].startswith("-P"):
+                        port = tokens[i][2:]
+                    elif tokens[i] == "-S":
+                        i += 1
+                        socket = tokens[i]
+                    elif tokens[i].startswith("-S"):
+                        socket = tokens[i][2:]
+                    i += 1
+    if valid:
+        if port:
+            try:
+                port = int(port)
+            except:
+                log_warning("wb_utils", "Error parsing connstring, port value '%s' should be a number\n" % port)
+                port = None
+        if not port:
+            port = 3306
+        conn = grt.classes.db_mgmt_Connection()
+        conn.owner = grt.root.wb.rdbmsMgmt
+        conn.name = connstr
+        if socket:
+            conn.driver = get_driver("MysqlNativeSocket")
+        else:
+            conn.driver = get_driver("MysqlNative")
+
+        if user:
+            conn.parameterValues["userName"] = user
+        if host:
+            conn.parameterValues["hostName"] = host
+        if port:
+            conn.parameterValues["port"] = port
+        if socket:
+            conn.parameterValues["socket"] = socket
+
+        hostIdentifier = conn.driver.hostIdentifierTemplate
+        for key, value in conn.parameterValues.items():
+            hostIdentifier = hostIdentifier.replace("%"+key+"%", str(value))
+        conn.hostIdentifier = hostIdentifier
+
+        return conn
+
+    return None
+
+
 @ModuleInfo.plugin("wb.tools.copyConnectionString", caption="Copy Connection String to Clipboard", input= [wbinputs.selectedConnection()], pluginMenu="Home/Connections")
 @ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection)
 def copyConnectionString(conn):
-    #<user>[:<password>]@<host>[:<port>][:<socket>]
-    connstr = ""
-    if conn.driver.name == "MysqlNative":
-        connstr = "%s:@%s:%s:" % (conn.parameterValues["userName"], conn.parameterValues["hostName"], conn.parameterValues["port"])
-    elif conn.driver.name == "MysqlNativeSocket":
-        connstr = "%s:@%s::%s" % (conn.parameterValues["userName"], conn.parameterValues["hostName"], conn.parameterValues["socket"])
-    elif conn.driver.name == "MysqlNativeSSH":
-        connstr = "%s:@%s::%s" % (conn.parameterValues["userName"], conn.parameterValues["hostName"], conn.parameterValues["port"])
+    connstr = connectionStringFromConnection(conn)
     mforms.Utilities.set_clipboard_text(connstr)
 
 
+@ModuleInfo.plugin("wb.tools.copyJDBCConnectionString", caption="Copy JDBC Connection String to Clipboard", input= [wbinputs.selectedConnection()], pluginMenu="Home/Connections")
+@ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection)
+def copyJDBCConnectionString(conn):
+    if conn.parameterValues.has_key("schema"):
+        params = "/"+conn.parameterValues["schema"]
+    else:
+        params = "/"
+    params += "?user=%s" % conn.parameterValues["userName"]
+
+    if conn.driver.name == "MysqlNative":
+        connstr = "jdbc:mysql://%s:%s" % (conn.parameterValues["hostName"], conn.parameterValues["port"])
+    elif conn.driver.name == "MysqlNativeSocket":
+        connstr = "jdbc:mysql://%s:%s" % (conn.parameterValues["hostName"], conn.parameterValues["socket"])
+    elif conn.driver.name == "MysqlNativeSSH":
+        mforms.Utilities.show_error("Copy JDBC Connection String",
+            "Cannot create JDBC connection string for %s. The connection uses a SSH tunnel." % conn.name,
+            "OK", "", "")
+        return
+    mforms.Utilities.set_clipboard_text(connstr+params)
+
+
+@ModuleInfo.plugin("wb.tools.createMissingLocalConnections", caption="Create Missing Local Connections", input= [], pluginMenu="Home/Connections")
+@ModuleInfo.export(grt.INT)
+def createMissingLocalConnections():
+
+    grt.modules.Workbench.createInstancesFromLocalServers()
+    
+    grt.modules.Workbench.refreshHomeConnections()
+    
+    return 1
+
+@ModuleInfo.plugin("wb.tools.connectionFromClipboard", caption="New Connection from Clipboard", input= [], pluginMenu="Home/Connections")
+@ModuleInfo.export(grt.INT)
+def newConnectionFromClipboard():
+    text = mforms.Utilities.get_clipboard_text()
+    if not text:
+        return 0
+
+    existing = set()
+    # make a normalized set of the connections that already exist
+    for con in grt.root.wb.rdbmsMgmt.storedConns:
+        existing.add(connectionStringFromConnection(con))
+
+    parse_errors = False
+    for line in text.encode("utf8").split("\n"):
+        conn = connectionFromString(line)
+        if not conn and not parse_errors:
+            mforms.Utilities.show_error("New Connection from Clipboard", "Could not parse connection parameters from string '%s'" % line, "OK", "", "")
+            parse_errors = True
+            continue
+        
+        if connectionStringFromConnection(conn) in existing:
+            mforms.Utilities.show_error("New Connection from Clipboard", "The connection %s already exists and was not added." % line,
+                                        "OK", "", "")
+            continue
+        i = 1
+        name = conn.parameterValues.get("hostName", "local")
+        prefix = name
+        while any(conn.name == name for conn in grt.root.wb.rdbmsMgmt.storedConns):
+            name = "%s (%i)" % (prefix, i)
+            i += 1
+        conn.name = name
+        log_info("Added connection %s from clipboard\n" % conn.name)
+        grt.root.wb.rdbmsMgmt.storedConns.append(conn)
+
+    grt.modules.Workbench.refreshHomeConnections()
+
+    return 1
+
+
 @ModuleInfo.plugin("wb.tools.cmdlineClient", caption="Start Command Line Client", input= [wbinputs.selectedConnection()], pluginMenu="Home/Connections")
 @ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection)
 def startCommandLineClientForConnection(conn):
@@ -241,11 +437,14 @@ def startCommandLineClientForConnection(conn):
 
 
 if sys.platform == "linux2":
-    odbcadmin = os.getenv("MWB_BINARIES_DIR")+"/iodbcadm-gtk"
     @ModuleInfo.export(grt.INT)
     def startODBCAdmin():
-        if os.path.exists(odbcadmin):
-            subprocess.call(odbcadmin + " &", shell=True)
+        path = os.getenv('PATH')
+        wb_bindir = os.getenv('MWB_BINARIES_DIR')
+        if ( (wb_bindir and os.path.isfile(os.path.join(wb_bindir, 'iodbcadm-gtk'))) or
+             (path and any( os.path.isfile(os.path.join(prefix, 'iodbcadm-gtk')) for prefix in path.split(':') ))
+           ):
+            subprocess.Popen('iodbcadm-gtk', shell=True, close_fds=True)
             return 1
         else:
             return 0
@@ -259,86 +458,77 @@ elif sys.platform == "darwin":
 elif sys.platform == "win32":
     @ModuleInfo.export(grt.INT)
     def startODBCAdmin():
-        subprocess.call("odbcad32.exe", shell=True)
+        subprocess.Popen("odbcad32.exe", shell=True, creationflags=subprocess.CREATE_NEW_PROCESS_GROUP, close_fds=True)
         return 1
 
 
-# Utilities only work in Py 2.6
-disable_utilities = False
-# find out version of default python interpreter
-if sys.platform != "win32":
-    pyversion, junk = subprocess.Popen("python --version", stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True).communicate()
-    if pyversion < '2.6':
-        disable_utilities = True
+@ModuleInfo.plugin("wb.tools.utilitiesShell", caption="Start Shell for MySQL Utilities", groups=["Others/Menu/Ungrouped"])
+@ModuleInfo.export(grt.INT)
+def startUtilitiesShell():
+    import platform
+    import os
 
-if not disable_utilities:
-    @ModuleInfo.plugin("wb.tools.utilitiesShell", caption="Start Shell for MySQL Utilities", groups=["Others/Menu/Ungrouped"])
-    @ModuleInfo.export(grt.INT)
-    def startUtilitiesShell():
-        import platform
-        import os
+    utilities_url = ("http://dev.mysql.com/downloads/tools/utilities/" if grt.root.wb.info.edition == "Community" else
+                     "https://edelivery.oracle.com/EPD/Search/get_form?product=18251")
 
-        utilities_url = ("http://dev.mysql.com/downloads/tools/utilities/" if grt.root.wb.info.edition == "Community" else
-                         "https://edelivery.oracle.com/EPD/Search/get_form?product=18251")
-
-        if platform.system() == "Windows":
-            guessed_path = None
-            for varname in ["ProgramFiles(x86)", "ProgramFiles"]:
-                if not os.getenv(varname):
-                    continue
-                path = os.path.join(os.getenv(varname), "MySQL", "MySQL Utilities", "mysqluc.exe")
-                if os.path.exists(path):
-                    guessed_path = path
-                    break
-            if any(os.path.exists(os.path.join(f, "mysqluc.exe")) for f in os.getenv("PATH").split(";")):
-                # Utils path is in PATH already
-                command = r'start cmd /K "mysqluc"'
-                subprocess.Popen(command, shell = True)
-            elif guessed_path:
-                command = r'start cmd /K "%s"' % guessed_path
-                subprocess.Popen(command, shell = True)
-            else:
-                if mforms.Utilities.show_message("MySQL Utilities", "The command line MySQL Utilities could not be "
-                                                 "located.\n\nTo use them, you must download and install the utilities "
-                                                 "package for your system from %s" % utilities_url,
-                                           "OK", "Download...", "") != mforms.ResultOk:
-                    mforms.Utilities.open_url(utilities_url)
-        elif platform.system() == "Darwin":
-            path = [f+"/mysqluc" for f in os.getenv("PATH").split(":")+["/usr/local/bin"] if os.path.exists(f+"/mysqluc")]
-            if path:
-                os.system(r"""osascript -e 'tell application "Terminal" to do script "%s -e \"help utilities\""' -e 'tell front window of application "Terminal" to set custom title to "MySQL Utilities"'""" % path[0])
-            else:
-                if mforms.Utilities.show_message("MySQL Utilities", "The command line MySQL Utilities could not be "
-                                                 "located.\n\nTo use them, you must download and install the utilities "
-                                                 "package for your system from %s" % utilities_url,
-                                                 "OK", "Download...", "") != mforms.ResultOk:
-                    mforms.Utilities.open_url(utilities_url)
+    if platform.system() == "Windows":
+        guessed_path = None
+        for varname in ["ProgramFiles(x86)", "ProgramFiles"]:
+            if not os.getenv(varname):
+                continue
+            path = os.path.join(os.getenv(varname), "MySQL", "MySQL Utilities", "mysqluc.exe")
+            if os.path.exists(path):
+                guessed_path = path
+                break
+        if any(os.path.exists(os.path.join(f, "mysqluc.exe")) for f in os.getenv("PATH").split(";")):
+            # Utils path is in PATH already
+            command = r'start cmd /K "mysqluc"'
+            subprocess.Popen(command, shell = True)
+        elif guessed_path:
+            command = r'start cmd /K "%s"' % guessed_path
+            subprocess.Popen(command, shell = True)
         else:
-            if not any(os.path.exists(f+"/mysqluc") for f in os.getenv("PATH").split(":")):
-                if mforms.Utilities.show_message("MySQL Utilities", "The command line MySQL Utilities could not be "
-                                                 "located.\n\nTo use them, you must download and install the utilities "
-                                                 "package for your system from %s" % utilities_url,
-                                               "OK", "Download...", "") != mforms.ResultOk:
-                    mforms.Utilities.open_url(utilities_url)
-            else:
-                term = get_linux_terminal_program()
-                if term:
-                    import tempfile
-                    fd, setup_script = tempfile.mkstemp(prefix="delme.", dir=mforms.App.get().get_user_data_folder())
-                    f = os.fdopen(fd, "w+")
-                    f.write('echo "The following MySQL Utilities are available:"\n')
-                    f.write('mysqluc -e "help utilities"\n')
-                    f.write('rm -f "%s"\n' % setup_script)
-                    f.write('bash -i\n')
-                    f.close()
-                    os.chmod(setup_script, 0700)
+            if mforms.Utilities.show_message("MySQL Utilities", "The command line MySQL Utilities could not be "
+                                             "located.\n\nTo use them, you must download and install the utilities "
+                                             "package for your system from %s" % utilities_url,
+                                       "Download...", "Cancel", "") == mforms.ResultOk:
+                mforms.Utilities.open_url(utilities_url)
+    elif platform.system() == "Darwin":
+        # PATH seems to be stripped down when WB is started from a binary .app
+        if any(os.path.exists(f+"/mysqluc") for f in os.getenv("PATH").split(":") + ["/usr/local/bin"]):
+            os.system(r"""osascript -e 'tell application "Terminal" to do script "mysqluc -e \"help utilities\""' -e 'tell front window of application "Terminal" to set custom title to "MySQL Utilities"'""")
+        else:
+            if mforms.Utilities.show_message("MySQL Utilities", "The command line MySQL Utilities could not be "
+                                             "located.\n\nTo use them, you must download and install the utilities "
+                                             "package for your system from %s" % utilities_url,
+                                             "Download...", "Cancel", "") == mforms.ResultOk:
+                mforms.Utilities.open_url(utilities_url)
+    else:
+        if not any(os.path.exists(f+"/mysqluc") for f in os.getenv("PATH").split(":")):
+            if mforms.Utilities.show_message("MySQL Utilities", "The command line MySQL Utilities could not be "
+                                             "located.\n\nTo use them, you must download and install the utilities "
+                                             "package for your system from %s" % utilities_url,
+                                           "Download...", "Cancel", "") == mforms.ResultOk:
+                mforms.Utilities.open_url(utilities_url)
+        else:
+            term = get_linux_terminal_program()
+            if term:
+                import tempfile
+                fd, setup_script = tempfile.mkstemp(prefix="delme.", dir=mforms.App.get().get_user_data_folder())
+                f = os.fdopen(fd, "w+")
+                f.write('echo "The following MySQL Utilities are available:"\n')
+                f.write('mysqluc -e "help utilities"\n')
+                f.write('rm -f "%s"\n' % setup_script)
+                f.write('bash -i\n')
+                f.close()
+                os.chmod(setup_script, 0700)
 
-                    if 'konsole' in term:
-                        subprocess.call([term, "-e", "/bin/bash", setup_script])
-                    else:
-                        subprocess.call(["/bin/bash", "-c", "%s -e %s &" % (term, setup_script)])
+                if 'konsole' in term:
+                    subprocess.call([term, "-e", "/bin/bash", setup_script])
                 else:
-                    raise RuntimeError("Terminal program could not be found")
+                    subprocess.call(["/bin/bash", "-c", "%s -e %s &" % (term, setup_script)])
+            else:
+                raise RuntimeError("Terminal program could not be found")
 
 
 class CheckForUpdateThread(threading.Thread):
@@ -372,15 +562,15 @@ class CheckForUpdateThread(threading.Thread):
             try:
                 current_version = (grt.root.wb.info.version.majorNumber, grt.root.wb.info.version.minorNumber, grt.root.wb.info.version.releaseNumber)
                 edition = '' if grt.root.wb.info.license == 'GPL' else '-commercial'
-                
+                filename=u'mysql-workbench' + (edition or '-community')
                 packages = ( package for package in self.dom.getElementsByTagName('Package') if package.parentNode.parentNode.attributes['name'].nodeValue == u'workbench-win32' + edition)
-                version_strings = ( node.attributes['thisVersion'].nodeValue for node in packages if node.attributes['type'].nodeValue == 'MSI' )
+                version_strings = (node.attributes['thisVersion'].nodeValue for node in packages if node.attributes['filename'].nodeValue.startswith(filename) and node.attributes['filename'].nodeValue.endswith(u'msi'))
                 versions = tuple( tuple( int(num) for num in version_string.split('.') ) for version_string in version_strings )
-                newest_version = max( versions ) if versions else current_version
-                
+                version_list = [v for v in versions]
+                newest_version = max( version_list ) if version_list else current_version
                 if newest_version > current_version:
                     if mforms.Utilities.show_message('New Version Available', 'The new MySQL Workbench %s has been released.\nYou can download the latest version from\nhttp://www.mysql.com/downloads/workbench.' % '.'.join( [str(num) for num in newest_version] ),
-                                                  'OK', 'Cancel', 'Get it Now') == mforms.ResultOther:
+                                                  'Get it Now', 'Maybe Later', "") == mforms.ResultOk:
                         mforms.Utilities.open_url('http://www.mysql.com/downloads/workbench')
                 else:
                     mforms.Utilities.show_message('MySQL Workbench is Up to Date', 'You are already using the latest version of MySQL Workbench.', 'OK', '', '')
