@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -380,6 +380,8 @@ public:
   MySQLTriggerPanel(MySQLTableEditorBE *editor)
   : mforms::Box(true), _editor(editor), _list(mforms::TreeFlatList|mforms::TreeSizeSmall), _button(mforms::SmallButton), _refreshing(false)
   {
+    _edited_trigger_index = 0;
+
     set_spacing(8);
     set_padding(4);
     
@@ -397,9 +399,13 @@ public:
     _rtable.set_column_spacing(6);
     
     _name.set_name("trigger name");
-    _name.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::name_changed, this));
+    _name.set_read_only(true);
+    //TODO: Check if we can make this code usable again
+//    _name.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::name_changed, this));
     _definer.set_name("trigger definer");
-    _definer.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::definer_changed, this));
+    _definer.set_read_only(true);
+    //TODO: Check if we can make this code usable again
+//    _definer.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::definer_changed, this));
 
     _namel.set_style(mforms::SmallStyle);
     _definerl.set_style(mforms::SmallStyle);
@@ -458,34 +464,30 @@ public:
     _list.clear();
 
     grt::ListRef<db_Trigger> triggers(_editor->get_table()->triggers());
-    std::vector<db_TriggerRef> trigvec;
+    std::map<std::string, db_TriggerRef> trigmap;
     GRTLIST_FOREACH(db_Trigger, triggers, trig)
-    {
-      trigvec.push_back(*trig);
-    }
-//    std::sort(trigvec.begin(), trigvec.end(), &MySQLTriggerPanel::compare_order);
-    mforms::TreeNodeRef node;
-    for (std::vector<db_TriggerRef>::const_iterator trig = trigvec.begin(); trig != trigvec.end(); ++trig)
     {
       std::string t = (*trig)->timing();
       t.append(" ").append((*trig)->event());
-      leftover.erase(t);
-      node = _list.add_node();
-      node->set_string(0, (*trig)->name());
-      node->set_string(1, t);
-      
-      if (_selected_trigger == *trig)
-        _list.select_node(node);
+      trigmap.insert(std::make_pair(t, *trig));
     }
-    
+//    std::sort(trigvec.begin(), trigvec.end(), &MySQLTriggerPanel::compare_order);
+
+    mforms::TreeNodeRef node;
+    std::map<std::string, db_TriggerRef>::iterator it;
     for (std::set<std::string>::const_iterator t = leftover.begin(); t != leftover.end(); ++t)
     {
       node = _list.add_node();
-      node->set_string(0, "-");
+      if ((it = trigmap.find(*t)) != trigmap.end())
+        node->set_string(0, it->second->name());
+      else
+        node->set_string(0, "-");
+
       node->set_string(1, *t);
     }
 
     _list.select_node(_list.node_at_row(old_selected));
+
     _refreshing = false;
   }
   
@@ -497,43 +499,47 @@ public:
       if (_code_editor->is_dirty() && _selected_trigger->sqlDefinition() != _code_editor->get_string_value())
       {
         AutoUndoEdit undo(_editor, _selected_trigger, "sql");
+        _editor->freeze_refresh_on_object_change();
         grt::IntegerRef res = _editor->_sql_parser->parse_trigger(_selected_trigger, _code_editor->get_string_value().c_str());
+        _editor->thaw_refresh_on_object_change(true);
         _name.set_value(_selected_trigger->name());
         _definer.set_value(_selected_trigger->definer());
+        mforms::TreeNodeRef node = _list.node_at_row(_edited_trigger_index);
+        if (node)
+        {
+          node->set_string(0, _selected_trigger->name());
+        }
         undo.end(strfmt(_("Edit trigger `%s` of `%s`.`%s`"), _selected_trigger->name().c_str(), _editor->get_schema_name().c_str(), _editor->get_name().c_str()));
       }
     }
     else
-    {
-      _editor->commit_changes();
       refresh();
-    }
-  }
-  
-  void name_changed()
-  {
-    if (_selected_trigger.is_valid())
-    {
-      AutoUndoEdit undo(_editor, _selected_trigger, "name");
-      _selected_trigger->name(_name.get_string_value());
-      undo.end(base::strfmt("Rename trigger to %s", _name.get_string_value().c_str()));
-      mforms::TreeNodeRef node = _list.get_selected_node();
-      if (node)
-        node->set_string(0, _selected_trigger->name().c_str());
-      selection_changed();
-    }
-  }
-
-  void definer_changed()
-  {
-    if (_selected_trigger.is_valid())
-    {
-      AutoUndoEdit undo(_editor, _selected_trigger, "definer");
-      _selected_trigger->definer(_definer.get_string_value());
-      undo.end(base::strfmt("Change trigger definer to %s", _definer.get_string_value().c_str()));
-      selection_changed();
-    }
   }
+  //TODO: Check if we can make this code usable again
+//  void name_changed()
+//  {
+//    if (_selected_trigger.is_valid())
+//    {
+//      AutoUndoEdit undo(_editor, _selected_trigger, "name");
+//      _selected_trigger->name(_name.get_string_value());
+//      undo.end(base::strfmt("Rename trigger to %s", _name.get_string_value().c_str()));
+//      mforms::TreeNodeRef node = _list.get_selected_node();
+//      if (node)
+//        node->set_string(0, _name.get_string_value());
+//      selection_changed();
+//    }
+//  }
+  //TODO: Check if we can make this code usable again
+//  void definer_changed()
+//  {
+//    if (_selected_trigger.is_valid())
+//    {
+//      AutoUndoEdit undo(_editor, _selected_trigger, "definer");
+//      _selected_trigger->definer(_definer.get_string_value());
+//      undo.end(base::strfmt("Change trigger definer to %s", _definer.get_string_value().c_str()));
+//      selection_changed();
+//    }
+//  }
 
   void clicked()
   {
@@ -617,6 +623,7 @@ public:
       }
     }
 
+    bool empty_trigger = false;
     if (_selected_trigger != trigger)
     {
       _selected_trigger = trigger;
@@ -629,33 +636,46 @@ public:
         _name.set_enabled(true);
         _definer.set_enabled(true);
       
-        sql.append(base::strfmt("USE `%s`;\nDELIMITER $$\n\n", _editor->get_schema_name().c_str()));
+        sql.append(base::strfmt("USE `%s`;\nDELIMITER $$\n", _editor->get_schema_name().c_str()));
         if (trigger->sqlDefinition().empty())
         {
           sql.append(base::strfmt("CREATE TRIGGER `%s` %s %s ON `%s` FOR EACH ROW\n", trigger->name().c_str(),
             timing.c_str(), event.c_str(), _editor->get_name().c_str()));
-          sql.append("-- Edit trigger body code below this line. Do not edit lines above this one\n");
+          
+          // No longer necessary. We can now parse the entire sql (if the user types complete nonsense
+          // we are all lost however, but that's no news).
+          //sql.append("-- Edit trigger body code below this line. Do not edit lines above this one\n");
         }
         else
           sql.append(trigger->sqlDefinition());
+
+        _edited_trigger_index = _list.row_for_node(_list.get_selected_node());
+        _code_editor->set_text_keeping_state(sql.c_str());
       }
       else
       {
-        _name.set_value("");
-        _definer.set_value("");
-        _name.set_enabled(false);
-        _definer.set_enabled(false);
-        _button.set_text("Add Trigger");
-        editor_enabled = false;
-
-        sql = "-- Trigger not defined. Click Add Trigger to create it.\n";
+        empty_trigger = true;
       }
-
-      _button.set_enabled(true);
-      _code_editor->set_text_keeping_state(sql.c_str());
-      _code_editor->reset_dirty();
-      _code_editor->set_enabled(editor_enabled);
     }
+
+    if (empty_trigger || !_selected_trigger.is_valid())
+    {
+      _name.set_value("");
+      _definer.set_value("");
+      _name.set_enabled(false);
+      _name.set_read_only(true);
+      _definer.set_enabled(false);
+      _definer.set_read_only(true);
+      _button.set_text("Add Trigger");
+      editor_enabled = false;
+
+      _edited_trigger_index = _list.row_for_node(_list.get_selected_node());
+      sql = "-- Trigger not defined. Click Add Trigger to create it.\n";
+      _code_editor->set_text_keeping_state(sql.c_str());
+    }
+    _button.set_enabled(true);
+    _code_editor->reset_dirty();
+    _code_editor->set_enabled(editor_enabled);
   }
   
 private:
@@ -669,6 +689,7 @@ private:
   mforms::Button _button;
   mforms::CodeEditor *_code_editor;
   db_TriggerRef _selected_trigger;
+  int _edited_trigger_index;
   bool _refreshing;
 };
 
@@ -925,7 +946,7 @@ std::string MySQLTableEditorBE::get_table_option_by_name(const std::string& name
   return std::string("");
 }
 
-
+//TODO: Check if we still need this, if not ... remove.
 std::string MySQLTableEditorBE::get_all_triggers_sql() const
 {
   std::string retval;
@@ -992,16 +1013,16 @@ bool MySQLTableEditorBE::can_close()
 
 //--------------------------------------------------------------------------------------------------
 
+//TODO: Do we still need this?
 void MySQLTableEditorBE::commit_changes()
 {
-  mforms::CodeEditor* editor = get_sql_editor()->get_editor_control();
-  if (editor->is_dirty())
-  {
-    // TODO: no longer needed since we don't set all triggers at once anymore.
-    set_triggers_sql(get_all_triggers_sql(), true); //we need to always get all editors code not just the currently visible
-    // Note: don't reset the dirty state of the editor to be able to undo beyond this commit.
-  }
-
+//  mforms::CodeEditor* editor = get_sql_editor()->get_editor_control();
+//  if (editor->is_dirty())
+//  {
+//    // TODO: no longer needed since we don't set all triggers at once anymore.
+//    set_triggers_sql(get_all_triggers_sql(), true); //we need to always get all editors code not just the currently visible
+//    // Note: don't reset the dirty state of the editor to be able to undo beyond this commit.
+//  }
   // TODO: anything else to store?
 }
 
