@@ -328,7 +328,7 @@ public:
     _heading.set_wrap_text(true);
     _heading.set_style(mforms::WizardHeadingStyle);
     _heading.set_text(_("This script will now be executed on the DB server to create your databases.\nYou may make changes before executing."));
-    add(&_heading, false, false);
+    add(&_heading, false, true);
   }
 
   virtual void enter(bool advancing)
