@@ -30,53 +30,6 @@ from wb_server_management import wbaOS
 from wb_common import OperationCancelledError, InvalidPasswordError, Users
 
 import re
-import socket
-
-#-------------------------------------------------------------------------------
-def get_local_ip_list():
-    cmd    = "/bin/sh -c ifconfig"
-    regexp = "inet addr:([0-9a-f:\.]+)"
-
-    if hasattr(sys, 'getwindowsversion'):
-        cmd    = 'ipconfig'
-        regexp = "IPv4.*: +([0-9a-f:\.]+)"
-    elif 'darwin' in sys.platform:
-        regexp = "inet ([0-9a-f\.:]+) netmask"
-
-    out = None
-    try:
-        out = os.popen(cmd)
-    except Exception, e:
-        # We do not care if execution failed, the list will just be empty
-        #print e
-        out = None
-    except:
-        print "Unknown exception while running '%s'"%cmd
-
-    try:
-        ret = []
-        if out:
-            result = out.read()
-            m = re.findall(regexp, result)
-
-            for ip in m:
-                try:
-                    t_ip = socket.gethostbyname(ip)
-                    ret.append(ip)
-                except socket.gaierror,e:
-                    print ip, str(e)
-    except Exception, e:
-        # We do not care if execution failed, the list will just be empty
-        #print e
-        ret = []
-    except:
-        print "Unknown exception while running '%s'"%cmd
-        ret = []
-
-    return ret
-
-local_ip_list = get_local_ip_list()
-
 
 #===============================================================================
 
@@ -104,7 +57,7 @@ class ServerProfile(object):
             return default_value
         try:
             value = self.__settings.serverInfo[name]
-        except KeyError, exc:
+        except KeyError:
             value = default_value
         if type(value) is not int:
             try:
@@ -119,7 +72,7 @@ class ServerProfile(object):
         # always returns an int, a None value will be replaced with default_value
         try:
             value = self.__settings.loginInfo[name]
-        except KeyError, exc:
+        except KeyError:
             value = default_value
         if type(value) is not int:
             try:
@@ -136,7 +89,7 @@ class ServerProfile(object):
             return default_value
         try:
             value= self.__settings.serverInfo[name]
-        except KeyError, exc:
+        except KeyError:
             value = default_value
         if value is None:
             value= default_value
@@ -148,7 +101,7 @@ class ServerProfile(object):
         # always returns an str or unicode, a None value will be replaced with default_value
         try:
             value= self.__settings.loginInfo[name]
-        except KeyError, exc:
+        except KeyError:
             value = default_value
         if value is None:
             value= default_value
@@ -218,7 +171,7 @@ class ServerProfile(object):
 
     @property
     def is_local(self):
-        local_addrs = ["localhost", "", "0", "127.0.0.1"] + local_ip_list
+        local_addrs = ["localhost", "", "0", "127.0.0.1", "::1"]
         #if self.uses_wmi:
         #    return self.wmi_hostname in local_addrs
         #elif self.uses_ssh:
@@ -269,6 +222,11 @@ Please edit your connection settings and try again."""
                 host_name = self.db_connection_params.parameterValues["hostName"]
         return host_name
 
+    @property
+    def mysql_username(self):
+        if self.db_connection_params:
+            return self.db_connection_params.parameterValues["userName"]
+        return ""
 
     @property
     def uses_ssh(self):
@@ -329,19 +287,10 @@ Please edit your connection settings and try again."""
             ret = self._str_server_info("sys.mysqld.stop")
         return ret
 
-    @property
-    def check_server_status_cmd(self):
-        return self._str_server_info("sys.mysqld.status")
-
     @property
     def use_sudo(self):
         return self._int_server_info("sys.usesudo")
 
-    @property
-    def use_sudo_for_status(self):
-        return self._int_server_info("sys.usesudostatus")
-
-
     @property
     def config_file_path(self):
         """ path to configuration file (eg my.cnf)
@@ -369,7 +318,6 @@ Please edit your connection settings and try again."""
     def server_version_tuple(self):
         if not self.server_version:
             return ()
-        import re
         version_tokens = self.server_version.split('.')
         ver = []
         for token in version_tokens:
@@ -473,10 +421,12 @@ Please edit your connection settings and try again."""
     def detected_os_version(self):
         return self.__settings.serverInfo['detected_os_version']
 
+    @property
+    def path_module(self):
+        return ntpath if self.target_os == wbaOS.windows else posixpath
 
     def join_target_path(self, path, *comps):
-        pathmodule = ntpath if self.target_os == wbaOS.windows else posixpath
-        return pathmodule.join(path, *comps)
+        return self.path_module.join(path, *comps)
 
     # OS specific things
     @property
@@ -506,14 +456,6 @@ class PasswordHandler(object):
                     password_type = "sudo"
                 else:
                     password_type = None
-        elif service_type == "service.status":
-            if profile.target_is_windows:
-                password_type = None
-            else:
-                if profile.use_sudo_for_status:
-                    password_type = "sudo"
-                else:
-                    password_type = None
         elif service_type == "remoteshell":
             if profile.uses_ssh:
                 if profile.ssh_usekey:
@@ -529,7 +471,7 @@ class PasswordHandler(object):
         return password_type
 
     def get_password_parameters(self, service_type):
-        # known service types: remoteshell, ssh, sshkey, file, service.startstop, service.status
+        # known service types: remoteshell, ssh, sshkey, file, service.startstop
         # these are mapped to
         # known password types: ssh, sshkey, sudo, UAC, wmi
         # which are then mapped to (title, service, account) or "UAC" or None
@@ -665,8 +607,8 @@ class ServerControlBase(object):
             traceback.print_exc()
             finish_callback(err)
 
-    def get_status(self, verbose=0): # overriden by concrete subclasses
-        return 0
+    def get_status(self, verbose=0): # overriden by concrete subclasses, if available
+        return None
 
 
 class ServerControlShell(ServerControlBase):
@@ -674,25 +616,6 @@ class ServerControlShell(ServerControlBase):
         """ Can also throw OperationCancelledError"""
         ServerControlBase.__init__(self, profile, helper, password_delegate)
 
-        # Ask for password here, as this class is instantiated from the main thread
-        # The other methods may be instantiated from other threads, which will not work
-        # with the password dialogs
-
-        # Ensure sudo password for status checks is cached. For other sudo uses,
-        # the password should be requested when the action is started from the UI (ie, in the code called
-        # by the Start/Stop button in Server startup tab)
-        if self.profile.use_sudo_for_status:
-            password_delegate.get_password_for("service.status")
-
-        self.prepare_filter = None
-        self.apply_filter = None
-
-
-    def set_filter_functions(self, prepare_f, apply_f):
-        self.prepare_filter = prepare_f
-        self.apply_filter = apply_f
-
-
     def get_password(self):
         return self.password_delegate.get_password_for("service.startstop")
 
@@ -703,7 +626,7 @@ class ServerControlShell(ServerControlBase):
         if self.profile.use_sudo:
             try:
                 password = self.get_password()
-            except OperationCancelledError, exc:
+            except OperationCancelledError:
                 return False
         else:
             password = None
@@ -718,7 +641,7 @@ class ServerControlShell(ServerControlBase):
         if self.profile.use_sudo:
             try:
                 password = self.get_password()
-            except OperationCancelledError, exc:
+            except OperationCancelledError:
                 return False
         else:
             password = None
@@ -730,16 +653,16 @@ class ServerControlShell(ServerControlBase):
         """ Can also throw InvalidPasswordError"""
         self.info("Starting server...") # TODO: This does not come to info output
         try:
-            self.info("Executting '%s'" % self.profile.start_server_cmd)
+            self.info("Executing '%s'" % self.profile.start_server_cmd)
             as_user = Users.ADMIN if self.profile.use_sudo else Users.CURRENT
             r = self.helper.execute_command(self.profile.start_server_cmd,
                                     as_user=as_user,
                                     user_password=password,
                                     output_handler=lambda s:self.info("Start server: %s"%(s if ((type(s) is str) or (type(s) is unicode)) else "").replace(password or "", "")))
-        except InvalidPasswordError, exc:
+        except InvalidPasswordError:
             # forget password, so that next time it will be requested
             self.password_delegate.reset_password_for("service.startstop")
-            raise exc
+            raise
 
         if r:
             return True
@@ -764,86 +687,16 @@ class ServerControlShell(ServerControlBase):
                                     as_user=as_user,
                                     user_password=password,
                                     output_handler=strip_pwd)
-        except InvalidPasswordError, exc:
+        except InvalidPasswordError:
             # forget password, so that next time it will be requested
             self.password_delegate.reset_password_for("service.startstop")
-            raise exc
+            raise
 
         if r:
             return True
         return False
 
 
-    def get_status(self, verbose=1, password=None):
-        """ Can also throw InvalidPasswordError"""
-        if verbose > 1:
-            output = lambda s:self.info("Check server: %s"%s)
-        else:
-            output = None
-
-        password = None
-        #unused user_gave_new_password = False
-
-        script = self.profile.check_server_status_cmd
-
-        if verbose > 1:
-            self.info("Checking server status...")
-            if self.profile.use_sudo_for_status:
-                self.info("Executing: %s (using sudo)" % script)
-                if not password:
-                    self.info("Note: no sudo password supplied")
-            else:
-                self.info("Executing: %s" % script)
-
-        if self.prepare_filter:
-            raw_script, filters = self.prepare_filter(script)
-        else:
-            raw_script = script
-
-        lines = []
-        def collect_output(line, l=lines, dump=output):
-            l.append(line)
-            if dump:
-                dump(line)
-        try:
-            as_user = Users.ADMIN if self.profile.use_sudo_for_status else Users.CURRENT
-            result = self.helper.execute_command(raw_script,
-                                    as_user=as_user,
-                                    user_password=password,
-                                    output_handler=collect_output)
-        except InvalidPasswordError, exc:
-            self.info("Invalid password")
-            raise exc
-
-        if self.apply_filter and filters:
-            filters_text, filters_code = self.apply_filter("\n".join(lines), filters)
-
-            if filters_code is not None:
-                if result is not None:
-                    result = int(result) or filters_code
-                else:
-                    result = filters_code
-        else:
-            #output_text = "\n".join(lines)
-            pass
-
-
-        if verbose > 1:
-            self.info("Server check returned %s" % result)
-
-        status = "unknown"
-        if result == 0:
-            if verbose:
-                self.info("Checked server status: Server is running.")
-            status = "running"
-        elif result == 1:
-            if verbose:
-                self.info("Checked server status: Server is stopped.")
-            status = "stopped"
-
-        return status
-
-
 class ServerControlWMI(ServerControlBase):
     def __init__(self, profile, helper, password_delegate):
         ServerControlBase.__init__(self, profile, helper, password_delegate)
@@ -893,15 +746,6 @@ class ServerControlWMI(ServerControlBase):
             serverInfo["sys.mysqld.stop"] = "sc stop " + service
             print "WMIShell: stop command set to '%s'"%profile.stop_server_cmd
 
-        if profile.check_server_status_cmd == "" or (service not in profile.check_server_status_cmd):
-            serverInfo["sys.mysqld.status"] = "sc query %s | wba_filter(RUNNING)"%service
-            print "WMIShell: status command set to '%s'"%profile.check_server_status_cmd
-
-    def set_filter_functions(self, prepare_f, apply_f):
-        if self.profile.is_local and self.shell:
-            print "WMIShell: Set filters for local windows cli"
-            self.shell.prepare_filter = prepare_f
-            self.shell.apply_filter = apply_f
 
     def get_password(self):
         if self.profile.is_local:
@@ -938,7 +782,7 @@ class ServerControlWMI(ServerControlBase):
         try:
             # ask subclass to provide password
             password = self.get_password()
-        except OperationCancelledError, exc:
+        except OperationCancelledError:
             return False
         thread = threading.Thread(target=self.worker_thread, args=(self.start, password, finish_callback))
         thread.run()
@@ -947,7 +791,7 @@ class ServerControlWMI(ServerControlBase):
     def stop_async(self, finish_callback):
         try:
             password = self.get_password()
-        except OperationCancelledError, exc:
+        except OperationCancelledError:
             return False
         thread = threading.Thread(target=self.worker_thread, args=(self.stop, password, finish_callback))
         thread.run()
@@ -982,7 +826,8 @@ class ServerControlWMI(ServerControlBase):
                 raise RuntimeError("Error stopping service %s through WMI\n%s" % (service, result))
             return not result.startswith("error")
 
-    def get_status(self, verbose=1, password=None):
+
+    def get_status(self, verbose=1):
         "Returned value is one of running, stopping, starting, stopped, unknown"
         service = self.profile.wmi_service_name
         action = "status"
